unit BCMA_Main;
{
================================================================================
*	File:  BCMA_Main.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 131 $  $Modtime: 5/06/02 10:34a $
*
*	Description:  This is the Main Form for the application.  It displays the
*               Virtual Due List and processes all scanner input for scanned
*               drugs.
*
*  9/29/2008 IMPORTANT NOTICE CONCERNING COMPILER DIRECTIVES:
*    1. add MSF_ON and leave it set to on unless you go through the code
*       remove the directives and switched code.  This was added when
*       we were developing MSF Lite which never went out.
*
*    2. The test mode define symbol TEST_ON allows the user to use the hot key
*       CTRL-SHIFT-= to get the IEN popup using DisplayIEN.
*       Set it to TEST_OFF in production.  Remove TEST_ON from the compiler conditionals.
*  3/9/2009
*    3. The debug log mode define symbol DEBUGLOG_ON allows the test user to use
*       switch /DEBUGLOG_ON on the command line to log the input and
*       return values of RPC calls in bcma.log.
*  6/23/2010
*    4. Include setIHS override switch parameter.
*       When true, force agency code to I for IHS.
*  10/19/2010
*    5. Added ShowActiveCtrlName in test mode to allow identification of
*       active control during run-time.
================================================================================
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, StdCtrls, ComCtrls, Grids, Buttons,
  BCMA_Objects, IniFiles, ToolWin, BCMA_Util,
  BCMA_IV, ImgList, ComObj, BcmaOrderCom_TLB, XPStyleActnCtrls, ActnList,
  ActnMan, ActnColorMaps, ActnCtrls, ActnMenus, StdActns,
  uCCOW, ehshelprouter, ehsbase, ehswhatsthis,
  StdStyleActnCtrls, oInterfaces, VA508AccessibilityManager,
  VA508AccessibilityRouter, VAUtils, ehscontextmap, AppEvnts;

const

  MAXFORMHEIGHT = 1200;
  MAXFORMWIDTH = 1600;
  ErrIVAction = 'This bag was not found in any of the orders currently' + #13 +
    'displayed on the VDL.  This could indicate a problem with the data.  An action ' + #13
    +
    'on this bag can not be taken at this time.';
  ErrRowTooTall = 'Too much information to display, ' +
    'view the order details to see all information.';
type
  TfrmMain = class(TForm)
    (*
      The Main Form for the application.  It displays the Virtual Due List and
      processes all scanner input for scanned drugs.
    *)
    edtScannerInput: TEdit;
    pnlMainForm: TPanel;
    pnlBottomForm: TPanel;
    pnlScannerStatus: TPanel;
    pnlSpecialFunctions: TPanel;
    pnlScannerInput: TPanel;
    lblScannerStatus: TLabel;
    lblScanMedication: TLabel;
    pnlScannerIndicator: TPanel;
    pnlBCMA: TPanel;
    gbBCMA: TGroupBox;
    lblBlack: TLabel;
    lblSilver: TLabel;
    pnlMainHeading: TPanel;
    pnlPatientDemographics: TPanel;
    pnlDueListParameters: TPanel;
    StatusBar: TStatusBar;
    pnlVirtualDueList: TPanel;
    GroupBox1: TGroupBox;
    lblStartTime: TLabel;
    lblStopTime: TLabel;
    cmbxStopTime: TComboBox;
    GroupBox2: TGroupBox;
    cbxContinuous: TCheckBox;
    cbxPRN: TCheckBox;
    cbxOnCall: TCheckBox;
    cbxOneTime: TCheckBox;
    rePatientDemographics: TRichEdit;
    StatusBarTimer: TTimer;
    cmbxStartTime: TComboBox;
    pgctrlVirtualDueList: TPageControl;
    tbshtUnitDose: TTabSheet;
    hdrUnitDose: THeaderControl;
    tbshtIVPIVPB: TTabSheet;
    lblVDLUnitDose: TLabel;
    tbshtIV: TTabSheet;
    ImageList1: TImageList;
    pnlIVTab: TPanel;
    spltIV: TSplitter;
    pnlIV: TPanel;
    lblVDLIV: TLabel;
    fraIV1: TfraIV;
    pnlAllergies: TPanel;
    mnuTakeActionOnWS: TMenuItem;
    ActionManager: TActionManager;
    actionFileOpenPatient: TAction;
    actionFileClosePatient: TAction;
    actionFileExit: TAction;
    actionViewAllergies: TAction;
    actionViewPatientDemographics: TAction;
    actionMedTabUD: TAction;
    actionMedTabPB: TAction;
    actionMedTabIV: TAction;
    actionReportsDueList: TAction;
    actionReportsMedLog: TAction;
    actionReportsMAH: TAction;
    actionReportsMissedMeds: TAction;
    actionReportsAdminTimes: TAction;
    actionDueListAddComment: TAction;
    actionDueListDisplayOrder: TAction;
    actionDueListMedHistory: TAction;
    actionDueListMissingDose: TAction;
    actionDueListPRNEffect: TAction;
    actionDueListUnableToScan: TAction;
    actionDueListTakeActionOnWS: TAction;
    actionMarkHeld: TAction;
    actionMarkUndo: TAction;
    actionMarkRefused: TAction;
    actionMarkRemoved: TAction;
    actionSortByStatus: TAction;
    actionSortByNurse: TAction;
    actionSortByHSM: TAction;
    actionSortByType: TAction;
    actionSortByActiveMed: TAction;
    actionSortByMedSol: TAction;
    actionSortByDosage: TAction;
    actionSortByInfusionRate: TAction;
    actionSortByRoute: TAction;
    actionSortByAdminTime: TAction;
    actionSortByLastAction: TAction;
    actionHelpContentIndex: TAction;
    actionHelpIndex: TAction;
    actionHelpAboutBCMA: TAction;
    actionMOB: TAction;
    actionTools: TAction;
    actionDueListRefresh: TAction;
    actionReportsPRNEffectivenessList: TAction;
    CoolBar1: TCoolBar;
    ActionToolBar: TActionToolBar;
    shpContinuous: TShape;
    shpOneTime: TShape;
    shpOnCall: TShape;
    shpPRN: TShape;
    lblBCMAClinicalReminder: TLabel;
    lvwReminders: TListView;
    actionReportsVariance: TAction;
    actionReportVitalsCumulative: TAction;
    ActionSortByBagInformation: TAction;
    ActionFileBreakPatientContext: TAction;
    pnlCCOW: TPanel;
    imgCCOWStatus: TImage;
    ActionFileEditMedLog: TAction;
    WhatsThis1: TWhatsThis;
    HelpRouter1: THelpRouter;
    ActionFileOpenReadOnly: TAction;
    lblReadOnly: TLabel;
    actionFlag: TAction;
    ActionToolBar1: TActionToolBar;
    ColorMapFlag: TXPColorMap;
    ActionJoinSetNewContext: TAction;
    ActionJoinUseExistingContext: TAction;
    tbshtCoverSheet: TTabSheet;
    lblCoverSheetLoad: TLabel;
    actionMedTabCoverSheet: TAction;
    ActionCSExpandGrp0: TAction;
    ActionCSExpandGrp1: TAction;
    ActionCSExpandGrp2: TAction;
    ImageList2: TImageList;
    actionReportUnknownActions: TAction;
    mnuPopUpUnableToScan: TMenuItem;
    ActionReportUnableToScanDetailed: TAction;
    ActionReportUnableToScanSummary: TAction;
    actionFileOpenLimitedAccess: TAction;
    actionFileCreateWardStock: TAction;
    PopupMenu: TPopupMenu;
    AddComment1: TMenuItem;
    DisplayOrder1: TMenuItem;
    UnabletoScan1: TMenuItem;
    TakeActionOnWS1: TMenuItem;
    mnuN1: TMenuItem;
    mnuPopUpMark: TMenuItem;
    Held2: TMenuItem;
    Undo1: TMenuItem;
    Refused2: TMenuItem;
    Removed2: TMenuItem;
    mnuN2: TMenuItem;
    MedHistory1: TMenuItem;
    MissingDose1: TMenuItem;
    PRNEffectiveness1: TMenuItem;
    mnuMainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuView: TMenuItem;
    mnuReports: TMenuItem;
    mnuDueList: TMenuItem;
    mnuHelp: TMenuItem;
    mnuFileOpenPatientRecord: TMenuItem;
    mnuFileOpenLimitedAccess: TMenuItem;
    mnuFileOpenReadOnly: TMenuItem;
    mnuFileClosePatientRecord: TMenuItem;
    mnuFileN1: TMenuItem;
    mnufileEditMedLog: TMenuItem;
    mnuFileN2: TMenuItem;
    mnuFileRejoinPatientLink: TMenuItem;
    mnuFileBreakPatientLink: TMenuItem;
    mnuFileN3: TMenuItem;
    mnuFilieExit: TMenuItem;
    mnuViewMedTab: TMenuItem;
    SetNewContext1: TMenuItem;
    UseExistingContext1: TMenuItem;
    mnuMedTabCoversheet: TMenuItem;
    mnuMedTabUnitDose: TMenuItem;
    mnuMedTabIVPIVPB: TMenuItem;
    mnuMedTabIV: TMenuItem;
    mnuViewN1: TMenuItem;
    mnuViewAllergies: TMenuItem;
    mnuViewPatientDemographics: TMenuItem;
    mnuViewFlag: TMenuItem;
    mnuReportsDueList: TMenuItem;
    mnuReportsMedicationLog: TMenuItem;
    mnuReportsMedicationAdminHistory: TMenuItem;
    mnuReportsMissedMedications: TMenuItem;
    mnuReportsPRNEffectivenessList: TMenuItem;
    mnuReportsAdministrationTimes: TMenuItem;
    mnuReportsMedicationVarianceLog: TMenuItem;
    mnuReportsUnknownActions: TMenuItem;
    mnuReportsVitalsCumulative: TMenuItem;
    mnuReportsUnabletoScanDetailed: TMenuItem;
    mnuReportsUnabletoScanSummary: TMenuItem;
    mnuDueListAddComment: TMenuItem;
    mnuDueListDisplayOrder: TMenuItem;
    mnuDueListN1: TMenuItem;
    mnuDueListMark: TMenuItem;
    mnuMarkHeld: TMenuItem;
    mnuMarkUndo: TMenuItem;
    mnuMarkRefused: TMenuItem;
    mnuMarkRemoved: TMenuItem;
    mnuDueListN2: TMenuItem;
    mnuDueListMedHistory: TMenuItem;
    mnuDueListMissingDose: TMenuItem;
    mnuDueListPRNEffectiveness: TMenuItem;
    mnuDueListUnabletoScan: TMenuItem;
    mnuDueListUnabletoScanCreateWS: TMenuItem;
    mnuDueLiTtakeActiononWS: TMenuItem;
    mnuDueListN3: TMenuItem;
    mnuDueListSortBy: TMenuItem;
    mnuSortByStatus: TMenuItem;
    mnuSortByVerifyingNurse: TMenuItem;
    mnuSortByHospitalSelfMed: TMenuItem;
    mnuSortByType: TMenuItem;
    mnuSortByActiveMedication: TMenuItem;
    mnuSortByMedicationSolution: TMenuItem;
    mnuSortByDosage: TMenuItem;
    mnuSortByInfusionRate: TMenuItem;
    mnuSortByRoute: TMenuItem;
    mnuSortByAdministrationTime: TMenuItem;
    mnuSortByLastAction: TMenuItem;
    mnuSortByBagInformation: TMenuItem;
    mnuTools: TMenuItem;
    mnuHelpContentsandIndex: TMenuItem;
    mnuHelpIndex: TMenuItem;
    mnuHelpN1: TMenuItem;
    mnuHelpAboutBCMA: TMenuItem;
    mnuDueListN4: TMenuItem;
    mnuDueListRefresh: TMenuItem;
    actionDueListDrugIEN: TAction;
    mnuDueListDrugIENCode: TMenuItem;
    PopUpDrugIENCode: TMenuItem;
    actionDueListAvailableBags: TAction;
    AvailableBags1: TMenuItem;
    mnuDueListAvailableBags: TMenuItem;
    ActionReportsMedicationOverview: TAction;
    mnuReportsMedicationOverview: TMenuItem;
    ActionReportsPRNOverview: TAction;
    mnuReportsPRNOverview: TMenuItem;
    actionReportsIVOverview: TAction;
    mnuReportsIVOverview: TMenuItem;
    actionReportsExpiredOrders: TAction;
    mnuReportsExpiredReport: TMenuItem;
    actionReportsIVBagStatus: TAction;
    mnuReportsIVBagStatus: TMenuItem;
    actionToolsOptions: TAction;
    mnuToolsOptions: TMenuItem;
    mnuToolsLine1: TMenuItem;
    mnuReportsCoverSheet: TMenuItem;
    ActionReportsMedicationTherapy: TAction;
    mnuMedicationTherapy: TMenuItem;
    btnEnableScanner: TButton;
    mnuTakeActionOnBag: TMenuItem;
    actionDueListTakeActionOnBag: TAction;
    mnuDueListTakeActionOnBag: TMenuItem;
    ActionDueListDspSpecInstr: TAction;
    DisplaySpecialInstructions1: TMenuItem;
    DisplaySpecialInstructions2: TMenuItem;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    stAllergies: TStaticText;
    lstUnitDose: TListBox;
    sgUnitDose: TStringGrid;
    VA508CompAccessSgUnitDose: TVA508ComponentAccessibility;
    stVDLUnitDose: TStaticText;
    stAlt: TStaticText;
    VA508CompAccessSgIVBagDetail: TVA508ComponentAccessibility;
    stScannerStatusReady: TVA508StaticText;
    ApplicationEvents1: TApplicationEvents;

    procedure actionDueListTakeActionOnBagUpdate(Sender: TObject);
    procedure actionDueListTakeActionOnBagExecute(Sender: TObject);
    procedure mnuPopUpUnableToScanClick(Sender: TObject);
    procedure fraIV1mnuIVHistoryPopup(Sender: TObject);
    procedure actionFileExitExecute(Sender: TObject);
    (*
      Closes the form.
    *)

    procedure actionFileOpenPatientExecute(Sender: TObject);
    (*
      Closes the current patient, if any, and displays an InputQuery to scan the
      patient's armband.  Calls the BCMA_Patient.ScanPatient method to validate
      the patient.  If valid, the patient's data will be loaded into the
      BCMA_Patient object.  Method RebuildVirtualDueList is called to build
      and display the Virtual Due List.  The focus is then set to the
      edtScannerInput editbox to initiate the medication pass.
    *)

    procedure pnlScannerIndicatorClick(Sender: TObject);
    (*
      Sets focus to the edtScannerInput field.
    *)

    procedure edtScannerInputEnter(Sender: TObject);
    (*
      Sets the pnlScannerIndicator color to clLime, indicating that scanner input
      is possible.
    *)

    procedure edtScannerInputExit(Sender: TObject);
    (*
      Sets the pnlScannerIndicator color to clRed, indicating that scanner input
      will not be accepted.
    *)

    procedure FormCreate(Sender: TObject);
    (*
      Allocates memory, initializes variables and readonly displays.  Objects
      BCMA_UserParameters and BCMA_SiteParameters and created and loaded here.
      Empty BCMA_Patient and BCMA_ScannedDrug objects are created here as well.
    *)

    procedure FormActivate(Sender: TObject);
    (*
      Executed when the form receives focus, from application startup,
      returning from another form or when returning from another Windows
      application.  Refreshes the form and resets focus to the
      edtScannerInput field.
    *)

    procedure actionReportsDueListExecute(Sender: TObject);
    (*
      Calls method DueListReport to create the Due List report for the current
      patient.
    *)

    procedure actionReportsMedLogExecute(Sender: TObject);
    (*
      Calls method MedicationLogReport to create the Medication Log report
      for the current patient.
    *)

    procedure actionReportsMAHExecute(Sender: TObject);
    (*
      Calls method MedicationsGivenReport to create the Medications Given
      report for the current patient.
    *)

    procedure actionReportsAdminTimesExecute(Sender: TObject);
    (*
      Calls method WardAdministrationTimeReport to create the Ward
      Administration Times report for the current patient.
    *)

    procedure actionFileClosePatientExecute(Sender: TObject);
    (*
      Calls ClearVDList to clear the VirtualDueList grid, clears the
      VisibleMedList TList and clears BCMA_Patient.  Also closes any secondary
      (report) forms not yet closed.  All menu options, with the exception of
      mnFileOpenPatient, mnFileExit, mnHelpAbout, mnHelpContents,
      and mnHelpIndex are disabled.
    *)

    procedure FormResize(Sender: TObject);
    (*
      Re-centers the BCMA background whenever the form is resized.  Adjusts
      the VDList stringgrid to fit within the parent panel.
    *)

    procedure actionHelpAboutBCMAExecute(Sender: TObject);
    (*
      Creates and shows the AboutDlg form.
    *)

    procedure actionDueListMissingDoseExecute(Sender: TObject);
    (*
      Calls method EnterMissingDose to enter a missing dose request.
    *)

    procedure actionHelpContentIndexExecute(Sender: TObject);
    (*
      Activates Windows help, opened to the Contents page.
    *)

    procedure actionHelpIndexExecute(Sender: TObject);
    (*
      Activates Windows help, opened to the Index page.
    *)

    procedure mnAddCommentClick(Sender: TObject);
    (*
      Allows the user to enter a comment for the medication order highlighted
      on the Virtual Due List.
    *)

    procedure actionDueListMedHistoryExecute(Sender: TObject);
    (*
      Calls method MedicationHistoryReport to display a Medication History
      report for the highlighted order.
    *)

    procedure actionMarkRefusedExecute(Sender: TObject);
    (*
      Calls method MarkNotGiven to set the status of the highlighted
      medication as Held.
    *)

    procedure cmbxStopTimeChange(Sender: TObject);
    (*
      Calls the RebuildVirtualDueList procedure to refresh the Virtual Due
      List display.  Called whenever the StartTime or StopTime conboboxes
      change value.
    *)

    procedure cbxContinuousClick(Sender: TObject);
    (*
      Calls the RebuildVirtualDueList procedure to refresh the Virtual Due
      List display.  Called whenever the Continuous, PRN, One-Time and On-Call
      checkboxes are clicked.
    *)

    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    (*
      Calls method mnFileClosePatientClick to clear all patient infrormation.
      Saves the form position and size, the sort column and the status' of the
      checkboxes into the BCMA_UserParameters properties.  When the
      BCMA_UserParameters.free method is called all changes will be saved
      to the server.
    *)

    procedure FormShow(Sender: TObject);
    (*
      The BCMA_UserParameters properties are used to position and size the
      form the same as the last time the User logged in.  The Continuous,
      PRN, OneTime and OnCall checkboxes are reset as well.  The initial
      Sort Column for the Virtual Due List is also reset.
    *)

    procedure FormDestroy(Sender: TObject);
    (*
      Releases all allocated memory by freeing create d objects -
      BCMA_ScannedDrug, VisibleMedList, BCMA_Patient, BCMA_SiteParameters and
      BCMA_UserParameters.  When the BCMA_UserParameters.free method is called
      all changes will be saved to the server.
    *)

    procedure actionViewAllergiesExecute(Sender: TObject);
    (*
      Uses RPC 'PSB REACTIONS' and TfrmReport to show a report of patient
      allergies.
    *)

    procedure actionDueListUnableToScanExecute(Sender: TObject);
    (*
      Displays a list of the Ordered Drug IEN's for the highlighted order.
    *)

    procedure edtScannerInputKeyPress(Sender: TObject; var Key: Char);
    (*
      Calls method setDLMenus to enable/disable menus before the menu opens.
    *)

    procedure StatusBarTimerTimer(Sender: TObject);
    (*
      Adjusts the time displayed by the StatusBar Clock each time the
      StatusBarTimer Timer event fires.  The Clock displays the time as
      Now + BCMA_SiteParameters.ServerClockVariance.
    *)

    procedure mnStartIVClick(Sender: TObject);
    (*
      Logs an IV order as given, using method BCMA_MedOrder.LogOrder.  Calls
      RebuildVirtualDueList to refresh the Virtual Due List.
    *)

    procedure actionDueListAddCommentExecute(Sender: TObject);
    (*
      Calls method mnAddCommentClick.
    *)

    procedure actionDueListPRNEffectExecute(Sender: TObject);
    (*
      Calls method frmPRNEffectiveness.Execute to enter a PRNEffectivenes
      comment for a highlighted PRN order.
    *)

    procedure rePatientDemographicsEnter(Sender: TObject);
    (*
      Sets the bevel value to bvRaised, to indicate that the Patient
      Demographics 'button' has focus.
    *)

    procedure actionReportsMissedMedsExecute(Sender: TObject);
    (*
      Calls method MissedMedicationsReport to display a Missed Medications
      Report for the current patient.
    *)

    procedure actionMarkHeldExecute(Sender: TObject);
    (*
      Calls method MarkNotGiven to set the status of the highlighted
      medication as Held.
    *)

    procedure actionSortByActiveMedExecute(Sender: TObject);
    (*
      This method is called by the OnClick events of all of the sub-menuitems
      of the SortBy menuitem.  Each of the sub-menuitems has a 'tag' value
      equal to its corresponding VDL column number.  When the OnClick event
      occurs, this method sets the Sort Column equal to the tag value and then
      calls method RebuildVirtualDueList to repaint the VDL with the new sort
      order.
    *)

    procedure actionDueListRefreshExecute(Sender: TObject);
    (*
      Calls the RebuildVirtualDueList procedure to reload patient orders from
      the server and repaint the Virtual Due List.
    *)

    procedure mnDueListClick(Sender: TObject);
    (*
      Calls method setDLMenus to enables/disable 'Due List' menuitem sub-menus.
    *)

    procedure rePatientDemographicsMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    (*
      Calls method showPatientDemographics to show Patient Demographics.
    *)

    procedure rePatientDemographicsKeyPress(Sender: TObject;
      var Key: Char);
    (*
      If the user presses the <Enter> key, method showPatientDemographics is
      called to show Patient Demographics.
    *)

    procedure rePatientDemographicsExit(Sender: TObject);
    {
      Sets the bevel value to bvNone, to indicate that the Patient
      Demographics 'button' no longer has focus.
    }

    procedure AppDeactivate(Sender: TObject);
    {
      Calls method ScannerActivate
    }

    procedure AppActivate(Sender: TObject);

    procedure lstUnitDoseMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    {
      Called before lstUnitDoseDrawItem to calculate the height of the row
    }

    procedure lstUnitDoseDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    {
      Draws all the data on the VDL, prior to this, lstUnitDoseMeasureItem is
      called to calculate the row height.
    }

    procedure hdrUnitDoseSectionResize(HeaderControl: THeaderControl;
      Section: THeaderSection);
    {
      When a column header is resized, RebuildVirtualDueList is called which
      then repaints the screen.
    }

    procedure lstUnitDoseDblClick(Sender: TObject);
    {
      Displays the order detail for the hightlighted order.
    }

    procedure hdrUnitDoseMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    {
      Calls method RebuildVirtualDueList
    }

    procedure pgctrlVirtualDueListChange(Sender: TObject);
    {
      Moves controls from one tab to the next via the 'parent' property.
      Shows/hides sort menus according to current tab, calls method
      RebuildVirtualDueList
    }

    procedure hdrUnitDoseSectionClick(HeaderControl: THeaderControl;
      Section: THeaderSection);
    {
      Sets the current sort column for the current tab based on the 'clicked'
      column.  Also, reverses the current sort order.
    }

    procedure ShowHintProc(var HintStr: string;
      var CanShow: Boolean; var HintInfo: THintInfo);
    {
    }

    procedure lstUnitDoseMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    {
      If right clicked, if only one order is highlighted, sets either selected
      or itemindex, depdending on multiselect.
    }

    procedure actionMarkUndoExecute(Sender: TObject);
    {
      Calls method MarkNotGiven
    }

    procedure mnuRemovedClick(Sender: TObject);
    {
      Calls method MarkNotGiven
    }

    procedure sbrVDLScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    {
    }

    procedure lstUnitDoseCreateParams(var Params: TCreateParams);
    {
    }

    procedure lstUnitDoseMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    {
      Calls Method setDLMenus and ScannerActivate;
    }

    procedure lstUnitDoseContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    {
      If only one item is highlighted, clears all selected items and selects
      the currently highlighted administration.
    }

    procedure lstUnitDoseKeyPress(Sender: TObject; var Key: Char);
    {
      If a carriage return, call method lstUnitDoseDblClick to display the order
    }

    procedure lstUnitDoseClick(Sender: TObject);
    {
      If this we are on the IV tab, and currentOrderNumber doesn't
      match the current order on the VDL, set currentOrderNumber to the order
      nuumber on the vdl and call method RebuildIVOrderHistory
    }

    procedure fraIV1tvwIVHistoryChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    {
      To be removed
    }

    procedure fraIV1tvwIVHistoryChange(Sender: TObject; Node: TTreeNode);
    {
      Calls method tvwIVHistorySortStatus to obtain and sort all the dates, then
      sort bags by current status and build the tvwIVHistory
    }

    procedure fraIV1tvwIVHistoryMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    {
      If right click, call method SetIVHistMenus to set the menus, then display
      right click menu
    }

    procedure fraIV1lstIVBagDetailMeasureItem(Control: TWinControl;
      Index: Integer; var Height: Integer);
    {
      Calculates the height of each row
    }

    procedure fraIV1lstIVBagDetailDrawItem(Control: TWinControl;
      Index: Integer; Rect: TRect; State: TOwnerDrawState);
    {
      Draw the data on each row
    }

    procedure MarkHeldIV;
    {
      Call method fraIV1mnuHeldClick or MarkNotGiven depending on the current
      tab
    }

    procedure MarkRefusedIV;
    {
      Call method fraIV1mnuRefusedClick or MarkNotGiven depnding on the current
      tab
    }

    procedure fraIV1tvwIVHistoryExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    {
      set tvwIVExpandCollapse = true
    }

    procedure fraIV1tvwIVHistoryCollapsing(Sender: TObject;
      Node: TTreeNode; var AllowCollapse: Boolean);
    {
      set tvwIVExpandCollapse = true
    }

    procedure AddCommentIV;
    {
      Locates the order associated with the selected bag and allows the user
      to add a comment to the bag.
    }

    procedure fraIV1mnuMissingDoseClick(Sender: TObject);
    {
      Displays the Missing Dose form to allow the user to submit a missing dose
      for the selected bag
    }

    procedure actionToolsOptionsExecute(Sender: TObject);
    {
      displays frmOptions
    }

    procedure fraIV1hdrIVBagDetailSectionResize(
      HeaderControl: THeaderControl; Section: THeaderSection);
    {
      Allows the user to resize the header, keep track of the new column sizes
      and update the max width.
    }

    procedure fraIV1hdrIVBagDetailMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    {
      Set lstBagDetailColumnWidths[x] = to the header column widths, then
      call method RebuildBagDetail
    }

    procedure actionMedTabUDExecute(Sender: TObject);
    {
      Set the tabsheet to the unit dose tab, call method pgctrlVirtualDueListChange
    }

    procedure actionMedTabPBExecute(Sender: TObject);
    {
      Set the tabsheet to the IVP/IVPB tab, call method pgctrlVirtualDueListChange
    }

    procedure actionMedTabIVExecute(Sender: TObject);
    {
      Set the tabsheet to the IV tab, call method pgctrlVirtualDueListChange
    }

    procedure actionReportsPRNEffectivenessListExecute(Sender: TObject);
    {
      Call method PRNEffectivenessListReport
    }

    procedure actionViewPatientDemographicsExecute(Sender: TObject);
    {
      Call method showPatientDemographics
    }

    procedure actionMOBExecute(Sender: TObject);
    {
      Creates frmCPRSOrderManager, then calls CreateOleObject to create the
      CPRS COM object, and then passes the current broker parameters.
    }

    procedure actionMarkRemovedExecute(Sender: TObject);
    {
      Calls method MarkNotGiven
    }

    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    {
      Checks the CloseFrm property to see if the application can be closed.
    }

    procedure fraIV1tvwIVHistoryClick(Sender: TObject);
    {
      Calls method ScannerActivate
    }

    procedure fraIV1lstIVBagDetailClick(Sender: TObject);
    {
      Calls method ScannerActivate
    }

    procedure actionDueListTakeActionOnWSExecute(Sender: TObject);
    procedure actionSortByMedSolUpdate(Sender: TObject);
    procedure actionSortByInfusionRateUpdate(Sender: TObject);
    procedure actionSortByActiveMedUpdate(Sender: TObject);
    procedure actionSortByDosageUpdate(Sender: TObject);
    procedure actionSortByHSMUpdate(Sender: TObject);
    procedure actionSortByAdminTimeUpdate(Sender: TObject);
    procedure actionSortByLastActionUpdate(Sender: TObject);
    procedure actionSortByStatusUpdate(Sender: TObject);
    procedure actionViewAllergiesUpdate(Sender: TObject);
    procedure actionViewPatientDemographicsUpdate(Sender: TObject);
    procedure actionMedTabUDUpdate(Sender: TObject);
    procedure actionMedTabPBUpdate(Sender: TObject);
    procedure actionMedTabIVUpdate(Sender: TObject);
    procedure actionDueListAddCommentUpdate(Sender: TObject);
    procedure fraIV1tvwIVHistoryContextPopup(Sender: TObject;
      MousePos: TPoint; var Handled: Boolean);
    procedure actionDueListAvailableBagsUpdate(Sender: TObject);
    procedure actionDueListDisplayOrderUpdate(Sender: TObject);
    procedure actionMarkHeldUpdate(Sender: TObject);
    procedure actionMarkUndoUpdate(Sender: TObject);
    procedure actionMarkRefusedUpdate(Sender: TObject);
    procedure actionMarkRemovedUpdate(Sender: TObject);
    procedure actionDueListMedHistoryUpdate(Sender: TObject);
    procedure actionDueListMissingDoseUpdate(Sender: TObject);
    procedure actionDueListPRNEffectUpdate(Sender: TObject);
    procedure actionDueListUnableToScanUpdate(Sender: TObject);
    //    procedure actionDueListTakeActionOnWSUpdate(Sender: TObject);
    procedure actionDueListRefreshUpdate(Sender: TObject);
    procedure actionMOBUpdate(Sender: TObject);
    procedure lvwRemindersInfoTip(Sender: TObject; Item: TListItem;
      var InfoTip: string);
    procedure lvwRemindersDblClick(Sender: TObject);
    procedure lvwRemindersKeyPress(Sender: TObject; var Key: Char);
    procedure actionReportsVarianceExecute(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure actionReportVitalsCumulativeExecute(Sender: TObject);
    procedure actionReportVitalsCumulativeUpdate(Sender: TObject);
    procedure ActionSortByBagInformationUpdate(Sender: TObject);
    procedure ActionFileBreakPatientContextExecute(Sender: TObject);
    procedure ActionFileBreakPatientContextUpdate(Sender: TObject);
    procedure ActionJoinSetNewContextExecute(Sender: TObject);
    procedure cmbxStartTimeKeyPress(Sender: TObject; var Key: Char);
    procedure cmbxStopTimeKeyPress(Sender: TObject; var Key: Char);
    procedure ActionFileEditMedLogExecute(Sender: TObject);
    procedure WhatsThis1Popup(Sender, HelpItem: TObject;
      HContext: THelpContext; X, Y: Integer; var DoPopup: Boolean);
    procedure ActionFileEditMedLogUpdate(Sender: TObject);
    procedure ActionFileOpenReadOnlyExecute(Sender: TObject);
    procedure actionFlagExecute(Sender: TObject);
    procedure actionFileClosePatientUpdate(Sender: TObject);
    procedure actionFlagUpdate(Sender: TObject);
    procedure ActionJoinSetNewContextUpdate(Sender: TObject);
    procedure actionFileExitUpdate(Sender: TObject);
    procedure ActionJoinUseExistingContextExecute(Sender: TObject);
    procedure ActionJoinUseExistingContextUpdate(Sender: TObject);
    procedure edtScannerInputKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtScannerInputContextPopup(Sender: TObject;
      MousePos: TPoint; var Handled: Boolean);
    procedure actionMedTabCoverSheetExecute(Sender: TObject);
    procedure actionMedTabCoverSheetUpdate(Sender: TObject);
    procedure ActionCSExpandGrp0Execute(Sender: TObject);
    procedure ActionCSExpandGrp1Execute(Sender: TObject);
    procedure ActionCSExpandGrp2Execute(Sender: TObject);
    procedure actionReportUnknownActionsExecute(Sender: TObject);
    procedure actionReportUnknownActionsUpdate(Sender: TObject);
    procedure btnEnableScannerClick(Sender: TObject);
    procedure PopupActionBarPopup(Sender: TObject);
    procedure fraIV1tvwIVHistoryDblClick(Sender: TObject);
    procedure edtScannerInputKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionReportUnableToScanDetailedExecute(Sender: TObject);
    procedure ActionReportUnableToScanSummaryExecute(Sender: TObject);
    procedure actionFileOpenLimitedAccessUpdate(Sender: TObject);
    procedure actionFileOpenLimitedAccessExecute(Sender: TObject);
    procedure actionFileCreateWardStockUpdate(Sender: TObject);
    procedure actionFileCreateWardStockExecute(Sender: TObject);
    procedure ActionReportUnableToScanDetailedUpdate(Sender: TObject);
    procedure ActionReportUnableToScanSummaryUpdate(Sender: TObject);
    procedure actionDueListDrugIENUpdate(Sender: TObject);
    procedure actionDueListDrugIENExecute(Sender: TObject);
    procedure ActionReportsMedicationOverviewExecute(Sender: TObject);
    procedure ActionReportsPRNOverviewExecute(Sender: TObject);
    procedure actionReportsIVOverviewExecute(Sender: TObject);
    procedure actionReportsExpiredOrdersExecute(Sender: TObject);
    procedure actionReportsIVBagStatusExecute(Sender: TObject);
    procedure ActionReportsMedicationTherapyExecute(Sender: TObject);
    procedure actionSortByNurseUpdate(Sender: TObject);
    procedure actionSortByTypeUpdate(Sender: TObject);
    procedure actionSortByRouteUpdate(Sender: TObject);
    procedure ActionDueListDspSpecInstrExecute(Sender: TObject);
    procedure sgUnitDoseDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure ActionDueListDspSpecInstrUpdate(Sender: TObject);
    procedure sgUnitDoseClick(Sender: TObject);
    procedure sgUnitDoseDblClick(Sender: TObject);
    procedure sgUnitDoseKeyPress(Sender: TObject; var Key: Char);
    procedure sgUnitDoseMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure sgUnitDoseMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure sgUnitDoseContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    procedure sgUnitDoseSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure VA508CompAccessSgUnitDoseValueQuery(Sender: TObject;
      var Text: string);
    function GetRowHeight(CtrlCanvas: TCanvas; CtrlRect: TRect; rowidx:
      Integer): Integer;
    procedure VA508CompAccessSgUnitDoseItemQuery(Sender: TObject;
      var Item: TObject);
    procedure VA508CompAccessSgIVBagDetailItemQuery(Sender: TObject;
      var Item: TObject);
    procedure VA508CompAccessSgIVBagDetailValueQuery(Sender: TObject;
      var Text: string);
    procedure fraIV1sgIVBagDetailSelectCell(Sender: TObject; ACol,
      ARow: Integer; var CanSelect: Boolean);
    procedure fraIV1sgIVBagDetailDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure lstUnitDoseEnter(Sender: TObject);
    procedure sgUnitDoseEnter(Sender: TObject);
    procedure SchedTypeClick(Sender: TObject);
    procedure fraIV1sgIVBagDetailEnter(Sender: TObject);
    procedure cmbxStopTimeEnter(Sender: TObject);
    procedure cmbxStartTimeEnter(Sender: TObject);
    procedure cmbxStartTimeExit(Sender: TObject);
    procedure cmbxStopTimeExit(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure rePatientDemographicsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure pnlPatientDemographicsEnter(Sender: TObject);
    procedure pnlPatientDemographicsExit(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);

  private
    { Private declarations }

    FUDViewed,
      FPBViewed,
      FIVViewed,
      FCloseFrm,
      FWardStockInProc,
      FClosing,
      FTransferUDDisplayed,
      FTransferPBDisplayed: Boolean;

    {
        procedure RebuildVirtualDueList(ReloadMedOrders: boolean);
        (*
          Refreshes the VirtualDueList display corresponding to the filtering choices
          made in the Continuous, PRN, One-Time and On-Call checkboxes and the
          StartTime and StopTime comboboxes.
        *)
    }
    function AcceptMedOrder(MedOrder: TBCMA_MedOrder; SchedContinuous, SchedPRN,
      SchedOnCall, SchedOneTime: Boolean): boolean;
    (*
      Returns True if the input MedOrder is visible according to the selected
      Schedule Type checkboxes.  If the ScheduleType is 'C' and the OrderType
      is 'UnitDose', the Administration Time must be within the StartTime and
      StopTime window.
    *)

    procedure FillCbxTime(cbx: TComboBox; TwentyFourHours: Boolean);
    (*
      Uses BCMA_SiteParameters.ServerClockVariance as an offset to fill the
      StartTime and StopTime comboboxes.
    *)

    procedure showPatientDemographics;
    (*
      Uses RPC 'PSB PTINQUIRY' to retrieve patient demographics from the VistA
      server.  The information is displayed using the frmReport form.
    *)

    function scanMultipleDoses(idxOrder, idxDrug: integer): boolean;
    (*
      If the OrderedDrugCount > 1 or the QtyOrdered > 1 then form
      TfrmMultipleDrugs is created to allow scanning the additional Ordered
      Drug packages.
    *)

    procedure MarkNotGiven(newStatus: string);
    (*
      Uses form TfrmMedLog to set an order's status to 'Held' or 'Refused'.
      Method RebuildVirtualDueList is called to repaint the Virtual Due List.
    *)

    procedure setDLMenus;
    (*
      Enables/disables sub-menus, as appropriate for the highlighted
      VDL order, for the menuitem 'Due List', prior to showing them.
    *)

    procedure setStartStopTimes(TwentyFourHours: Boolean = False);
    (*
      Calls method FillCbxTime to fill the StartTime and StopTime comboboxes.
      Uses BCMA_SiteParameters properties VDLStartTime and VDLStopTime to
      set the initial values of each combobox.
    *)

    procedure ExceptionHandler(Sender: TObject; E: Exception);
    (*
      Calls the application exception handler, BCMAExceptionHandler, whenever
      an unhandled exception occurs.
    *)

    procedure MessageHandler(var Msg: TMsg; var Handled: Boolean);

    function GetNormalRect(AControl: TWinControl): TRect;
    (*
          Gets a Forms Top and Left position
    *)

    procedure WMGetMinMaxInfo(var M: TMessage); message WM_GETMINMAXINFO;
    {*
      Set the BCMA_Main maxheight and width via a message-handling procedure.
      Windows sends this message in order to get the min/max sizes.
      Setting form.constraints.maxheight/maxwidth didn't seem to work.
      These would somehow get increased for some unknown reason.
      Setting them via this procedure now has locked them at the size we
      have defined
    *}

    procedure WMActivate(var M: TMessage); message WM_ACTIVATE;

    procedure RebuildIVOrderHistory(RepaintIVHistory: Boolean; IVOrderNumber:
      string);
    {
      Loads all the bags associated with the highlighted order by calling method
      LoadIVBags.  Fetches the unique dates for all the bags, sorts them and adds
      them to the tree.
    }

    procedure RebuildBagDetail(Reload: Boolean);
    {
      Fetches the bag detail for the selected IV Bag by calling method
      GetBagDetails, after which adds the detail to lstIVBagDetail which in turn
      causes the repaint
    }

    function tvwIVHistorySortStatus(StringIn: string): TList;
    {
      Loops thru all the bags, finding bags that match StringIn, sorts those
      by status according to a predfined order and places them in a Tlist.
    }

//    function GetIVOrder(UniqueIDString: string): TBCMA_MedOrder;
    { Used on IV tab -
    Retrieves the order number for a specific Unique ID, if the unique ID
    is found in more then one order, displays an error message and retrurns nil.
    }

    function GetIVOrders(UniqueIDString: string): TStringList;
    { Used on PB tab - similiar to above except
    Retrieves all the orders that contain a specific Unique ID
    }

    function GetWardStockOrder(Add, Sol: TStringList): TStringList;
    {
      Finds orders that contains the exact additives and solutions that the user scanned
      and returns those orders via a TStringList.  IF no match is found, display
      a messagebox.
    }

    function SelectOrderID(OrderList: TStringList; DefaultOnlyOne: Boolean):
      integer;
    {
      If a user must select from more then one matching order, this code creates
      TfrmMultOrder and customizes it based on the current tab, which allows
      the user to select the order they desire.
    }

    procedure CheckIdleTimeout;
    {
      The purpose of this check is to see if BCMA has not been used for a
      specific period of time, and if so, shut down.
      Checks to see if the main form is visible, and there has been no mouse
      or keyboard activity for the preset time defined in the site parameters.
      If the preset time has lapsed, then form TfrmTimeOut will be displayed,
      and 30 second coutdown will start, in which the user can abort the countdown.
    }

    procedure CreateWardStock(ScannedDrugIEN: string; var CurFlowUID: string; var
      toBeWardStock: Pointer);
    {
      Uses method GetWardStockOrder to accumulate all the scanned additives and
      solutions and locate the orders that match the items scanned.  If the
      additives and solutions match more then order, the user will need to select
      the order they wish to give the wardstock item against.  Then checks to
      see if there is already a bag infusing, if so they are prompted to complete
      it.  Then the wardstock bag is marked infusing.
    }

    procedure CreateWardStock2(MedOrder: TBCMA_MedOrder; ScannedDrugIEN: string;
      var CurFlowUID: string; var
      toBeWardStock: Pointer; var PassFail: Boolean; VitalsInfo, PainInfo:
      string; LogState: TUASLogAction);
    {  JK
      Uses method GetWardStockOrder to accumulate all the scanned additives and
      solutions and locate the orders that match the items scanned.  If the
      additives and solutions match more then order, the user will need to select
      the order they wish to give the wardstock item against.  Then checks to
      see if there is already a bag infusing, if so they are prompted to complete
      it.  Then the wardstock bag is marked infusing.
    }

    procedure SetIVHistMenus;
    {
      This section of code sets the appropriate right click menu options
      when the user right clicks on tvwIVHistory.  It also sets the matching
      menu options on the main form.
    }
    procedure DisplayIgnoredAdmins(TStringIn: TStringList);
    {
      This populates the MultOrder form with a list of administrations
      that held or refused didn't apply to when the user highlights mult
      administrations and tries to mark then as held or refused.
    }

    procedure DisplayIEN;
    {
      Display the IEN for a VDL drug
    }

    procedure DisplayTransferMessage;

    procedure OpenPatient(CCOWPatientID: string = ''; CCOWPatientIDType: string
      =
      'SS';
      OpenLimitedAccess: Boolean = False);
    procedure AddToolsApps;
    procedure ToolsAppClick(Sender: TObject);

  public

    UAS_LogState: TUASLogAction; {JK 4/27/2008}
    WorkflowType: TWorkFlowType;
    {JK 4/28/2008 - needed to aid in determining what workflow route is occurring}

  { Public declarations }
    procedure RebuildVirtualDueList(ReloadMedOrders: boolean);
    (*
      Refreshes the VirtualDueList display corresponding to the filtering choices
      made in the Continuous, PRN, One-Time and On-Call checkboxes and the
      StartTime and StopTime comboboxes.
    *)

    function isValidScannedDrug(ScannedDrugIEN: string; DeferPRNProcessing:
      Boolean;
      isPRNCancelled: Boolean; PRNVitalsInfo, PRNPainInfo: string): Boolean;
    (*
      This is the method that validates scanned drugs and logs all medication
      passes on the UD tab and IVP/IVPB tab.
      It calls BCMA_ScannedDrug.isValidDrug(ScannedDrugIEN) to make
      sure the drug is in the database.  A check is then made to ensure that
      the scanned drug is found as part of at least one of the orders that is
      currently visible on the Virtual Due List.  If the drug appears on more
      than one order, form TfrmMultOrder displays the common orders so that
      the user can select one.  The order itself is validated using method
      TBCMA_MedOrder.ValidOrder.  After order validation, if
      TBCMA_MedOrder.Status = -1, a 'DO NOT GIVE!' message is shown.  If
      TBCMA_MedOrder.Status = 0, method scanMultipleDoses is called to check
      for multiple for Ordered Drugs.  If TBCMA_MedOrder.Status = 1, then
      method ConfirmOrder is called to Confirm the Order prior to calling
      method scanMultipleDoses.  Finally, if the order is logged as given,
      the function return value = True.
    *)

    procedure AddComment(aMedOrder: TBCMA_MedOrder; aIVBag: TBCMA_IVBags);
    (*
      Uses form TfrmMedLog to log a comment for the highlighted medication
      order.
    *)

//-    function ConfirmOrder(aMedOrder: TBCMA_MedOrder): boolean;
    procedure ConfirmOrder(aMedOrder: TBCMA_MedOrder; DeferPRNProcessing:
      Boolean;
      var isPRNCancelled: Boolean; var VitalsInfo, PainInfo: string);
    (*
      Uses form TfrmMedLog to confirm the highlighted medication order.  If
      the ScheduleType is Continuous, a comment is required.  If the
      ScheduleType is PRN, the a selection from the list of available
      PRN Reasons Given is required..
    *)
    procedure ScannerActivate();
    {
      if pnlMainForm is Visible and edtScannerInput is Enabled then
      setfocus to edtScannerInput call method edtScannerInputEnter
    }

    function ScanIV(ScannedDrugIEN: string; aIVActionType: TIVActionTypes; var
      CurFlowUID: string; var toBeWardStock: Pointer): Boolean;
    {
      This another BIG chunk of code that needs to be broke up.  It attempts to
      figure out what the user is actually scanning, which may be a UNIQUE ID or
      a WARDSTOCK UNIQUE ID, or it may be an additive or solution. If it is a Unique ID
      code attemps to search through all the orders and locate the order that
      contains the unique id.  If it is Ward Stock unique ID, the user must first
      highlight the order that matches the bag they are scanning/entering.  If this
      is an additive or solution, then we call method CreateWardStock.  If this
      was a unique id (including wardstock ID) eventually TfrmScanIV is displayed
      to allow the user to change the status of the bag.
    }

    procedure SetVDLMessage(StringIn: string);
    {
      Based on which tab the user is on, sets the caption property of a label.
    }

    procedure ForceVDLRebuild;
    {
      When called, displays a message to a user stating that the VDL will be
      refreshed, then calls method RebuildVirtualDueList
    }

    procedure ProcessPRNS(aMedOrder: TBCMA_MedOrder; var isPRNCancelled:
      Boolean; var PRNVitalsInfo, PRNPainInfo: string);

    procedure AbleToChangeContext(var OkayToChange: TVACCOW_AbleToChangeStatus;
      var MsgText: string; const NewID, NewICN, NewName, Site: string; const
      NewProd: Boolean);
    procedure UpdateCCOWLinkStatus(const Status: TVACCOW_LinkStatus);
    procedure SetCurrentPtIEN(const aPtId, aPtICN, aPtName, aPtSite: string;
      const aProd: Boolean);
    procedure GetSite(var Site: string);
    procedure CCOWError(const CCOWErr: Exception);
    procedure SetFormCaption;
    function MarkRemoved(aMedOrder: TBCMA_MedOrder = nil): Boolean;

    procedure CreateWardStockForFiveRightsOverride(aMedOrder: TBCMA_MedOrder; var
      BagName: string;
      var LogState: TUASLogAction; VitalsInfo, PainInfo: string); {JK 4/27/2008}

    function GetSGRowHeight(irow: Integer): Integer;

    procedure SetSGRowOld(irow: Integer);
    procedure SetSGRow(irow: Integer);

    property UDViewed: Boolean read FUDViewed write FUDViewed;
    property PBViewed: Boolean read FPBViewed write FPBViewed;
    property IVViewed: Boolean read FIVViewed write FIVViewed;
    property CloseFrm: Boolean read FCloseFrm write FCloseFrm default True;
    property WardStockInProc: Boolean read FWardStockInProc write
      FWardStockInProc;
    property Closing: Boolean read FClosing write FClosing default False;
    property TransferUDDisplayed: Boolean read FTransferUDDisplayed write
      FTransferUDDisplayed default False;
    property TransferPBDisplayed: Boolean read FTransferPBDisplayed write
      FTransferPBDisplayed default False;
  end;

  TShowHintEvent = procedure(var HintStr: string;
    var CanShow: Boolean; var HintInfo: THintInfo) of object;

procedure ShowCtrlName(Location: string; AControl: TControl);

type
  TSortType = (stAscending, stDescending);

var
  frmMain: TfrmMain;
  FixedColumnWidths: integer;
  SortColUD: TVDLColumnTypes;
  SortColPB: lstPBColumnTypes;
  SortColIV: lstIVColumnTypes;
  LeftClick: Boolean;
  FormResize: Boolean;
  CurrentItem: Integer;
  currentOrderNumber,
    currentAdministrationTime: string;
  CurrentBagID: Pointer;
  BagDetailItemsHeight: Integer;
  SortType: TSortType;
  lstCurrentTab: lstTabs;
  tvwIVExpandCollapse: Boolean;
  CurrentIEN: string;
  CurrentIVUID: string;
  ScannedInput: string;
  HoldOrder: Boolean;
  ReadOnly: Boolean; //VDL set to Read-Only?
  LimitedAccess: Boolean;
  OpenReadOnly: Boolean; //Open Read-Only Menu option used?
  // rpk: isUnableToScan is set true when actually performing UAS such as UAS action execute
  isUnableToScan: Boolean; {JK 4/26/2008 - global variable}
  EditMedLog: Boolean; {JK 7/13/2008 CQ #125}

implementation

uses
  Math,
  TRpcB,
  MFunStr,
  //	About,
  FAboutDlg,
  BCMA_Startup,
  BCMA_Common,
  Report,
  ReportRequest,
  MedLog,
  MultOrder,
  Instructor,
  MissingDose,
  MultipleDrugs,
  PRNEffectiveness,
  InputQry,
  ScanIV,
  WardStock,
  Options,
  SelectReason,
  BCMA_OrderMan,
  BCMA_Timeout,
  fPRNMedLog,
  uIVUtilities,
  fPtSelect,
  fEdtMedLogAdminSelect,
  Splash,
  oCoverSheet,
  fCoverSheet,
  fScanWristBand,
  fUnableToScan,
  fExpiredPatches,
  oReport,
  fDspMemo; // rpk 3/9/2009

var
  //  Data: array of array of string;

  SelectedX: integer;
  SelectedY: integer;
  StartTime, StopTime: string;

{$R *.DFM}

procedure ShowCtrlName(Location: string; AControl: TControl);
begin
  // DEBUG for control name
  if AControl <> nil then begin
    frmMain.StatusBar.Panels[1].Text := Location + ': ' + AControl.Name;
    frmMain.StatusBar.Invalidate;
  end;
end; // ShowCtrlName

procedure ShowActiveCtrlName;
var
  location: string;
  aWCtrl: TWinControl;
  aForm: TForm;
begin
  aWCtrl := nil;
  location := '';
  // DEBUG for identifying active form / control
  if screen <> nil then begin
    aForm := screen.ActiveForm;
    if aForm <> nil then begin
      location := aForm.Name;
      aWCtrl := aForm.ActiveControl;
    end;
  end;

  if AWCtrl <> nil then begin
    frmMain.StatusBar.Panels[1].Text := Location + ': ' + AWCtrl.Name;
    frmMain.StatusBar.Invalidate;
  end;
end; // ShowActiveCtrlName


function MedListCompare(Item1, Item2: pointer): Integer;
begin
  result := 1;
  with frmmain do
    case lstCurrentTab of
      //    case pgctrlVirtualDueList.ActivePageIndex of
      ctUD:
        case SortColUD of
          ctScanStatus:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ScanStatus,
              TBCMA_MedOrder(Item2).ScanStatus);
          ctSelfMed:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).SelfMed,
              TBCMA_MedOrder(Item2).SelfMed);
          ctScheduleType:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ScheduleType,
              TBCMA_MedOrder(Item2).ScheduleType);
          ctActiveMedication:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ActiveMedication,
              TBCMA_MedOrder(Item2).ActiveMedication);
          ctDosage:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Dosage,
              TBCMA_MedOrder(Item2).Dosage);
          ctRoute:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Route,
              TBCMA_MedOrder(Item2).Route);
          ctAdministrationTime:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).AdministrationTime,
              TBCMA_MedOrder(Item2).AdministrationTime);
          ctTimeLastGiven:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).TimeLastAction,
              TBCMA_MedOrder(Item2).TimeLastAction);
          ctVerifyNurse:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).VerifyNurse,
              TBCMA_MedOrder(Item2).VerifyNurse);
        end;
      ctPB:
        case SortColPB of
          pbScanStatus:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ScanStatus,
              TBCMA_MedOrder(Item2).ScanStatus);
          pbScheduleType:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ScheduleType,
              TBCMA_MedOrder(Item2).ScheduleType);
          pbActiveMedication:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ActiveMedication,
              TBCMA_MedOrder(Item2).ActiveMedication);
          pbInfusionRate:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Dosage,
              TBCMA_MedOrder(Item2).Dosage);
          pbRoute:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Route,
              TBCMA_MedOrder(Item2).Route);
          pbAdministrationTime:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).AdministrationTime,
              TBCMA_MedOrder(Item2).AdministrationTime);
          pbLastAction:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).TimeLastAction,
              TBCMA_MedOrder(Item2).TimeLastAction);
          pbVerifyNurse:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).VerifyNurse,
              TBCMA_MedOrder(Item2).VerifyNurse);
        end;
      ctIV:
        case SortColIV of
          ivOrderStatus:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).OrderStatus,
              TBCMA_MedOrder(Item2).OrderStatus);
          ivType:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).AdministrationUnit,
              TBCMA_MedOrder(Item2).AdministrationUnit);
          ivActiveMedication:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).ActiveMedication,
              TBCMA_MedOrder(Item2).ActiveMedication);
          ivInfusionRate:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Dosage,
              TBCMA_MedOrder(Item2).Dosage);
          ivRoute:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).Route,
              TBCMA_MedOrder(Item2).Route);
          ivVerifyNurse:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).VerifyNurse,
              TBCMA_MedOrder(Item2).VerifyNurse);
          ivBagInformation:
            result := AnsiCompareStr(TBCMA_MedOrder(Item1).TimeLastAction,
              TBCMA_MedOrder(Item2).TimeLastAction);
        end;

    end;

  if SortType = stDescending then begin
    if result = 1 then
      result := -1
    else if result = -1 then
      result := 1;
  end;

end;

procedure StringSplitter(longString: string; maxWidth: integer; var StringList:
  TStringList; canvas: TCanvas);
var
  ip: integer;

  sub,
    str: string;

  function dPos(str: string): integer;
  var
    DelimiterSet: set of char;
    ii: integer;
  begin
    DelimiterSet := [' ', ',', ';', '/', '.', '-', '!', '^'];
    ii := 1;
    while not (str[ii] in DelimiterSet) and
      (ii < length(str)) do
      inc(ii);
    result := ii;
  end;

begin
  str := longString;
  StringList.clear;
  StringList.add('');
  with canvas do
    while length(str) > 0 do begin
      ip := dPos(str);
      sub := copy(str, 1, ip);
      str := copy(str, ip + 1, length(str));
      if TextWidth(StringList.strings[StringList.count - 1] + sub) < maxWidth
        then
        StringList.strings[StringList.count - 1] :=
          StringList.strings[StringList.count - 1] + sub
      else
        StringList.add(sub);
    end;
end;

function IVHistoryDatesSort(Item1, Item2: Pointer): Integer;
begin
  Result := AnsiCompareStr(TBCMA_IVBags(Item1).TimeLastGiven,
    TBCMA_IVBags(Item2).TimeLastGiven);
  if Result = 1 then
    Result := -1
  else if Result = -1 then
    Result := 1;

end;

function TfrmMain.GetNormalRect(AControl: TWinControl): TRect;
var
  WP: TWindowPlacement;
begin
  WP.length := SizeOf(WP);
  GetWindowPlacement(AControl.Handle, @WP);
  Result := WP.rcNormalPosition;
end;

procedure TfrmMain.WMGetMinMaxInfo(var M: TMessage);
begin
  with PMINMAXINFO(m.lParam)^ do begin
    //    ptMaxTrackSize := Point(MAXFORMWIDTH, MAXFORMHEIGHT);
  end;
  m.result := 0;
end;

procedure TfrmMain.WMActivate(var M: TMessage);
begin
  inherited;
  //  if m.WParam = WA_INACTIVE then
  //    AppDeactivate(frmMain)
  //  else if m.WParam = WA_ACTIVE then
   //   ScannerActivate;
end;

procedure TfrmMain.ExceptionHandler(Sender: TObject; E: Exception);
begin
  BCMAExceptionHandler(Sender, E);
end;

procedure TfrmMain.actionFileExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.showPatientDemographics;
begin
  with pnlPatientDemographics do begin
    BevelInner := bvLowered;
    Update;

    with BCMA_Report do begin
      Clear;
      ReportType := rtPatientInquiry;
      PatientIEN := BCMA_Patient.IEN;
      PatientWard := 'P';
      Execute;
    end;
    BevelInner := bvRaised;
    Update;
  end;
end;

procedure TfrmMain.actionHelpAboutBCMAExecute(Sender: TObject);
begin
  AboutDlg.Execute;
end;

procedure TfrmMain.FillCbxTime(cbx: TComboBox; TwentyFourHours: Boolean);
//const
//  STARTTIMEDECREMENT = 12;
var
  hour,
    minutes,
    seconds,
    mseconds,
    year,
    month,
    day: word;
  STARTTIMEDECREMENT: Integer;
  ii: integer;
  ss: string;
  zdate: Tdate;
begin
  if TwentyFourHours then
    STARTTIMEDECREMENT := 24
  else
    STARTTIMEDECREMENT := 12;
  DecodeTime(now + BCMA_SiteParameters.ServerClockVariance, hour, minutes,
    seconds, mseconds);
  DecodeDate(now + BCMA_SiteParameters.ServerClockVariance, year, month, day);
  zDate := EncodeDate(year, month, day);
  with cbx do begin
    clear;
    sorted := False;
    hour := hour + (minutes div 30);
    for ii := hour - STARTTIMEDECREMENT to hour + STARTTIMEDECREMENT do begin
      if ii < 0 then
        ss := DateTimeToMDateTime((zDate - 1) + encodeTime(ii + 24, 0, 0, 0))
      else if ii < 24 then
        ss := DateTimeToMDateTime(zDate + encodeTime(ii, 0, 0, 0))
      else
        ss := DateTimeToMDateTime((zDate + 1) + encodeTime(ii - 24, 0, 0,
          0));

      items.addObject(DisplayVADate(ss), TmDateTime.create);
      TmDateTime(items.Objects[items.count - 1]).mDateTime := ss;
    end;
  end;
end;

procedure TfrmMain.setStartStopTimes(TwentyFourHours: Boolean = False);
var
  ii, StopDateMax: integer;
  zDate: Tdate;
  hour,
    minutes,
    seconds,
    mseconds,
    year,
    month,
    day: word;
begin
  FillCbxTime(cmbxStartTime, TwentyFourHours);
  cmbxStopTime.items.assign(cmbxStartTime.items);

  if TwentyFourHours then
    StopDateMax := 47
  else
    StopDateMax := 23;

  DecodeDate(now + BCMA_SiteParameters.ServerClockVariance, year, month, day);
  zDate := EncodeDate(year, month, day);

  DecodeTime(now + BCMA_SiteParameters.ServerClockVariance, hour, minutes,
    seconds, mseconds);

  hour := hour + (minutes div 30);
  with cmbxStartTime.items do
    if hour = 24 then
      ii := IndexOf(DisplayVADate(DateTimeToMDateTime(zDate + 1 + encodeTime(0,
        0, 0, 0))))
    else
      ii := IndexOf(DisplayVADate(DateTimeToMDateTime(zDate + encodeTime(hour,
        0, 0, 0))));

  with BCMA_SiteParameters do begin
    cmbxStartTime.ItemIndex := max(ii - VDLStartTime, 0);
    cmbxStopTime.ItemIndex := min(ii + VDLStopTime, StopDateMax);
  end;
end;

procedure TfrmMain.actionFileOpenPatientExecute(Sender: TObject);
//var
//  scanIEN,
//    CheckCaption: string;
//  CheckState: Boolean;
begin
  OpenPatient;
  exit;
end;

procedure TfrmMain.actionFileClosePatientExecute(Sender: TObject);
var
  ii: integer;
  msg: string;
  RequireUDView,
    RequirePBView: Boolean;

begin
  CloseFrm := True;
  if BCMA_Patient.IEN <> '' then
    with BCMA_Patient do begin

      if ((ActiveUDOrders = True) and (UDViewed = False)) then
        RequireUDView := True
      else
        RequireUDView := false;

      if (ActivePBOrders = True) and (PBViewed = False) then
        RequirePBView := True
      else
        RequirePBView := false;

      if (RequireUDView or RequirePBView) and (Closing = False) then begin
        msg := 'ACTIVE MEDICATION ORDERS are available for review on these Medication Tab(s):'
          + #13#13;
        if RequireUDView then
          msg := msg + '=> Unit Dose' + #13;
        if RequirePBView then
          msg := msg + '=> IVP/IVPB' + #13;

        msg := msg + #13 +
          'Are you sure you want to close this patient''s record before viewing these Tab(s)?';

        if DefMessageDlg(msg, mtInformation, [mbYes, mbNo], 0) = idNo then
          with pgctrlVirtualDueList do begin
            if RequireUDView then
              ActivePageIndex := 1
            else if RequirePBView then
              ActivePageIndex := 2
            else
              ActivePageIndex := 1;

            pgctrlVirtualDueListChange(Self);
            CloseFrm := False;
            exit;
          end;
      end;
    end;

  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      CloseMe;

  UDViewed := False;
  PBViewed := False;
  IVViewed := False;

  TransferUDDisplayed := False;
  TransferPBDisplayed := False;

  pnlMainForm.Visible := False;
  rePatientDemographics.clear;
  stAllergies.caption := '';

  edtScannerInput.Enabled := False;

  VisibleMedList.Clear;
  BCMA_Patient.Clear;

  tbshtUnitDose.ImageIndex := 0;
  tbshtIVPIVPB.ImageIndex := 0;
  tbshtIV.ImageIndex := 0;

  lstCurrentTab := ctUD;

  if BCMA_User.IsReadOnly = False then
    ReadOnly := false;
  LimitedAccess := False;
  SetFormCaption;
  if ReadOnly = false then
    pnlScannerInput.Enabled := true;
  for ii := Application.ComponentCount - 1 downto 0 do
    if Application.Components[ii] is TForm then
      if not (Application.Components[ii] is TfrmMain) then
        TForm(Application.Components[ii]).Release;
end;

procedure TfrmMain.edtScannerInputEnter(Sender: TObject);
begin
  pnlScannerIndicator.Color := clLime;
  stScannerStatusReady.Caption := 'Ready';
  //msf disable
{$IFDEF MSF_ON}
  btnEnableScanner.Enabled := False;
{$ENDIF}
  StopKeyboardTimer;
end;

procedure TfrmMain.edtScannerInputExit(Sender: TObject);
begin
  pnlScannerIndicator.Color := clRed;
  stScannerStatusReady.Caption := 'Not Ready';
  //msf disable
{$IFDEF MSF_ON}
  if (lstCurrentTab <> ctCS) and not ReadOnly and not LimitedAccess then
    btnEnableScanner.Enabled := True;
{$ENDIF}
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // add default setting;  Is this reset other than Broker.CallServer error?
  Shutdown := False; // rpk 10/20/2010

  //  Caption := Application.Title + ' - v' + AppFileVersion;
  if BCMA_User.IsReadOnly then
    ReadOnly := True;
  if ReadOnly = true then
    pnlScannerInput.Enabled := False;
  SetFormCaption;
  //This is set so 'hint' clinical reminder appears correctly.
  //Changing the 'hint' on the fly causes a flicker which is rather
  //ugly.
  Screen.HintFont.Name := 'Courier';
  Screen.HintFont.Size := 8;
  //  Application.HintPause := 1500;
  Application.HintHidePause := 2500;
  Application.OnException := ExceptionHandler;
  Application.OnDeactivate := AppDeactivate;
  Application.OnShowHint := ShowHintProc;
  Application.OnMessage := MessageHandler;
  Application.OnActivate := AppActivate;

  SortColUD := ctActiveMedication;
  SortColPB := PBActiveMedication;
  SortColIV := IVActiveMedication;

  UserIsLoggedOn := False;
  VisibleMedList := TList.create;
  IVHistoryDates := TList.Create;

  CreateCoverSheetObject;

  // Create BCMA_UserParameters Object and load it.
  BCMA_UserParameters := TBCMA_UserParameters.Create(BCMA_Broker);

  // Create BCMA_SiteParameters Object and load it.
  BCMA_SiteParameters := TBCMA_SiteParameters.Create(BCMA_Broker);
  with BCMA_SiteParameters do
    if LoadData then begin
      UserIsLoggedOn := True;

      // Create Patient
      BCMA_Patient := TBCMA_Patient.Create(BCMA_Broker);

      // Create ScannedDrug
      BCMA_ScannedDrug := TBCMA_DispensedDrug.Create(BCMA_Broker);

      BCMA_Report := TBCMA_Report.Create(BCMA_Broker);

      with StatusBar do begin
        panels[1].text := BCMA_User.UserName + InstructorName;
        panels[2].text := BCMA_User.DivisionName;
        panels[3].text := 'Server Time: ' +
          FormatDateTime('ddddd hh:mm',
          now + BCMA_SiteParameters.ServerClockVariance);
      end;

      with pnlBCMA.Constraints do begin
        MinHeight := gbBCMA.Height;
        MinWidth := gbBCMA.Width;
      end;

      with frmMain.Constraints do begin
        //        MaxHeight := MAXFORMHEIGHT;
        //        MaxWidth := MAXFORMWIDTH;
      end;

      with gbBCMA do begin
        top := (pnlBCMA.height - height) div 2;
        left := (pnlBCMA.width - width) div 2;
      end;

      IdleTime := Now;

    end;

  CheckMOBDLL;
  if TestBuild = True then
    DefMessageDlg('BCMA Version: ' + AppFileVersion + #13#13 +
      'This version of BCMA is to be used for testing purposes only ' + #13 +
      'and should not be used in a production environment.', mtInformation,
      [mbOK], 0);
  CloseFrm := True;
  lstUnitdose.Visible := False;
  sgUnitDose.Hide;

  lvwReminders.Column[1].Width := lvwReminders.Width -
    lvwReminders.Column[0].Width;

  Screen.Cursor := crHourglass;
  //  FrmSplash.lblStatus.Caption := 'Attempting communication with CCOW...';
  //  FrmSplash.stStatus.Caption := 'Attempting communication with CCOW...';
  //  FrmSplash.stStatus.Invalidate;  // rpk 6/8/2009
  //  FrmSplash.Update;
  FrmSplash.WriteStatus('Attempting communication with CCOW...'); // rpk 6/9/2009
  VACCOW := NewCCOWContexter(Application);
  with VACCOW do begin
    NotifyNewPatientID := SetCurrentPtIEN;
    NotifyNewLinkStatus := UpdateCCOWLinkStatus;
    NotifyAbleToChange := AbleToChangeContext;
    NotifyCCOWError := CCOWError;
    NotifyGetSite := GetSite;
  end;

  if NoCCOW then
    VACCOW.CCOWEnabled := False
  else begin
    //    FrmSplash.lblStatus.Caption := 'Communicating with CCOW Vault...';
    //    FrmSplash.stStatus.Caption := 'Communicating with CCOW Vault...';
    //    FrmSplash.stStatus.Invalidate;  // rpk 6/8/2009
    //    FrmSplash.Update;
    FrmSplash.WriteStatus('Communicating with CCOW Vault...'); // rpk 6/9/2009
  end;

  VACCOW.JoinContext('BCMA#');
  frmMain.StatusBar.Panels[0].Text := '';

  pnlCCOW.Visible := VACCOW.LinkStatus <> lsNoCCOW;
  Screen.Cursor := crDefault;

  lblReadOnly.Visible := BCMA_User.IsReadOnly;
  //  stReadOnly.Visible := BCMA_User.IsReadOnly;
  if BCMA_User.IsReadOnly = False then
    ActionFileOpenReadOnly.Visible := True
  else
    actionFileOpenReadOnly.visible := False;

  ColorMapFlag.FontColor := clRed;
  ColorMapFlag.SelectedFontColor := clRed;
  AddToolsApps;

  isUnableToScan := False; {JK 4/26/2008}
  UnableToScan := False; {JK 5/21/2008}

  DspMemo := nil; // rpk 3/9/2009

  sgUnitDose.DoubleBuffered := True;
  fraIV1.sgIVBagDetail.DoubleBuffered := True;

end; // FormCreate

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  ScannerActivate;
end;

procedure TfrmMain.actionReportsDueListExecute(Sender: TObject);
begin
  DueListReport(BCMA_Patient.IEN);
end;

procedure TfrmMain.actionReportsIVBagStatusExecute(Sender: TObject);
begin
  IVBagStatusReport;
end;

procedure TfrmMain.actionReportsIVOverviewExecute(Sender: TObject);
begin
  IVOverviewReport;
end;

procedure TfrmMain.actionReportsMedLogExecute(Sender: TObject);
begin
  MedicationLogReport(BCMA_Patient.IEN);
  ScannerActivate;
  //  if edtScannerInput.enabled then
  //    edtScannerInput.SetFocus;
end;

procedure TfrmMain.actionReportsMAHExecute(Sender: TObject);
begin
  MedicationsGivenReport(BCMA_Patient.IEN);
  ScannerActivate;
  //  if edtScannerInput.enabled then
  //    edtScannerInput.SetFocus
end;

procedure TfrmMain.actionDueListMedHistoryExecute(Sender: TObject);
begin
  if ScreenReaderSystemActive then // rpk 7/20/2010
    with TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row - 1]) do
      MedicationHistoryReport(PatientIEN, OrderableItemIEN, OrderNumber)
  else // rpk 7/20/2010
    with TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
      MedicationHistoryReport(PatientIEN, OrderableItemIEN, OrderNumber);
  ScannerActivate;
  //  edtScannerInput.Enabled := True;
  //  edtScannerInput.SetFocus;
end;

procedure TfrmMain.actionReportsAdminTimesExecute(Sender: TObject);
begin
  WardAdministrationTimeReport(BCMA_Patient.IEN);
end;

function TfrmMain.GetSGRowHeight(irow: Integer): Integer;
var
  maxheight, icol, cellHeight: Integer;
  aRect: TRect;
begin
  // find max height of active medications to set row high enough for display
  maxheight := 0;

  for Icol := 0 to sgUnitDose.ColCount - 1 do begin
    ARect := sgUnitDose.CellRect(icol, irow);
    with TBCMA_MedOrder(VisibleMedList.items[irow]) do begin
      CellHeight := DrawText(Canvas.Handle, PChar(ActiveMedication),
        Length(ActiveMedication),
        ARect, DT_SINGLELINE or DT_CALCRECT);
      maxheight := max(maxheight, CellHeight);
    end;
  end;

  Result := maxheight;

end; // GetSGRowHeight

function TfrmMain.AcceptMedOrder(MedOrder: TBCMA_MedOrder; SchedContinuous,
  SchedPRN, SchedOnCall, SchedOneTime: Boolean): boolean;
begin
  with MedOrder do begin
    result := false;
    if lstCurrentTab = ctUD then begin
      result := (
        (SchedContinuous and (uppercase(ScheduleType) = 'C')) and
        ((
        (AdministrationTime >=
        TmDateTime(cmbxStartTime.items.objects[cmbxStartTime.itemIndex]).mDateTime) and
        (AdministrationTime <=
        TmDateTime(cmbxStopTime.items.objects[cmbxStopTime.itemIndex]).mDateTime)
        ))
        ) or

      (SchedPRN and (uppercase(ScheduleType) = 'P')) or
        (SchedOnCall and (uppercase(ScheduleType) = 'OC')) or
        (SchedOneTime and (uppercase(ScheduleType) = 'O')) or
        ((AdministrationUnit = 'PATCH') and (ScanStatus = 'G'));
    end
    else if
      lstCurrentTab = ctPB then begin
      result := (
        (SchedContinuous and (uppercase(ScheduleType) = 'C')) and
        ((
        (AdministrationTime >=
        TmDateTime(cmbxStartTime.items.objects[cmbxStartTime.itemIndex]).mDateTime) and
        (AdministrationTime <=
        TmDateTime(cmbxStopTime.items.objects[cmbxStopTime.itemIndex]).mDateTime)
        ))
        ) or

      (SchedPRN and (uppercase(ScheduleType) = 'P')) or
        (SchedOnCall and (uppercase(ScheduleType) = 'OC')) or
        (SchedOneTime and (uppercase(ScheduleType) = 'O'));
    end
    else if lstCurrentTab = ctIV then
      Result := True;
  end;
end; // AcceptMedOrder

procedure TfrmMain.RebuildVirtualDueList(ReloadMedOrders: boolean);
var
  ii, i, jj, x: integer;
  NumOfTitles: Integer;
  maxwidth: Integer;
  HeaderSection: THeaderSection;
  TempWidth, TotalWidth: Integer;
  CellHeight: Integer;
  ARect: TRect;
  gr: TGridRect;
begin
  NumOfTitles := 0; // rpk 4/15/2009

  edtScannerInput.Enabled := False;
  SetVDLMessage('Loading Active Orders');
  //  lblVDLUnitDose.Show;
  //  lblVDLUnitDose.repaint;
  //  stVDLUnitDose.Show;
  //  stVDLUnitDose.Repaint;
  //  StatusBar.Repaint;
  StatusBar.Invalidate; // rpk 9/18/2009

  lstUnitDose.Visible := False;
  sgUnitDose.Hide;
  //  sgUnitDose.Show;  // rpk 10/30/2009
  hdrUnitDose.visible := false;

  case lstCurrentTab of
    ctIV: begin
        pnlIVTab.Visible := False;
//        lblVDLIV.Repaint;
        lblVDLIV.Invalidate; // rpk 9/10/2010
        with FraIV1 do begin
          if ReloadMedOrders = True then begin
            FraIV1.lblIVHistory.Caption := 'No Order Selected.';
            IVHistClearing := True;
            FraIV1.tvwIVHistory.Items.Clear;
            IVHistClearing := False;
            tvwIVHistory.Visible := False;
          end;
          lstIVBagDetail.Visible := False;
          sgIVBagDetail.Hide;
          lblBagDetail.Caption := 'No Bag Selected.';
          stBagDetail.Caption := 'No Bag Selected.';
        end;
      end;
  end;

  //  lblVDLUnitDose.repaint;
  //  stVDLUnitDose.BringToFront;
  //  stVDLUnitDose.Repaint;

    {save the current highlighted order}
  if ReloadMedOrders = False then begin
    if ScreenReaderSystemActive then begin
      with sgUnitDose do
        if VisibleMedList.Count > 0 then
          if row > 0 then
            with TBCMA_MedOrder(VisibleMedList.items[row - 1]) do begin
              currentOrderNumber := OrderNumber;
              currentAdministrationTime := AdministrationTime;
            end
    end
    else begin
      with lstUnitDose do
        if VisibleMedList.Count > 0 then
          if ItemIndex > -1 then
            with TBCMA_MedOrder(VisibleMedList.items[ItemIndex]) do begin
              currentOrderNumber := OrderNumber;
              currentAdministrationTime := AdministrationTime;
            end;
    end;
  end // ReloadMedOrders = False
  else begin
    CurrentOrderNumber := '';
    CurrentAdministrationTime := '';
  end;

  hdrUnitDose.Sections.BeginUpdate;
  VisibleMedList.clear;
  sgFree(-1, -1, sgUnitDose); // rpk 8/12/2009
  sgUnitDose.Repaint; // rpk 9/11/2010

  //  hdrUnitDose.Sections.Clear; {Removed by JK 1/31/2008}

  shpContinuous.Brush.Color := clWhite;
  shpPRN.Brush.color := clWhite;
  shpOneTime.Brush.Color := clWhite;
  shpOnCall.Brush.Color := clWhite;
  //  shpContinuous.Visible := False;
  //  shpPRN.Visible := False;
  //  shpOneTime.Visible := False;
  //  shpOnCall.Visible := False;;

  with BCMA_Patient do begin
    if ReloadMedOrders then begin
      frmMain.StatusBar.Panels[0].Text := 'Retrieving Administrations...';
      frmMain.StatusBar.Repaint;

      LoadMedOrders;
      tbshtUnitDose.ImageIndex := Ord(ActiveUDOrders);
      tbshtIVPIVPB.ImageIndex := Ord(ActivePBOrders);
      tbshtIV.ImageIndex := Ord(ActiveIVOrders);
      frmMain.StatusBar.Panels[0].Text := '';
      DisplayTransferMessage;
    end;

    if MedOrders.Count > 0 then begin
      if (cbxContinuous.Checked = False) and (cbxPRN.Checked = False) and
        (cbxOnCall.Checked = False) and (cbxOneTime.Checked = False) and
        (lstCurrentTab <> ctIV) then begin
        SetVDLMessage('No Schedule Type(s) Selected!');
      end;
      for jj := 0 to MedOrders.count - 1 do begin
        if AcceptMedOrder(MedOrders[jj], cbxContinuous.checked, cbxPRN.checked,
          cbxOnCall.checked, cbxOneTime.checked) then
          VisibleMedList.add(MedOrders[jj]);
        if (lstCurrentTab <> ctIV) and (AcceptMedOrder(MedOrders[jj], True,
          True, True, True)) then
          with TBCMA_MedOrder(MedOrders[jj]) do begin
            if uppercase(ScheduleType) = 'C' then
              shpContinuous.Brush.Color := clLime;
            if uppercase(ScheduleType) = 'P' then
              shpPRN.Brush.Color := clLime;
            if uppercase(ScheduleType) = 'OC' then
              shpOnCall.Brush.Color := clLime;
            if uppercase(ScheduleType) = 'O' then
              shpOneTime.Brush.Color := clLime;
          end;
      end;

      if lstCurrentTab <> ctIV then begin
        if not ScreenReaderSystemActive then // rpk 8/18/2010
          SetVDLMessage('There are no administrations to display based on your current Start/Stop Time and/or Schedule Type selections.');
        lstUnitDose.Clear;
        sgFree(-1, -1, sgUnitDose);
        sgUnitDose.Hide;
      end;
    end;
  end;

  //the following must occur after we populate VisbleMedList
  //but before we start painting the VDL.
  if (lstCurrentTab = ctIV) and ReloadMedOrders then begin
    LoadIVOrderChangeInfo;
  end;

  with VisibleMedList do
    if Count > 0 then begin
      pnlIVTab.Visible := True;
      ReDrawSuspend(hdrUnitDose.Handle);
      //      hdrUnitDose.Sections.Clear;  {Removed by JK 1/31/2008}

      hdrUnitDose.visible := True;

      // find max width of active medications to set first column wide enough for display
      maxwidth := 0;
      for I := 0 to Count - 1 do begin
        ARect := sgUnitDose.CellRect(0, i + 1);
        with TBCMA_MedOrder(VisibleMedList.items[i]) do begin
          CellHeight := DrawText(Canvas.Handle, PChar(ActiveMedication),
            Length(ActiveMedication),
            ARect, DT_SINGLELINE or DT_CALCRECT);
          maxwidth := max(maxwidth, ARect.Right - ARect.Left + 5);
        end;
      end;

      // sgUnitDose.ColWidths[0] := VDLColumnWidths[ctActiveMedication];
      sgUnitDose.ColWidths[0] := maxwidth;

      // fill titles for header control and top row of string grid (used as header)
      case lstCurrentTab of
        //      case pgctrlVirtualDueList.ActivePageIndex of
        ctUD: begin
            sgUnitDose.ColCount := length(VDLColumnTitles) + 1;
            //            hdrUnitDose.Sections.Clear;  // rpk 8/5/2009
            for ii := 0 to length(VDLColumnTitles) - 1 do begin
              if (hdrUnitDose.Sections.Count <= ii) then
                HeaderSection := hdrUnitDose.Sections.Add
              else
                HeaderSection := hdrUnitDose.Sections[ii];
              HeaderSection.Text := VDLColumnTitles[TVDLColumnTypes(ii)];
              sgUnitDose.Cells[ii + 1, 0] :=
                VDLColumnTitles[TVDLColumnTypes(ii)]; // rpk 8/13/2009
              hdrUnitDose.Sections.Items[ii].Width :=
                VDLColumnWidths[TVDLColumnTypes(ii)];
              sgUnitDose.ColWidths[ii + 1] :=
                VDLColumnWidths[TVDLColumnTypes(ii)];
            end;
          end;

        ctPB: begin
            sgUnitDose.ColCount := length(lstPBColumnTitles) + 1;
            for ii := 0 to length(lstPBColumnTitles) - 1 do begin
              HeaderSection := hdrUnitDose.Sections.Add;
              HeaderSection.Text := lstPBColumnTitles[lstPBColumnTypes(ii)];
              sgUnitDose.Cells[ii, 0] :=
                lstPBColumnTitles[lstPBColumnTypes(ii)];
              hdrUnitDose.Sections.Items[ii].Width :=
                lstPBColumnWidths[lstPBColumnTypes(ii)];
              sgUnitDose.ColWidths[ii + 1] :=
                lstPBColumnWidths[lstPBColumnTypes(ii)];
            end;
          end;

        ctIV: begin
            sgUnitDose.ColCount := length(lstIVColumnTitles) + 1;
            for ii := 0 to length(lstIVColumnTitles) - 1 do begin
              HeaderSection := hdrUnitDose.Sections.Add;
              HeaderSection.Text := lstIVColumnTitles[lstIVColumnTypes(ii)];
              sgUnitDose.Cells[ii, 0] :=
                lstIVColumnTitles[lstIVColumnTypes(ii)];
              hdrUnitDose.Sections.Items[ii].Width :=
                lstIVColumnWidths[lstIVColumnTypes(ii)];
              sgUnitDose.ColWidths[ii + 1] :=
                lstIVColumnWidths[lstIVColumnTypes(ii)];
            end;
          end;

      end; // case lstCurrentTab

      {Set the maxwidth so columns can't be scrolled off window}
      TotalWidth := 0;
      with hdrUnitDose.Sections do begin
        for ii := 0 to Count - 1 do begin
          TempWidth := ((Count - (ii + 1)) * 5);
          items[ii].maxwidth := hdrUnitDose.width - (TempWidth +
            TotalWidth);
          TotalWidth := TotalWidth + Items[ii].Width;
          items[ii].MinWidth := 5;
        end;
      end;

      ReDrawActivate(hdrUnitDose.Handle);
      with VisibleMedList do
        if count > 0 then begin
          lstUnitDose.Clear;
          sgFree(-1, -1, sgUnitDose);
          Sort(MedListCompare);

//          sgUnitDose.RowCount := count + 1;
          sgInit(sgUnitDose, 1, count);
          sgUnitDose.Repaint; // ??? rpk 10/30/2009

          for x := 0 to count - 1 do begin
            lstUnitDose.Items.Add(TBCMA_MedOrder(VisibleMedList[x]).OrderNumber);
            sgUnitDose.Cells[0, x + 1] :=
              (TBCMA_MedOrder(VisibleMedList[x]).ActiveMedication);
            //            sgUnitDose.RowHeights[x + 1] :=
            //              GetRowHeight(sgUnitDose.Canvas, sgUnitDose.CellRect(0, x + 1), x);
            //             SetSGRow(x + 1);

            //            lstUnitDose.Items.Create(lstUnitDose);

            //
            // When the user refreshes the VDL, this will re-highlight their selected order
            //
            if ReloadMedOrders = False then
              if (currentOrderNumber =
                TBCMA_MedOrder(VisibleMedList[x]).OrderNumber) and
                (currentAdministrationTime =
                TBCMA_MedOrder(VisibleMedList[x]).AdministrationTime) then begin
                //lstUnitDose.Selected[x] := True;
                if lstUnitDose.MultiSelect = False then
                  lstUnitDose.ItemIndex := x
                else
                  lstUnitDose.Selected[x] := True;

                // if sgUnitDose.Rows[x] MultiSelect = False then
                sgUnitDose.row := x + 1;
                gr.Left := 1;
                gr.Right := 1;
                gr.Top := x + 1;
                gr.Bottom := x + 1;
                sgUnitDose.Selection := gr;
                //   sgUnitDose.Selection.Top := x+1;
                //   sgUnitDose.Selection.Bottom := x+1;
                // else
                //   lstUnitDose.Selected[x] := True;
              end;
          end;

          //          ReDrawActivate(lstUnitDose.Handle);
          //          ReDrawActivate(sgUnitDose.Handle);
                    //          hdrUnitDose.Repaint;
                    //          lstUnitDose.Visible := True;
                    //          sgUnitDose.Show;
          if ScreenReaderSystemActive then begin
            ReDrawActivate(sgUnitDose.Handle);
            lstUnitDose.SendToBack;
            lstUnitDose.Hide;
            hdrUnitDose.Hide;
            lstUnitDose.TabStop := False;
            stVDLUnitDose.TabStop := False;
            sgUnitDose.TabStop := True;
            sgUnitDose.Align := alClient;
            sgUnitDose.BringToFront;
            sgUnitDose.Show;
            sgUnitDose.Repaint;
//            sgUnitDose.Invalidate; // 9/10/2010
          end
          else begin
            ReDrawActivate(lstUnitDose.Handle);
            sgUnitDose.SendToBack;
            sgUnitDose.Hide;
            sgUnitDose.TabStop := False;
            stVDLUnitDose.TabStop := True;
            lstUnitDose.TabStop := True;
            hdrUnitDose.Show;
            hdrUnitDose.Repaint;
//            hdrUnitDose.Invalidate; // rpk 9/10/2010
            lstUnitDose.Align := alClient;
            lstUnitDose.BringToFront;
            lstUnitDose.Show;
            lstUnitDose.Repaint;
//            lstUnitDose.Invalidate; // rpk 9/10/2010
          end;
        end;
      //      if pgctrlVirtualDueList.ActivePageIndex = 2 then
      if lstCurrentTab = ctIV then begin
        pnlIV.Visible := True;
        pnlIVTab.Visible := True;
        spltIV.Visible := True;
      end;
    end;

  //  if (lstCurrentTab <> ctIV) and (VisibleMedList.Count = 0) then
  //  begin
  //    lstUnitDose.Clear;
  //    sgClear(-1, -1, sgUnitDose);
  //    sgUnitDose.Hide;
  //    SetVDLMessage('There are no administrations to display based on your current Start/Stop Time and/or Schedule Type selections.');
  //  end;

  if (ReloadMedOrders = True) and (lstCurrentTab <> ctIV) then
    with BCMA_Patient do begin
      if ScreenReaderSystemActive then
        sgUnitDose.Repaint
//        sgUnitDose.Invalidate
      else
        lstUnitDose.Repaint;
//        lstUnitDose.Invalidate;

      LoadPRNEffectiveness('');

      try
        if lvwReminders.Items[0] = nil then begin //JK 11/29/2007
          //          NewItems := TListItems.Create(lvwReminders);
          //          NewItem := TListItem.Create(NewItems);
          //          lvwReminders.Items.Add;
          //          lvwReminders.Items[0].Caption := IntToStr(BCMA_Patient.PRNEffectList.Count);
          //          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
          lvwReminders.AddItem(IntToStr(BCMA_Patient.PRNEffectList.Count), nil);
          // rpk 7/22/2009
          lvwReminders.Items[0].SubItems.Add('PRN Effectiveness');
          // rpk 7/22/2009
        end
        else begin
          lvwReminders.Items[0].Caption :=
            IntToStr(BCMA_Patient.PRNEffectList.Count); //JK 11/29/2007
          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
        end;
      except
        on E: Exception do begin
          ;
        end;
      end;
      lvwReminders.Invalidate; // rpk 9/24/2009
    end;

  if (VisibleMedList.Count > 0) {and (ReloadMedOrders = True)} then begin
    edtScannerInput.Enabled := True;
    ScannerActivate;
  end;

  //--Added 1/31/2008 JK to fix problem reported in defect 765 and 766  --//

  case lstCurrentTab of
    ctUD: NumOfTitles := Length(VDLColumnTitles);
    ctPB: NumOfTitles := Length(lstPBColumnTitles);
    ctIV: NumOfTitles := Length(lstIVColumnTitles);
  end;

  if lstCurrentTab = ctUD then begin
    try
      //      sgUnitDose.ColCount := 10;
      sgUnitDose.ColCount := length(VDLColumnTitles) + 1;
      for ii := 0 to length(VDLColumnTitles) - 1 do begin
        if (hdrUnitDose.Sections.Count <= ii) then
          HeaderSection := hdrUnitDose.Sections.Add
        else
          HeaderSection := hdrUnitDose.Sections[ii];
        HeaderSection.Text := VDLColumnTitles[TVDLColumnTypes(ii)];
        sgUnitDose.Cells[ii + 1, 0] := VDLColumnTitles[TVDLColumnTypes(ii)];
        hdrUnitDose.Sections.Items[ii].Width :=
          VDLColumnWidths[TVDLColumnTypes(ii)];
        sgUnitDose.ColWidths[ii + 1] := VDLColumnWidths[TVDLColumnTypes(ii)];
      end;

    except
      ;
    end;
  end;

  if lstCurrentTab = ctPB then begin
    try
      //      sgUnitDose.ColCount := 9;
      sgUnitDose.ColCount := length(lstPBColumnTitles) + 1;
      for ii := 0 to length(lstPBColumnTitles) - 1 do begin
        if (hdrUnitDose.Sections.Count <= ii) then
          HeaderSection := hdrUnitDose.Sections.Add
        else
          HeaderSection := hdrUnitDose.Sections[ii];
        HeaderSection.Text := lstPBColumnTitles[lstPBColumnTypes(ii)];
        sgUnitDose.Cells[ii + 1, 0] := lstPBColumnTitles[lstPBColumnTypes(ii)];
        hdrUnitDose.Sections.Items[ii].Width :=
          lstPBColumnWidths[lstPBColumnTypes(ii)];
        sgUnitDose.ColWidths[ii + 1] :=
          lstPBColumnWidths[lstPBColumnTypes(ii)];
      end;

    except
      ;
    end;
  end;

  if lstCurrentTab = ctIV then begin
    try
      //      sgUnitDose.ColCount := 8;
      sgUnitDose.ColCount := length(lstIVColumnTitles) + 1;
      for ii := 0 to length(lstIVColumnTitles) - 1 do begin
        if (hdrUnitDose.Sections.Count <= ii) then
          HeaderSection := hdrUnitDose.Sections.Add
        else
          HeaderSection := hdrUnitDose.Sections[ii];
        HeaderSection.Text := lstIVColumnTitles[lstIVColumnTypes(ii)];
        sgUnitDose.Cells[ii + 1, 0] := lstIVColumnTitles[lstIVColumnTypes(ii)];
        hdrUnitDose.Sections.Items[ii].Width :=
          lstIVColumnWidths[lstIVColumnTypes(ii)];
        sgUnitDose.ColWidths[ii + 1] :=
          lstIVColumnWidths[lstIVColumnTypes(ii)];
      end;
    except
      ;
    end;
  end;

  if hdrUnitDose.Sections.Count > NumOfTitles then begin
    for i := hdrUnitDose.Sections.Count - 1 downto NumOfTitles do
      hdrUnitDose.Sections.Delete(i);
  end;

  hdrUnitDose.Sections.EndUpdate;

  if ScreenReaderSystemActive then begin
    VA508AccessibilityManager1.RefreshComponents;
//    sgUnitDose.Invalidate; // rpk 9/24/2009
    sgUnitDose.Refresh; // rpk 9/1/2010
  end;

  // Debug only
  {lstUnitDose.SendToBack;
  lstUnitDose.Hide;
  hdrUnitDose.Hide;
  sgUnitDose.Align := alClient;
  sgUnitDose.BringToFront;
  sgUnitDose.Show;}
//  sgUnitDose.Repaint;

//  sgUnitDose.Invalidate; // rpk 9/24/2009
//  sgUnitDose.Repaint;  // 10/30/2009

//  frmMain.Update; // rpk 9/18/2009
  frmMain.Refresh; // rpk 9/3/2010

end; // RebuildVirtualDueList

procedure TfrmMain.FormResize(Sender: TObject);
begin

  with gbBCMA do
    if visible then begin
      top := (pnlBCMA.height - height) div 2;
      left := (pnlBCMA.width - width) div 2;
    end;

  //  with lblVDLUnitDose do
  //  begin
  //    Top := pgctrlVirtualDueList.ActivePage.Top;
  //    Left := pgctrlVirtualDueList.ActivePage.left;
  //    Width := pgctrlVirtualDueList.ActivePage.Width;
  //    Height := pgctrlVirtualDueList.ActivePage.Height;
  //  end;
  //  with lblVDLUnitDose do
  //  begin
  //    Top := pgctrlVirtualDueList.ActivePage.Top;
  //    Left := pgctrlVirtualDueList.ActivePage.left;
  //    Width := pgctrlVirtualDueList.ActivePage.Width;
  //    Height := pgctrlVirtualDueList.ActivePage.Height div 2;
  //  end;

  //  with stVDLUnitDose do
  //  begin
  //    Top := pgctrlVirtualDueList.ActivePage.Top;
  //    Left := pgctrlVirtualDueList.ActivePage.left;
  //    Width := pgctrlVirtualDueList.ActivePage.Width;
  //    Height := pgctrlVirtualDueList.ActivePage.Height;
  //  end;
    // position stVDLUnitDose in vertical middle with height of 41 for two lines of text
  with stVDLUnitDose do begin
    Top := pgctrlVirtualDueList.ActivePage.Top +
      (pgctrlVirtualDueList.ActivePage.Height div 2) - 20;
    Left := pgctrlVirtualDueList.ActivePage.left;
    Width := pgctrlVirtualDueList.ActivePage.Width;
    Height := 41;
  end;

  with BCMA_UserParameters do
    if not IsZoomed(handle) then begin
      MainFormTop := Top;
      MainFormLeft := Left;
      MainFormHeight := Height;
      MainFormWidth := Width;
    end;

  //Make shift way of forcing pnlAllergies to resize
  if frmmain.Visible = true then begin
    stAllergies.caption := '';
    stAllergies.caption := 'ALLERGIES: ' + BCMA_Patient.Allergies + '    ' +
      'ADRs: ' + BCMA_Patient.ADRs;
  end;

  pnlScannerInput.Top := pnlScannerStatus.Top;
  pnlScannerInput.Left := pnlScannerStatus.Left;
  pnlScannerInput.Width := pnlScannerStatus.Width;
  pnlScannerInput.Height := pnlScannerStatus.Height;

end; // FormResize

procedure TfrmMain.actionDueListMissingDoseExecute(Sender: TObject);
var
  TempNode: TTreeNode;
begin
  if lstCurrentTab = ctIV then begin
    TempNode := fraIV1.tvwIVHistory.Selected;
    if TempNode.Level = 1 then begin
      //      with lstUnitDose do
      ScannedInput := TBCMA_IVBags(tempNode.data).UniqueID;
      if ScreenReaderSystemActive then begin
        if sgUnitDose.Row > 0 then // rpk 9/2/2010
          EnterMissingDose(VisibleMedList[sgUnitDose.row - 1],
            TBCMA_IVBags(tempNode.data))
      end
      else
        EnterMissingDose(VisibleMedList[lstUnitDose.ItemIndex],
          TBCMA_IVBags(tempNode.data));

      ScannedInput := ''
    end;
    if ScreenReaderSystemActive then
      RebuildIVOrderHistory(True, TBCMA_MedOrder(VisibleMedList[sgUnitDose.row -
        1]).OrderNumber)
    else
      RebuildIVOrderHistory(True,
        TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber);
  end
  else begin
    if ScreenReaderSystemActive then begin
      if sgUnitDose.Row > 0 then // rpk 9/2/2010
        EnterMissingDose(VisibleMedList[sgUnitDose.row - 1], nil)
    end
    else begin
      with lstUnitDose do
        if visible then
          EnterMissingDose(VisibleMedList[lstUnitDose.ItemIndex], nil);
    end;
    self.repaint;
    RebuildVirtualDueList(True);
    // is edtScannerInput already enabled?
    edtScannerInput.Enabled := True; // rpk 6/17/2010
    ScannerActivate;
    //    edtScannerInput.SetFocus
  end;
end; // actionDueListMissingDoseExecute

procedure TfrmMain.actionHelpContentIndexExecute(Sender: TObject);
begin
  with Application do
    if not HelpCommand(HELP_FINDER, 0) then
      DefMessageDlg('Error finding Contents ' + application.helpfile, mtError, [mbOK],
        0);
end;

procedure TfrmMain.actionHelpIndexExecute(Sender: TObject);
const
  HELP_TAB = 15;
begin
  with Application do
//    if not HelpCommand(HELP_INDEX, 0) then // obsolete
    // HELP_TAB with non-zero data calls HtmlHelp with HH_DISPLAY_INDEX.
    // See ehshelprouter OnRouteHelp for details.
    if not HelpCommand(HELP_TAB, 1) then // rpk 9/18/2010
      DefMessageDlg('Error accessing Index ' + application.helpfile, mtError, [mbOK],
        0);
end;

procedure TfrmMain.AddComment(aMedOrder: TBCMA_MedOrder; aIVBag: TBCMA_IVBags);
begin
  with TfrmMedLog.create(application) do try
    MedOrder := aMedOrder;
    MedLogType := mtMedPass;
    cmtUserComments := '';
    PageControl.ActivePage := tsAddComment;
    MedOrder.UnknownMessageDisplayed := False;
    if MedOrder = nil then begin
      DefMessageDlg('Cannot Add a Comment:  No Medication Order Found!',
        mtError, [mbOK], 0);
      Exit;
    end
    else begin
      with MedOrder do
        if (ValidOrder) or ((MedOrder.ScanStatus = 'G') and (AdministrationUnit
          = 'PATCH')) then begin
          //          lblActiveMedication.caption := lblActiveMedication.caption +
          //            ActiveMedication;
          lblActiveMedication.caption := ActiveMedication; // rpk 8/16/2010

          if aIVBag = nil then
            if OrderedDrugNames.count > 0 then
              //              lblDispensedDrug.caption := lblDispensedDrug.caption +
              //                OrderedDrugNames[0]
              lblDispensedDrug.caption := OrderedDrugNames[0] // rpk 8/16/2010
            else
              lblDispensedDrug.caption := '';

          mmoSpecialInstructions.text := SpecialInstructions;
          mmoSpecialInstructions.SelStart := 0; // rpk 8/31/2010
//          GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
          mmoMessage.text := StatusMessage;
          mmoMessage.SelStart := 0; // rpk 8/31/2010
//          GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
          UserComments := '';
        end
        else if Status = -2 then begin
          ForceVDLRebuild;
          exit;
        end
        else if Status = -10 then
          exit;
    end;
    if showModal = mrOk then
      with aMedOrder do begin
        UserComments := cmtUserComments;
        LogOrder(mtAddComment, '', aIVBag);
      end;

  finally
    free;
  end;
end;

//-function TfrmMain.ConfirmOrder(aMedOrder: TBCMA_MedOrder): boolean;

procedure TfrmMain.ConfirmOrder(aMedOrder: TBCMA_MedOrder; DeferPRNProcessing:
  Boolean;
  var isPRNCancelled: Boolean; var VitalsInfo, PainInfo: string);
begin
  //-  result := False;
  isPRNCancelled := True;

  cmtUserComments := '';
  cmtReasonGivenPrn := '';

  if aMedOrder = nil then begin
    DefMessageDlg('Cannot Add a Comment:  No Medication Order Found!',
      mtError, [mbOK], 0);
    Exit;
  end;

  if not aMedOrder.ValidOrder then begin
    if aMedOrder.Status = -2 then
      ForceVDLRebuild;
    exit;
  end;

  if aMedOrder.ScheduleTypeID = stPRN then
    {JK 8/22/2008}
    if not DeferPRNProcessing then
      ProcessPRNS(aMedOrder, isPRNCancelled, VitalsInfo, PainInfo)
    else begin
      isPRNCancelled := False;
      Exit;
    end

  else
    with TfrmMedLog.create(application) do try
      MedOrder := aMedOrder;
      with MedOrder do begin
        //        lblActiveMedication.caption := lblActiveMedication.caption +
        //          ActiveMedication;
        lblActiveMedication.caption := ActiveMedication; // rpk 8/16/2010
        if OrderedDrugNames.count > 0 then
          //          lblDispensedDrug.caption := lblDispensedDrug.caption +
          //            OrderedDrugNames[0]
          lblDispensedDrug.caption := OrderedDrugNames[0] // rpk 8/16/2010
        else
          lblDispensedDrug.caption := '';
        mmoSpecialInstructions.text := SpecialInstructions;
        mmoSpecialInstructions.SelStart := 0; // rpk 8/31/2010
//        GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
        mmoMessage.text := StatusMessage;
        mmoMessage.SelStart := 0; // rpk 8/31/2010
//        GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
        UserComments := '';
        ReasonGivenPRN := '';

        case ScheduleTypeID of
          stContinuous: begin
              MedLogType := mtMedpass;

              tsConfirmContinuous.caption := 'Confirm Continuous Medication';
              tsConfirmContinuous.HelpContext := 347; // confirm continuous help
              PageControl.ActivePage := tsConfirmContinuous;
            end;

          stOnCall: begin
              MedLogType := mtMedpass;

              tsConfirmContinuous.caption := 'Confirm On-Call Medication';
              tsConfirmContinuous.HelpContext := 353; // confirm on-call help
              PageControl.ActivePage := tsConfirmContinuous;
            end;

          stPRN: begin
              MedLogType := mtMedpass;

              PageControl.ActivePage := tsConfirmPRN;
              lbxPRNReasons.items.Assign(BCMA_SiteParameters.ListReasonsGivenPRN);
            end;
        else
          exit;
        end;
      end;

      //-      result := (showModal = mrOK);
      isPRNCancelled := not (showModal = mrOK);

      with MedOrder do
        case ScheduleTypeID of
          stContinuous, stOnCall:
            UserComments := cmtUserComments;
          stPRN:
            ReasonGivenPRN := cmtReasonGivenPRN;
        end;
    finally
      free;
    end;
end;

procedure TfrmMain.mnAddCommentClick(Sender: TObject);
begin
  with lstUnitDose do
    AddComment(VisibleMedList.items[ItemIndex], nil);
  with sgUnitDose do
    AddComment(VisibleMedList.items[row - 1], nil);

  edtScannerInput.Enabled := True;
  ScannerActivate;
  //  edtScannerInput.SetFocus;
end;

procedure TfrmMain.MarkNotGiven(newStatus: string);
var
  zLogOrder: Boolean;
  x: integer;
  IgnoredAdmins: TStringList;
  frmMedLog: TfrmMedLog; // rpk 9/13/2010
begin
  IgnoredAdmins := nil; // rpk 4/15/2009
  frmMedLog := nil;
  zLogOrder := False;  // rpk 12/15/2010
  
  if (newStatus = 'R') or (newStatus = 'H') then begin
//    with TfrmMedLog.create(application) do try
    try
      frmMedLog := TfrmMedLog.create(application);
      with frmMedLog do begin
        IgnoredAdmins := TStringList.Create;
        if ScreenReaderSystemActive then begin
          if sgUnitDose.Selection.Top = sgUnitDose.Selection.Bottom then begin
          {// one row selection}
            MedOrder := VisibleMedList[sgUnitDose.row - 1];
            MedOrder.Action := newStatus;
            MedOrder.UnknownMessageDisplayed := false;
            with MedOrder do
              if ValidOrder then
                with MedOrder do begin
                //              lblActiveMedication.caption := lblActiveMedication.caption +
                //                ActiveMedication;
                  lblActiveMedication.caption := ActiveMedication; // rpk 8/16/2010
                  if OrderedDrugNames.count > 0 then
                  //                lblDispensedDrug.caption := lblDispensedDrug.caption +
                  //                  OrderedDrugNames[0]
                    lblDispensedDrug.caption := OrderedDrugNames[0] // rpk 8/16/2010
                  else
                    lblDispensedDrug.caption := '';
                  mmoSpecialInstructions.text := SpecialInstructions;
                  mmoSpecialInstructions.SelStart := 0; // rpk 9/1/2010
  //              GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
                  mmoMessage.text := StatusMessage;
                  mmoMessage.SelStart := 0; // rpk 9/1/2010
  //              GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
                end
              else if Status = -2 then begin
                ForceVDLRebuild;
                exit;
              end
              else if Status = -10 then
                exit
              else begin
                DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0);
                exit
              end;
            MedOrder.Action := '';
          end
          //      else if lstUnitDose.SelCount > 1 then
          else if sgUnitDose.Selection.Top < sgUnitDose.Selection.Bottom then
          {// multiple row selection}begin
          //        lblActiveMedication.caption := lblActiveMedication.caption +
          //          'Multiple Orders Selected';
            lblActiveMedication.caption := 'Multiple Orders Selected';
          // rpk 8/16/2010
            lblDispensedDrug.caption := '';
            mmoSpecialInstructions.text := 'Multiple Orders Selected';
            mmoSpecialInstructions.SelStart := 0; // rpk 9/1/2010
  //        GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
            mmoMessage.text := 'Multiple Orders Selected';
            mmoMessage.SelStart := 0; // rpk 9/1/2010
  //        GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
          end;
        end  // if screenreadersystemactive
        else begin
          if lstUnitDose.SelCount = 1 then begin
            MedOrder := VisibleMedList[lstUnitDose.ItemIndex];
            MedOrder.Action := newStatus;
            MedOrder.UnknownMessageDisplayed := false;
            with MedOrder do
              if ValidOrder then
                with MedOrder do begin
                //              lblActiveMedication.caption := lblActiveMedication.caption +
                //                ActiveMedication;
                  lblActiveMedication.caption := ActiveMedication; // rpk 8/16/2010
                  if OrderedDrugNames.count > 0 then
                  //                lblDispensedDrug.caption := lblDispensedDrug.caption +
                  //                  OrderedDrugNames[0]
                    lblDispensedDrug.caption := OrderedDrugNames[0] // rpk 8/16/2010
                  else
                    lblDispensedDrug.caption := '';
                  mmoSpecialInstructions.text := SpecialInstructions;
                  mmoSpecialInstructions.SelStart := 0; // rpk 8/31/2010
  //              GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
                  mmoMessage.text := StatusMessage;
                  mmoMessage.SelStart := 0; // rpk 8/31/2010
  //              GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
                end
              else if Status = -2 then begin
                ForceVDLRebuild;
                exit;
              end
              else if Status = -10 then
                exit
              else begin
                DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0);
                exit
              end;
            MedOrder.Action := '';
          end
          else if lstUnitDose.SelCount > 1 then begin
          //        lblActiveMedication.caption := lblActiveMedication.caption +
          //          'Multiple Orders Selected';
            lblActiveMedication.caption := 'Multiple Orders Selected';
          // rpk 8/16/2010
            lblDispensedDrug.caption := '';
            mmoSpecialInstructions.text := 'Multiple Orders Selected';
            mmoSpecialInstructions.SelStart := 0; // rpk 8/31/2010
  //        GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
            mmoMessage.text := 'Multiple Orders Selected';
            mmoMessage.SelStart := 0; // rpk 8/31/2010
  //        GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
          end;
        end;  // else not screenreader

        (* if sgUnitDose.Selection.Top = sgUnitDose.Selection.Bottom then
        {// one row selection}begin
          MedOrder := VisibleMedList[sgUnitDose.row - 1];
          MedOrder.Action := newStatus;
          MedOrder.UnknownMessageDisplayed := false;
          with MedOrder do
            if ValidOrder then
              with MedOrder do begin
              //              lblActiveMedication.caption := lblActiveMedication.caption +
              //                ActiveMedication;
                lblActiveMedication.caption := ActiveMedication; // rpk 8/16/2010
                if OrderedDrugNames.count > 0 then
                //                lblDispensedDrug.caption := lblDispensedDrug.caption +
                //                  OrderedDrugNames[0]
                  lblDispensedDrug.caption := OrderedDrugNames[0] // rpk 8/16/2010
                else
                  lblDispensedDrug.caption := '';
                mmoSpecialInstructions.text := SpecialInstructions;
                mmoSpecialInstructions.SelStart := 0; // rpk 9/1/2010
//              GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
                mmoMessage.text := StatusMessage;
                mmoMessage.SelStart := 0; // rpk 9/1/2010
//              GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
              end
            else if Status = -2 then begin
              ForceVDLRebuild;
              exit;
            end
            else if Status = -10 then
              exit
            else begin
              DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0);
              exit
            end;
          MedOrder.Action := '';
        end
        //      else if lstUnitDose.SelCount > 1 then
        else if sgUnitDose.Selection.Top < sgUnitDose.Selection.Bottom then
        {// multiple row selection}begin
        //        lblActiveMedication.caption := lblActiveMedication.caption +
        //          'Multiple Orders Selected';
          lblActiveMedication.caption := 'Multiple Orders Selected';
        // rpk 8/16/2010
          lblDispensedDrug.caption := '';
          mmoSpecialInstructions.text := 'Multiple Orders Selected';
          mmoSpecialInstructions.SelStart := 0; // rpk 9/1/2010
//        GetScreenReader.Speak(mmoSpecialInstructions.text);  // rpk 9/6/2010
          mmoMessage.text := 'Multiple Orders Selected';
          mmoMessage.SelStart := 0; // rpk 9/1/2010
//        GetScreenReader.Speak(mmoMessage.text);  // rpk 9/6/2010
        end; *)

        PageControl.ActivePage := tsNotGiven;

        if newStatus = 'H' then begin
          tsNotGiven.Caption := 'Medication Order Held';
//          lbxReasons.items.Assign(BCMA_SiteParameters.ListReasonsHeld);
          lbxNotGivenReasons.items.Assign(BCMA_SiteParameters.ListReasonsHeld);
        end
        else if newStatus = 'R' then begin
          tsNotGiven.Caption := 'Medication Order Refused';
//          lbxReasons.items.Assign(BCMA_SiteParameters.ListReasonsRefused);
          lbxNotGivenReasons.items.Assign(BCMA_SiteParameters.ListReasonsRefused);
        end
        else
          exit;

        cmtUserComments := '';
        cmtReasonGivenPrn := '';

        if showModal = mrOK then begin
          if ScreenReaderSystemActive then begin
            for x := 1 to sgUnitDose.Rowcount do begin
              if (sgUnitDose.selection.Top = sgUnitDose.Selection.Bottom) and
                (sgUnitDose.Selection.Top = x) then begin
                MedOrder := VisibleMedList[x - 1];
                with MedOrder do begin
                  if ValidOrder then begin
                    UserComments := '';
                    ReasonGivenPRN := '';
                    zLogOrder := False;
                    case OrderTypeID of
                      otUnitDose, otIV:
                        case ScheduleTypeId of
                          stContinuous, stOneTime:
                            if newStatus = 'H' then begin
                              if (ScanStatus = '') or (ScanStatus = 'M') then
                                zLogOrder := True
                            end
                            else if newStatus = 'R' then begin
                              if ((Scanstatus = '') or (ScanStatus = 'H')
                                or
                                (ScanStatus = 'M')) and (OrderStatus <>
                                'H') then
                                zLogOrder := True
                            end;
                          stOnCall: begin
                              if ((newStatus = 'R') and (ScanStatus = ''))
                                and (OrderStatus <> 'H') then
                                zLogOrder := True;
                              if newStatus = 'H' then
                                if ((ScanStatus = '') or (ScanStatus = 'M')) and
                                  (OrderStatus <> 'H') then
                                  zLogOrder := True
                            end;
                        end;
                    end;

                    if zLogOrder = True then begin
                      ReasonGivenPRN := cmtReasonGivenPRN;
                      UserComments := cmtUserComments;
                      MedOrder.LogOrder(mtMedPass, newStatus, nil);
                    end
                    else
                      IgnoredAdmins.Add(IntToStr(x - 1));
                  end
                  else if Status = -2 then begin
                    ForceVDLRebuild;
                    exit;
                  end
                  else
                    IgnoredAdmins.Add(IntToStr(x - 1));
                end;
              end;
//            DisplayIgnoredAdmins(IgnoredAdmins);
            end; // for sgunitdose rowcount
            DisplayIgnoredAdmins(IgnoredAdmins); // rpk 9/13/2010
          end
          else begin
            for x := 0 to lstUnitDose.Items.count - 1 do begin
              if lstUnitDose.selected[x] then begin
                MedOrder := VisibleMedList[x];
                with MedOrder do begin
                  if ValidOrder then begin
                    UserComments := '';
                    ReasonGivenPRN := '';
                    zLogOrder := False;
                    case OrderTypeID of
                      otUnitDose, otIV:
                        case ScheduleTypeId of
                          stContinuous, stOneTime:
                            if newStatus = 'H' then begin
                              if (ScanStatus = '') or (ScanStatus = 'M') then
                                zLogOrder := True
                            end
                            else if newStatus = 'R' then begin
                              if ((Scanstatus = '') or (ScanStatus = 'H')
                                or
                                (ScanStatus = 'M')) and (OrderStatus <>
                                'H') then
                                zLogOrder := True
                            end;
                          stOnCall: begin
                              if ((newStatus = 'R') and (ScanStatus = ''))
                                and (OrderStatus <> 'H') then
                                zLogOrder := True;
                              if newStatus = 'H' then
                                if ((ScanStatus = '') or (ScanStatus = 'M')) and
                                  (OrderStatus <> 'H') then
                                  zLogOrder := True
                            end;
                        end;
                    end;

                    if zLogOrder = True then begin
                      ReasonGivenPRN := cmtReasonGivenPRN;
                      UserComments := cmtUserComments;
                      MedOrder.LogOrder(mtMedPass, newStatus, nil);
                    end
                    else
                      IgnoredAdmins.Add(IntToStr(x));
                  end
                  else if Status = -2 then begin
                    ForceVDLRebuild;
                    exit;
                  end
                  else
                    IgnoredAdmins.Add(IntToStr(x));
                end; // with MedOrder
              end; // if selected
//          DisplayIgnoredAdmins(IgnoredAdmins);
            end; // for x
            DisplayIgnoredAdmins(IgnoredAdmins); // rpk 9/13/2010
          end; // else not screen reader active

        end; // if showmodal OK
      end;

    finally
//      free;
      IgnoredAdmins.free;
      frmMedLog.Free;
    end;
    RebuildVirtualDueList(False);
    self.repaint;
  end
  else if newStatus = 'N' then begin
    if ScreenReaderSystemActive then begin
      // copy of lstunitdose
      with TBCMA_MedOrder(VisibleMedList[sgUnitDose.row - 1]) do begin
        UnknownMessageDisplayed := False;
        Action := newStatus;
        if ValidOrder then begin
          LogOrder(mtMedPass, newStatus, nil);
          RebuildVirtualDueList(True);
          self.repaint;
        end
        else if Status = -2 then
          ForceVDLRebuild
        else if Status = -10 then
          exit
        else
          DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0)
      end;
    end // ScreenReaderSystemActive
    else begin
      with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
        UnknownMessageDisplayed := False;
        Action := newStatus;
        if ValidOrder then begin
          LogOrder(mtMedPass, newStatus, nil);
          RebuildVirtualDueList(True);
          self.repaint;
        end
        else if Status = -2 then
          ForceVDLRebuild
        else if Status = -10 then
          exit
        else
          DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0)
      end;
    end;

  end
  else if newStatus = 'RM' then begin

    if ScreenReaderSystemActive then begin
      // copy of lstunitdose
      with TBCMA_MedOrder(VisibleMedList[sgUnitDose.row - 1]) do begin
        UnknownMessageDisplayed := False;
        Action := newStatus;
        if ValidOrder then begin
          LogOrder(mtMedPass, newStatus, nil);
          RebuildVirtualDueList(False);
          Self.repaint;
        end
        else if Status = -2 then
          ForceVDLRebuild
        else if Status = -10 then
          exit
        else
          DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0)
      end;
    end // ScreenReaderSystemActive
    else begin
      with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
        UnknownMessageDisplayed := False;
        Action := newStatus;
        if ValidOrder then begin
          LogOrder(mtMedPass, newStatus, nil);
          RebuildVirtualDueList(False);
          Self.repaint;
        end
        else if Status = -2 then
          ForceVDLRebuild
        else if Status = -10 then
          exit
        else
          DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0)
      end;
    end;

  end;

end; // MarkNotGiven

procedure TfrmMain.actionMarkHeldExecute(Sender: TObject);
begin
  if lstCurrentTab = ctIV then
    MarkHeldIV
  else
    MarkNotGiven('H');
end;

procedure TfrmMain.actionMarkRefusedExecute(Sender: TObject);
begin
  if lstCurrentTab = ctIV then
    MarkRefusedIV
  else
    MarkNotGiven('R');
end;

procedure TfrmMain.mnStartIVClick(Sender: TObject);
var
  isPRNCancelled: Boolean; {JK 8/15/2008}
  VitalsInfo, PainInfo: string; {JK 8/15/2008}
begin
  if ScreenReaderSystemActive then begin
    with sgUnitDose do
      with TBCMA_MedOrder(VisibleMedList[sgUnitDose.row - 1]) do begin
        if ScheduleType = 'P' then begin
          ConfirmOrder(VisibleMedList.Items[sgUnitDose.row - 1], False,
            isPRNCancelled, VitalsInfo, PainInfo);
          if not isPRNCancelled then
            LogOrder(mtMedPass, 'G', nil);
        end
        else
          LogOrder(mtMedPass, 'G', nil);
      end;

  end // ScreenReaderSystemActive
  else begin
    with lstUnitDose do
      with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
        if ScheduleType = 'P' then begin
          //-        if ConfirmOrder(VisibleMedList.Items[lstUnitDose.ItemIndex]) then
          ConfirmOrder(VisibleMedList.Items[lstUnitDose.ItemIndex], False,
            isPRNCancelled, VitalsInfo, PainInfo);
          if not isPRNCancelled then
            LogOrder(mtMedPass, 'G', nil);
        end
        else
          LogOrder(mtMedPass, 'G', nil);
      end;
  end;

  RebuildVirtualDueList(False);
end; // mnStartIVClick

procedure TfrmMain.pnlPatientDemographicsEnter(Sender: TObject);
begin
  pnlPatientDemographics.BevelOuter := bvRaised;
  rePatientDemographics.Perform(WM_VSCROLL, SB_TOP, 0);
  rePatientDemographics.SelStart := 0;
end;

procedure TfrmMain.pnlPatientDemographicsExit(Sender: TObject);
begin
  rePatientDemographics.Perform(WM_VSCROLL, SB_TOP, 0);
  pnlPatientDemographics.BevelOuter := bvNone;
end;

procedure TfrmMain.pnlScannerIndicatorClick(Sender: TObject);
begin
  ScannerActivate;
  //  if edtScannerInput.enabled then
  //  begin
  //    edtScannerInputEnter(ActiveControl);
  //    edtScannerInput.SetFocus;
  //  end;

end;

procedure TfrmMain.cmbxStopTimeChange(Sender: TObject);
begin
  if sender = cmbxStartTime then begin
    with cmbxStartTime do
      if
        TmDateTime(cmbxStartTime.items.objects[cmbxStartTime.itemIndex]).mDateTime
        >
        TmDateTime(cmbxStopTime.items.objects[cmbxStopTime.itemIndex]).mDateTime
        then
        itemIndex := items.indexOf(cmbxStopTime.text);
    StartTime := cmbxStartTime.Text; // rpk 9/3/2010
  end;

  if sender = cmbxStopTime then begin
    with cmbxStopTime do
      if TmDateTime(cmbxStopTime.items.objects[cmbxStopTime.itemIndex]).mDateTime
        <
        TmDateTime(cmbxStartTime.items.objects[cmbxStartTime.itemIndex]).mDateTime then
        itemIndex := items.indexOf(cmbxStartTime.text);
    StopTime := cmbxStopTime.Text; // rpk 9/3/2010
  end;

  RebuildVirtualDueList(False);
  ScannerActivate;
end;

procedure TfrmMain.cmbxStopTimeEnter(Sender: TObject);
begin
  StopTime := cmbxStopTime.Text;
end;

procedure TfrmMain.cmbxStopTimeExit(Sender: TObject);
begin
  if StopTime <> cmbxStopTime.Text then
    cmbxStopTimeChange(Sender);
  StopTime := '';
end;

procedure TfrmMain.cbxContinuousClick(Sender: TObject);
begin
  case lstCurrentTab of
    ctIV:
      Exit;
  else begin
      RebuildVirtualDueList(False);
    end;

  end; // case

  ScannerActivate;
end;

procedure TfrmMain.SchedtypeClick(Sender: TObject);
begin
  case lstCurrentTab of
    ctIV:
      Exit;
  else
    RebuildVirtualDueList(False);
  end; // case

  ScannerActivate;

  TCheckBox(Sender).SetFocus;

end;

procedure TfrmMain.FormShow(Sender: TObject);
//var
//  x: integer;
begin
  with BCMA_UserParameters do
    if LoadData then begin
      setStartStopTimes;

      cbxContinuous.checked := ContiniousChecked;
      cbxPRN.checked := PRNChecked;
      cbxOneTime.checked := OneTimeChecked;
      cbxOnCall.checked := OnCallChecked;

      SortColUD := UDSortColumn;
      SortColPB := PBSortColumn;
      SortColIV := IVSortColumn;

      with frmMain do begin
        SetBounds(MainFormLeft, MainFormTop, MainFormWidth, MainFormHeight);
        Position := poDefaultSizeOnly;
        WindowState := MainFormState;
        CoolBar1.Bands[0].Width := 503;
        //the following keeps band[2] all the way to the left
        //immediately after band[1] whenever the form is resized.
        CoolBar1.Bands[1].Width := 1600;

//        ActionToolBar.AutoSizing := False; // rpk 8/3/2010 CQ 493
//        ActionToolBar1.AutoSizing := False; // rpk 8/3/2010 CQ 493

//        rePatientDemographics.SendToBack;  // rpk 10/1/2010
//        rePatientDemographics.TabStop := false;  // rpk 10/1/2010
//        rePatientDemographics.TabStop := ScreenReaderSystemActive; // rpk 10/1/2010

        repaint;
      end;
    end;

  Application.ProcessMessages;
  StatusBarTimer.enabled := True;
end; // FormShow

procedure TfrmMain.FormClick(Sender: TObject);
begin
//  ShowCtrlName('MainFormClick', ActiveControl);
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with BCMA_UserParameters do begin
    MainFormState := WindowState;

    with GetNormalRect(Self) do begin
      MainFormTop := Top;
      MainFormLeft := Left;
    end;

    if MainFormState = wsNormal then begin
      MainFormHeight := Height;
      MainFormWidth := Width;
    end;

    CurrentTab := lstCurrentTab;

    ContiniousChecked := cbxContinuous.checked;
    PRNChecked := cbxPRN.checked;
    OneTimeChecked := cbxOneTime.checked;
    OnCallChecked := cbxOnCall.checked;

    UDSortColumn := SortColUD;
    PBSortColumn := SortColPB;
    IVSortColumn := SortColIV;
  end;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  BCMA_ScannedDrug.free;
  VisibleMedList.free;
  BCMA_Patient.free;
  BCMA_SiteParameters.free;
  BCMA_UserParameters.free;
  BCMA_Report.free;
  BCMA_CoverSheet.free;

  BCMA_Broker.Free;
  CloseLogFile;
  VACCOW.Free;
end;

procedure TfrmMain.actionViewAllergiesExecute(Sender: TObject);
begin
  with BCMA_Report do begin
    Clear;
    ReportType := rtAllergyReactions;
    PatientIEN := BCMA_Patient.IEN;
    PatientWard := 'P';
    Execute;
  end;
end;

procedure TfrmMain.actionDueListUnableToScanExecute(Sender: TObject);
const
  UtSBagMessage =
    'There are no bags or no bags supported by the Unable to Scan feature for this order.';
var
  ii, y: integer;
  msg, tempUID, Result: string;
  Found: Boolean;
  BagList: TStringList;
  KeyStroke: Char;

  ResultTxt, CommentTxt: string;
  ReturnVal: Boolean;

  UnableToScanString: string;
  DispensedDrug: TBCMA_DispensedDrug;

  CurFlowUID: string; {JK 6/12/2008}
  InfusingBags: Boolean; {JK 6/12/2008}
  toBeWardStock: Pointer; {JK 6/12/2008}
  InfusableBagCount: Integer; {JK 6/16/2008}
  CheckInfusingBagsBailOut: Boolean; {JK 7/24/2008}

  AbbreviatedRoute: string; {JK 8/6/2008}

  PRNVitalsInfo: string; {JK 8/25/2008}
  PRNPainInfo: string; {JK 8/25/2008}

  OkToContinue: Boolean;

begin
  isUnableToScan := True; {JK 4/26/2008}
  Found := false;
  msg := '';
  lstUnitDose.Enabled := False;
  sgUnitDose.Enabled := False;
  BagList := TStringList.Create;
  DispensedDrug := nil; // rpk 4/15/2009

  PRNVitalsInfo := '';
  PRNPainInfo := '';

  try
    WorkFlowType := WF_UAS_Medication;

    if ScreenReaderSystemActive then begin
      with sgUnitDose do
        with TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row - 1]) do
          case OrderTypeID of

            {User selected Unable to Scan on a Unit Dose Order}
            otUnitDose: begin
                if OrderedDrugNames.count > 0 then begin
                  ScannedInput :=
                    TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.Row -
                    1]).OrderedDrugIENs[0];
                  UnableToScan := True;

                  if (OrderedDrugCount <> 1) or
                    (OrderedDrugs[0].QtyOrdered <> 1) or
                    (pos('.', OrderedDrugs[0].QtyOrderedText) <> 0) then begin

                    OkToContinue := isValidScannedDrug(ScannedInput, False,
                      False, '', '');
                    if not OkToContinue then
                      DefMessageDlg('Order Administration Cancelled!',
                        mtWarning,
                        [mbOK], 0);

                  end
                  else begin

                    {if a single-dose PRN med, call the PRN Effectiveness screen first}
                    OkToContinue := isValidScannedDrug(ScannedInput, False,
                      False, '', ''); {JK 6/27/2008}
                    if not OkToContinue then
                      DefMessageDlg('Order Administration Cancelled!',
                        mtWarning,
                        [mbOK], 0);
                  end;

                  UnableToScan := False;
                end
                else begin
                  msg := 'No Dispensed Drugs Found for this Order!';
                  DefMessageDlg(msg, mtInformation, [mbOk], 0);
                end;
              end;

            {User Selected Unable to Scan on an IV Order}
            otIV:
              case lstCurrentTab of
                {User selected unable to scan on an IV order that exists on the IVP/IVPB tab}
                ctPB: begin

                    {retreive all Unique IDs/bags that aren't linked to an admin}
                    if UniqueIDs.count > 0 then begin
                      for ii := 0 to UniqueIDs.count - 1 do begin
                        tempUID := piece(UniqueIDs[ii], '^', 1);
                        with BCMA_Patient do
                          for y := 0 to MedOrders.count - 1 do
                            with TBCMA_MedOrder(MedOrders[y]) do
                              if UniqueID = tempUID then begin
                                Found := True;
                                break;
                              end;
                        if (Found <> True) and (Pos('WS', tempUID) = 0) then
                          BagList.Add(tempUID);
                        Found := False;
                      end;

                      if BagList.Count = 0 then begin
                        msg := UtSBagMessage;
                        DefMessageDlg(msg, mtInformation, [mbOk], 0);
                        exit;
                      end;
                      BagList.Sort;

                      ScannedInput := Result;
                      UnableToScan := True;

                      //{if a single-dose PRN med, call the PRN Effectiveness screen first}
                      //  if TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).ScheduleTypeID = stPRN then
                      //    ProcessPRNS(TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]), isPRNCancelled, VitalsInfo, PainInfo);

                      fUnableToScan.UnableToScanExecute(1,
                        WF_UAS_MEDICATION,
                        BagList,
                        TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row -
                        1]),
                          ResultTxt,
                        CommentTxt,
                        ReturnVal,
                        PRNVitalsInfo,
                        PRNPainInfo,
                        nil);
                      if ReturnVal then begin

                        with TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row
                          - 1]) do
                          if OrderTypeID = otIV then

                            if lstCurrentTab = ctPB then begin

                              //-if Route = 'IV PUSH' then
                              //-  AbbreviatedRoute := 'IVP'
                              //-else if Route = 'IV PIGGYBACK' then
                              //-  AbbreviatedRoute := 'IVPB'
                              //-else
                              AbbreviatedRoute := '';

                              //-UnableToScanString := 'ID^' + AbbreviatedRoute + ' ' + ScannedInput
                              UnableToScanString := 'ID^' + ScannedInput
                            end
                            else
                              UnableToScanString := 'ID^' + ScannedInput
                          else begin
                            if DispensedDrug <> nil then
                              UnableToScanString := 'DD^' + DispensedDrug.IEN
                            else
                              UnableToScanString := 'DD^' + OrderedDrugs[0].IEN;
                          end;
                        BCMA_Common.LogUnableToScan(TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row -
                          1]).OrderNumber, ResultTxt, CommentTxt,
                            UnableToScanString, 'M');
                        UAS_LogState := LA_AlreadyLogged;

                        UnableToScan := False;
                      end;
                    end
                    else begin
                      msg := UtSBagMessage;
                      DefMessageDlg(msg, mtInformation, [mbOk], 0);
                    end;
                  end;

                {User selected Unable to Scan on IV order on the IV tab}
                ctIV: begin
                    with BCMA_Patient do
                      {If the user clicked on the treeview, then they selected a bag}
                      if (ActiveControl = fraIV1.tvwIVHistory) and (CurrentBagID
                        <> nil) then
                        Result := TBCMA_IVBags(CurrentBagID).UniqueID
                      else
                      {The user was not on the treeview, thus we need to prompt them to select a bag}begin
                        for y := 0 to IVBags.Count - 1 do
                          if (TBCMA_IVBags(IVBags[y]).ScanStatus <> 'C') and
                            (Pos('WS', TBCMA_IVBags(IVBags[y]).UniqueID) = 0)
                            then
                            BagList.Add(TBCMA_IVBags(IVBags[y]).UniqueID);

                        if BagList.Count = 0 then begin
                          msg := UtSBagMessage;
                          DefMessageDlg(msg, mtInformation, [mbOk], 0);
                          exit;
                        end;

                        BagList.Sort;
                      end;

                    {Is there a bag already infusing?}{JK 6/12/2008}
  //                  if CheckInfusingBags('', lstUnitDose, '', CurFlowUID, InfusingBags, CheckInfusingBagsBailOut) then begin
                    if CheckInfusingBags('', '', CurFlowUID, InfusingBags,
                      CheckInfusingBagsBailOut) then begin

                      if CurFlowUID <> '' then begin
                        ScannedInput := CurFlowUID;
                        toBeWardStock := nil;
                        if not ScanIV(ScannedInput, atScan, CurFlowUID,
                          toBeWardStock) then begin
                          DefMessageDlg('Order Administration Cancelled!',
                            mtWarning, [mbOK], 0);
                          UAS_LogState := LA_CANCELLED;
                          Exit;
                        end;
                      end;

                    end;

                    {JK - 7/24/2008}
                    if CheckInfusingBagsBailOut then begin
                      UAS_LogState := LA_CANCELLED;
                      Exit;
                    end;

                    {JK 6/16/2008 - Added InfusableBagCount to this logic to suppress
                     the UAS screen when there are no more available non-WS bags to hang.}
                    InfusableBagCount := 0;
                    with BCMA_Patient do
                      for y := 0 to IVBags.Count - 1 do
                        if (TBCMA_IVBags(IVBags[y]).ScanStatus <> 'C') and
                          (Pos('WS', TBCMA_IVBags(IVBags[y]).UniqueID) = 0) then
                          Inc(InfusableBagCount);

                    if InfusableBagCount > 0 then begin

                      UnableToScan := True;
                      fUnableToScan.UnableToScanExecute(1,
                        WF_UAS_MEDICATION,
                        nil,
                        TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row -
                        1]),
                          ResultTxt,
                        CommentTxt,
                        ReturnVal,
                        PRNVitalsInfo,
                        PRNPainInfo,
                        nil);
                      if ReturnVal then

                        {mimic keyboard input so we don't have to re-work a bunch of code }
                        with edtScannerInput do begin
                          KeyStroke := #13;
                          Text := '';
                          Text := ScannedInput;
                          OnKeyPress(nil, Keystroke);
                        end;
                      UnableToScan := False;
                    end; {if InfusableBagCount > 0}
                  end; {ctIV begin}
              end;
          end;
    end
    else begin // not in screen reader
      with lstUnitDose do
        with TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
          case OrderTypeID of

            {User selected Unable to Scan on a Unit Dose Order}
            otUnitDose: begin
                if OrderedDrugNames.count > 0 then begin
                  ScannedInput :=
                    TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).OrderedDrugIENs[0];
                  UnableToScan := True;

                  if (OrderedDrugCount <> 1) or
                    (OrderedDrugs[0].QtyOrdered <> 1) or
                    (pos('.', OrderedDrugs[0].QtyOrderedText) <> 0) then begin

                    OkToContinue := isValidScannedDrug(ScannedInput, False,
                      False, '', '');
                    if not OkToContinue then
                      DefMessageDlg('Order Administration Cancelled!',
                        mtWarning,
                        [mbOK], 0);

                  end
                  else begin

                    {if a single-dose PRN med, call the PRN Effectiveness screen first}
                    OkToContinue := isValidScannedDrug(ScannedInput, False,
                      False, '', ''); {JK 6/27/2008}
                    if not OkToContinue then
                      DefMessageDlg('Order Administration Cancelled!',
                        mtWarning,
                        [mbOK], 0);
                  end;

                  UnableToScan := False;
                end
                else begin
                  msg := 'No Dispensed Drugs Found for this Order!';
                  DefMessageDlg(msg, mtInformation, [mbOk], 0);
                end;
              end;

            {User Selected Unable to Scan on an IV Order}
            otIV:
              case lstCurrentTab of
                {User selected unable to scan on an IV order that exists on the IVP/IVPB tab}
                ctPB: begin

                    {retreive all Unique IDs/bags that aren't linked to an admin}
                    if UniqueIDs.count > 0 then begin
                      for ii := 0 to UniqueIDs.count - 1 do begin
                        tempUID := piece(UniqueIDs[ii], '^', 1);
                        with BCMA_Patient do
                          for y := 0 to MedOrders.count - 1 do
                            with TBCMA_MedOrder(MedOrders[y]) do
                              if UniqueID = tempUID then begin
                                Found := True;
                                break;
                              end;
                        if (Found <> True) and (Pos('WS', tempUID) = 0) then
                          BagList.Add(tempUID);
                        Found := False;
                      end;

                      if BagList.Count = 0 then begin
                        msg := UtSBagMessage;
                        DefMessageDlg(msg, mtInformation, [mbOk], 0);
                        exit;
                      end;
                      BagList.Sort;

                      ScannedInput := Result;
                      UnableToScan := True;

                      //{if a single-dose PRN med, call the PRN Effectiveness screen first}
                      //  if TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).ScheduleTypeID = stPRN then
                      //    ProcessPRNS(TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]), isPRNCancelled, VitalsInfo, PainInfo);

                      fUnableToScan.UnableToScanExecute(1,
                        WF_UAS_MEDICATION,
                        BagList,
                        TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]),
                        ResultTxt,
                        CommentTxt,
                        ReturnVal,
                        PRNVitalsInfo,
                        PRNPainInfo,
                        nil);
                      if ReturnVal then begin

                        with
                          TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
                          if OrderTypeID = otIV then

                            if lstCurrentTab = ctPB then begin

                              //-if Route = 'IV PUSH' then
                              //-  AbbreviatedRoute := 'IVP'
                              //-else if Route = 'IV PIGGYBACK' then
                              //-  AbbreviatedRoute := 'IVPB'
                              //-else
                              AbbreviatedRoute := '';

                              //-UnableToScanString := 'ID^' + AbbreviatedRoute + ' ' + ScannedInput
                              UnableToScanString := 'ID^' + ScannedInput
                            end
                            else
                              UnableToScanString := 'ID^' + ScannedInput
                          else begin
                            if DispensedDrug <> nil then
                              UnableToScanString := 'DD^' + DispensedDrug.IEN
                            else
                              UnableToScanString := 'DD^' + OrderedDrugs[0].IEN;
                          end;
                        BCMA_Common.LogUnableToScan(TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).OrderNumber, ResultTxt, CommentTxt,
                          UnableToScanString, 'M');
                        UAS_LogState := LA_AlreadyLogged;

                        UnableToScan := False;
                      end;
                    end
                    else begin
                      msg := UtSBagMessage;
                      DefMessageDlg(msg, mtInformation, [mbOk], 0);
                    end;
                  end;

                {User selected Unable to Scan on IV order on the IV tab}
                ctIV: begin
                    with BCMA_Patient do
                      {If the user clicked on the treeview, then they selected a bag}
                      if (ActiveControl = fraIV1.tvwIVHistory) and (CurrentBagID
                        <> nil) then
                        Result := TBCMA_IVBags(CurrentBagID).UniqueID
                      else
                      {The user was not on the treeview, thus we need to prompt them to select a bag}begin
                        for y := 0 to IVBags.Count - 1 do
                          if (TBCMA_IVBags(IVBags[y]).ScanStatus <> 'C') and
                            (Pos('WS', TBCMA_IVBags(IVBags[y]).UniqueID) = 0)
                            then
                            BagList.Add(TBCMA_IVBags(IVBags[y]).UniqueID);

                        if BagList.Count = 0 then begin
                          msg := UtSBagMessage;
                          DefMessageDlg(msg, mtInformation, [mbOk], 0);
                          exit;
                        end;

                        BagList.Sort;
                      end;

                    {Is there a bag already infusing?}{JK 6/12/2008}
  //                  if CheckInfusingBags('', lstUnitDose, '', CurFlowUID, InfusingBags, CheckInfusingBagsBailOut) then begin
                    if CheckInfusingBags('', '', CurFlowUID, InfusingBags,
                      CheckInfusingBagsBailOut) then begin

                      if CurFlowUID <> '' then begin
                        ScannedInput := CurFlowUID;
                        toBeWardStock := nil;
                        if not ScanIV(ScannedInput, atScan, CurFlowUID,
                          toBeWardStock) then begin
                          DefMessageDlg('Order Administration Cancelled!',
                            mtWarning, [mbOK], 0);
                          UAS_LogState := LA_CANCELLED;
                          Exit;
                        end;
                      end;

                    end;

                    {JK - 7/24/2008}
                    if CheckInfusingBagsBailOut then begin
                      UAS_LogState := LA_CANCELLED;
                      Exit;
                    end;

                    {JK 6/16/2008 - Added InfusableBagCount to this logic to suppress
                     the UAS screen when there are no more available non-WS bags to hang.}
                    InfusableBagCount := 0;
                    with BCMA_Patient do
                      for y := 0 to IVBags.Count - 1 do
                        if (TBCMA_IVBags(IVBags[y]).ScanStatus <> 'C') and
                          (Pos('WS', TBCMA_IVBags(IVBags[y]).UniqueID) = 0) then
                          Inc(InfusableBagCount);

                    if InfusableBagCount > 0 then begin

                      UnableToScan := True;
                      fUnableToScan.UnableToScanExecute(1,
                        WF_UAS_MEDICATION,
                        nil,
                        TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]),
                        ResultTxt,
                        CommentTxt,
                        ReturnVal,
                        PRNVitalsInfo,
                        PRNPainInfo,
                        nil);
                      if ReturnVal then

                        {mimic keyboard input so we don't have to re-work a bunch of code }
                        with edtScannerInput do begin
                          KeyStroke := #13;
                          Text := '';
                          Text := ScannedInput;
                          OnKeyPress(nil, Keystroke);
                        end;
                      UnableToScan := False;
                    end; {if InfusableBagCount > 0}
                  end; {ctIV begin}
              end;
          end;
    end; // not in screen reader

  finally
    if isUnableToScan then // rpk 12/2/2010
      isUnableToScan := False; {JK 7/25/2008 - CQ #148}
    if UnableToScan then // rpk 12/2/2010
      UnableToScan := False; // rpk 12/2/2010
    WorkFlowType := WF_Reset; {JK 9/18/2008}

    if ScreenReaderSystemActive then begin
      lstUnitDose.Enabled := False;
      sgUnitDose.Enabled := True;
//      sgClear(-1, -1, sgUnitDose);  // rpk 8/27/2010
//      sgUnitDose.Repaint; // rpk 8/26/2010
      // needed to maintain fixed column and row information
      // clear / repaint only restores the non-fixed rows, columns.
      RebuildVirtualDueList(False); // rpk 8/27/2010
    end
    else begin
      sgUnitDose.Enabled := False;
      lstUnitDose.Enabled := True;
    end;

    edtScannerInput.Enabled := True;
    ScannedInput := '';
    ScannerActivate;
  end;
end; // actionDueListUnableToScanExecute

//function TfrmMain.scanMultipleDoses(idxOrder, idxDrug: integer): boolean;
//var
//  AdministrationUnitNeeded, CheckState: Boolean;
//  x: integer;
//  ModalResult: Integer;
//
//  ResultTxt, CommentTxt: String;
//  ReturnVal: Boolean;
//  UnableToScanString: String;
//
//  function MarkUnableToScan(): Boolean;
//  var
//    x: integer;
//  begin
//    with TBCMA_MedOrder(VisibleMedList.items[idxOrder]) do
//      for x := 0 to OrderedDrugCount - 1 do
//        with OrderedDrugs[x] do
//        begin
//          QtyScanned := QtyOrdered;
//          if AdministrationUnitNeeded and (TrimLeft(QtyEntered) = '') then
//          begin
//            QtyEntered := inputPrompt(Name,
//              'Enter Quantity and Units (ie., 30 ml):',
//              '', 40, True, False, CheckState, '');
//            if TrimLeft(QtyEntered) = '' then exit;
//          end;
//        end;
//    result := True;
//  end;
//
//begin
//  {JK 8/14/2008 CQ #198 - moved this portion of the code to prompt for quantity ordered
//   after the UTS screen. See below the UTS call below.}
////  with TBCMA_MedOrder(VisibleMedList.items[idxOrder]) do
////    if (OrderedDrugCount = 1) and (StrToFloat(OrderedDrugs[0].QtyOrderedText) = 1) then begin
////      OrderedDrugs[0].QtyScanned := 1;
////      AdministrationUnitNeeded := (AdministrationUnit = '');
////      if AdministrationUnitNeeded then begin
////        OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
////          'Enter Quantity and Units (ie., 30 ml):',
////          '', 40, True, False, CheckState, '');
////        if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then exit;
////      end;
//
//  {Workflow for PRN Effectiveness changed in June 2008 after the usability
//   group determined that the single and multi-dose administration needed the
//   PRNEffectiveness screen to come up first in all situations. JK 6/30/2008}
//  if UnableToScan then begin
//
//    UnableToScanExecute(1,
//                        //WF_UAS_MEDICATION,
//                        WF_Normal_Single_UnitDose,
//                        nil,
//                        VisibleMedList.items[lstUnitDose.ItemIndex],
//                        ResultTxt,
//                        CommentTxt,
//                        ReturnVal,
//                        nil);
//    if ReturnVal then begin
//      {Log the UAS event}
//      UnableToScanString := 'DD^' + ScannedInput;
//      LogUnableToScan(TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).OrderNumber,
//                      ResultTxt, CommentTxt, UnableToScanString, 'M');
//      Result := True;
//    end;
//
//  end else
//    Result := True; {if UnableToScan}
//
//  {JK 8/14/2008 CQ #198 - moved this portion of the code to prompt for quantity ordered
//   after the UTS screen.}
//  if ReturnVal then
//    with TBCMA_MedOrder(VisibleMedList.items[idxOrder]) do
//      if (OrderedDrugCount = 1) and (StrToFloat(OrderedDrugs[0].QtyOrderedText) = 1) then begin
//        OrderedDrugs[0].QtyScanned := 1;
//        AdministrationUnitNeeded := (AdministrationUnit = '');
//        if AdministrationUnitNeeded then begin
//          OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
//            'Enter Quantity and Units (ie., 30 ml):',
//            '', 40, True, False, CheckState, '');
//          if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then exit;
//        end;
//
//  end
////    else if UnableToScan then
////      Result := MarkUnableToScan
//    {
//    begin
//      for x := 0 to OrderedDrugCount - 1 do
//      begin
//        OrderedDrugs[x].QtyScanned := OrderedDrugs[x].QtyOrdered;
//        if AdministrationUnitNeeded then
//        begin
//          OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
//            'Enter Quantity and Units (ie., 30 ml):',
//            '', 40, True, False, CheckState, '');
//          if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then exit;
//        end;
//      end;
//      result := True;
//    end
//}
//    else
//      with TfrmMultipleDrugs.create(application) do
//      try
//        MedOrder := VisibleMedList.items[idxOrder];
//        Drugidx := idxDrug;
//        ModalResult := ShowModal;
//        Result := (ModalResult = mrOK);
//        MedOrder.UserComments2 := mmoComments.Text;
//        if ModalResult = mrIgnore then
//          UnableToScan := True;
//      finally
//        Free;
//      end;
//end;

function TfrmMain.scanMultipleDoses(idxOrder, idxDrug: integer): boolean;
var
  AdministrationUnitNeeded, CheckState: Boolean;
  //  x: integer;
  //  ModalResult: Integer;

  ResultTxt, CommentTxt: string;
  ReturnVal: Boolean;
  UnableToScanString: string;

  PRNVitalsInfo: string; {JK 8/25/2008}
  PRNPainInfo: string; {JK 8/25/2008}

  //
  // function not called
  //
  //  function MarkUnableToScan(): Boolean;
  //  var
  //    x: integer;
  //  begin
  //    with TBCMA_MedOrder(VisibleMedList.items[idxOrder]) do
  //      for x := 0 to OrderedDrugCount - 1 do
  //        with OrderedDrugs[x] do begin
  //          QtyScanned := QtyOrdered;
  //          if AdministrationUnitNeeded and (TrimLeft(QtyEntered) = '') then begin
  //            QtyEntered := inputPrompt(Name,
  //              'Enter Quantity and Units (ie., 30 ml):',
  //              '', 40, True, False, CheckState, '');
  //            if TrimLeft(QtyEntered) = '' then
  //              Exit;
  //          end;
  //        end;
  //    Result := True;
  //  end;

begin
  // Fix for Remedy ticket 316010: Initialize Result to False.
  // This handles return on cancel exits.
  Result := False; // rpk 4/15/2009

  PRNVitalsInfo := '';
  PRNPainInfo := '';
  if idxOrder < 0 then begin
    ShowMessage('ScanMultipleDoses: idxOrder < 0');
    Exit;
  end;

  with TBCMA_MedOrder(VisibleMedList.items[idxOrder]) do

    if (OrderedDrugCount = 1) and (StrToFloat(OrderedDrugs[0].QtyOrderedText) =
      1) then begin
      OrderedDrugs[0].QtyScanned := 1;

      //      {JK 8/14/2008 CQ #198 - moved this portion of the code to prompt for quantity ordered
      //       after the UTS screen. See below the UTS call below.}
      //      AdministrationUnitNeeded := (AdministrationUnit = '');
      //      if AdministrationUnitNeeded then begin
      //        OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
      //          'Enter Quantity and Units (ie., 30 ml):',
      //          '', 40, True, False, CheckState, '');
      //        if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then
      //          Exit;
      //      end;

            {Workflow for PRN Effectiveness changed in June 2008 after the usability
             group determined that the single and multi-dose administration needed the
             PRNEffectiveness screen to come up first in all situations. JK 6/30/2008}
      if UnableToScan then begin

        UnableToScanExecute(1,
          //WF_UAS_MEDICATION,
          WF_Normal_Single_UnitDose,
          nil,
          //          VisibleMedList.items[lstUnitDose.ItemIndex],
          VisibleMedList.items[idxOrder], // rpk 8/25/2010
          ResultTxt,
          CommentTxt,
          ReturnVal,
          PRNVitalsInfo,
          PRNPainInfo,
          nil);
        if ReturnVal then begin
          {Log the UAS event}

          {JK 8/14/2008 CQ #198 - moved this portion of the code to prompt for quantity ordered
           after the UTS screen.}
          AdministrationUnitNeeded := (AdministrationUnit = '');
          if AdministrationUnitNeeded then begin
            OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
              'Enter Quantity and Units (ie., 30 ml):', '', 40, True, False,
              CheckState, '');
            if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then
              Exit;
          end;

          UnableToScanString := 'DD^' + ScannedInput;
          //          LogUnableToScan(TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]).OrderNumber,
          LogUnableToScan(TBCMA_MedOrder(VisibleMedList.items[idxOrder]).OrderNumber, // rpk 8/25/2010
            ResultTxt, CommentTxt, UnableToScanString, 'M');
          Result := True;
        end
        else
          Result := False; {JK 10/5/2008 CodeCR 249 - added Result := False}

      end // if unabletoscan
      else begin // not unabletoscan ?
        Result := True; {if UnableToScan}
        AdministrationUnitNeeded := (AdministrationUnit = '');
        if AdministrationUnitNeeded then begin
          OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
            'Enter Quantity and Units (ie., 30 ml):',
            '', 40, True, False, CheckState, '');
          if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then begin
            Result := False;
            Exit;
          end;
        end;
      end;
    end // quantity 1
    else begin
      //    else if UnableToScan then
      //      Result := MarkUnableToScan
          {
          begin
            for x := 0 to OrderedDrugCount - 1 do
            begin
              OrderedDrugs[x].QtyScanned := OrderedDrugs[x].QtyOrdered;
              if AdministrationUnitNeeded then
              begin
                OrderedDrugs[0].QtyEntered := inputPrompt(OrderedDrugs[0].Name,
                  'Enter Quantity and Units (ie., 30 ml):',
                  '', 40, True, False, CheckState, '');
                if TrimLeft(OrderedDrugs[0].QtyEntered) = '' then exit;
              end;
            end;
            result := True;
          end
      }
      with TfrmMultipleDrugs.Create(Application) do try
        MedOrder := VisibleMedList.Items[idxOrder];
        Drugidx := idxDrug;
        ModalResult := ShowModal;
        Result := (ModalResult = mrOK);
        MedOrder.UserComments2 := mmoComments.Text;
        if ModalResult = mrIgnore then
          UnableToScan := True;
      finally
        Free;
      end;

    end; // else multiple drugs
end; // scanMultipleDoses

//The following is in need of a serious re-write, way too long, too kludgy, been
//band-aide fixed/added to, one too many times...

function TfrmMain.isValidScannedDrug(ScannedDrugIEN: string; DeferPRNProcessing:
  Boolean; isPRNCancelled: Boolean; PRNVitalsInfo, PRNPainInfo: string): Boolean;
var
  idxOrder, idxDrug, y: integer;
  AdministrationUnitNeeded: boolean;
  CurFlowUID: string;
  aMedOrder: TBCMA_MedOrder;
  OrderList: TStringList;
  toBeWardStock: Pointer;
  aIVBag: TBCMA_IVBags; //bjr 8/3/10 for BCMA00000483
  i: Integer;
  DrugIEN, tempstr: string;

  function isVisibleOrder(DrugIEN, DrugName: string; var idxOrder, idxDrug:
    integer): boolean;
  var
    ii, jj: integer;
    ScannedOrderList: TStringList;
  begin
    idxOrder := -1;
    idxDrug := -1;

    ScannedOrderList := TStringList.Create;
    with ScannedOrderList do try
      with VisibleMedList do
        for ii := 0 to VisibleMedList.count - 1 do
          with TBCMA_MedOrder(VisibleMedList[ii]) do
            if (ScanStatus <> 'G') then
              if ScanStatus <> 'RM' then
                for jj := 0 to OrderedDrugIENs.count - 1 do
                  if (OrderedDrugIENs.strings[jj] = DrugIEN) then
                    ScannedOrderList.add(intToStr(ii) + ';' + intToStr(jj));

      case ScannedOrderList.count of
        0:
          DefMessageDlg('Scanned Drug Not Found in Virtual Due List or' + #13 +
            'It Has Already Been Given!' + #13#13 +
            DrugName, mtError, [mbOK], 0);

        1: begin
            idxOrder := strToInt(piece(ScannedOrderList[0], ';', 1));
            idxDrug := strToInt(piece(ScannedOrderList[0], ';', 2));
          end;

      else begin
          ii := SelectOrderID(ScannedOrderList, True);
          if ii > -1 then begin
            idxOrder := strToInt(piece(ScannedOrderList[ii], ';', 1));
            idxDrug := strToInt(piece(ScannedOrderList[ii], ';', 2));
          end;
        end;
      end;
    finally
      ScannedOrderList.free;
    end;
    result := (idxOrder > -1);
  end; (* isVisibleOrder *)

begin
  Result := False;
  idxOrder := -1; // rpk 8/25/2010
  toBeWardStock := nil;
  aMedOrder := nil;
  aIVBag := nil; // rpk 11/2/2010

  case lstCurrentTab of
    ctUD:
      with BCMA_ScannedDrug do begin
        if not isValidDrug(ScannedDrugIEN) then
          Exit;

        if UnableToScan then begin
          if ScreenReaderSystemActive then begin
            if sgUnitDose.Row > 0 then begin
              idxOrder := sgUnitDose.Row - 1;
              if (idxOrder < VisibleMedList.Count) then  // rpk 12/3/2010
                 aMedOrder := TBCMA_MedOrder(VisibleMedList.items[idxOrder])
              else
                aMedOrder := nil;  // rpk 12/3/2010
            end
          end
          else begin
            idxOrder := lstUnitDose.ItemIndex;
//            if (idxOrder > -1) then
//            if (idxOrder > -1) and (idxOrder <= VisibleMedList.Count) then // rpk 9/13/2010
            if (idxOrder > -1) and (idxOrder < VisibleMedList.Count) then // rpk 12/2/2010
              aMedOrder :=
                TBCMA_MedOrder(VisibleMedList.items[idxOrder])
            else
              aMedOrder := nil;
          end;

        end // unabletoscan
        else begin
          if not isVisibleOrder(IEN, Name, idxOrder, idxDrug) then
            Exit;

//          if idxOrder <= VisibleMedList.Count then
          if idxOrder < VisibleMedList.Count then // rpk 11/29/2010
            aMedOrder := TBCMA_MedOrder(VisibleMedList.items[idxOrder])
          else
            aMedOrder := nil;

          //
          // todo: Add String Grid Select
          //
          if ScreenReaderSystemActive then
            sgUnitDose.Row := idxOrder + 1 // rpk 9/7/2010
          else begin
            // unnecessary?
            for y := 0 to lstUnitDose.Count - 1 do
              lstUnitDose.selected[y] := false;
//            if (idxOrder > -1) then
//            if (idxOrder > -1) and (idxOrder <= lstUnitDose.Count) then // rpk 9/13/2010
            if (idxOrder > -1) and (idxOrder < lstUnitDose.Count) then begin // rpk 11/29/2010
              if lstUnitDose.ItemIndex <> idxOrder then // rpk 11/29/2010
                lstUnitDose.ItemIndex := idxOrder; // rpk 11/29/2010
              lstUnitDose.Selected[idxOrder] := True;
            end;
          end;

        end;
      end;

    ctPB: begin
        //with VisibleMedList do
        //  for y := 0 to VisibleMedList.count - 1 do
        if ScannedDrugIEN <> '' then
          with BCMA_Patient do
            for y := 0 to MedOrders.Count - 1 do
              with TBCMA_MedOrder(MedOrders[y]) do
                if UniqueID = ScannedDrugIEN then begin
                  DefMessageDlg('The scanned bag has already been given!',
                    mtError, [mbOK], 0);
                  exit;
                end;
        //        if UnableToScan then
        //        begin
        //          aMedOrder := TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]);
        //          idxOrder := lstUnitDose.ItemIndex;
        //          OrderList := TStringList.Create;
        //          OrderList.AddObject(IntToStr(idxOrder), VisibleMedList.items[lstUnitDose.ItemIndex]);
        //        end

        //        else

        if UnableToScan and (ScannedDrugIEN = '') then
          OrderList := nil
        else
          OrderList := GetIVOrders(ScannedDrugIEN);

        if HoldOrder = True then begin
          DefMessageDlg('Order is on Provider Hold!' + #13#13 +
            'DO NOT GIVE!', mtError, [mbOK], 0);
          HoldOrder := False;
          Exit
        end;

        if (OrderList <> nil) and (OrderList.Count > 0) then begin
          idxOrder := SelectOrderID(OrderList, True);
          if idxOrder = -1 then
            Exit;
          aMedOrder := TBCMA_MedOrder(VisibleMedList.Items[idxOrder]);
          if ScreenReaderSystemActive then
            sgUnitDose.Row := idxOrder + 1 // rpk 9/7/2010
          else begin
            for y := 0 to lstUnitDose.Items.Count - 1 do
              lstUnitDose.selected[y] := false;
            lstUnitDose.Selected[idxOrder] := True;
          end;

          tempstr := BCMA_Patient.LoadIVBags(aMedOrder.OrderNumber);
          if piece(tempstr, '^', 1) = '-1' then begin
            DefMessageDlg(piece(tempstr, '^', 2), mtError, [mbOK], 0);
          end
          else
            aIVBag := BCMA_Patient.GetIVBagFromUniqueID(ScannedDrugIEN);
        end
        else

          with BCMA_ScannedDrug do begin

            //The only way we should get here this far with an UnableToScan
            //of true should be when they selected unable to scan wardstock
            if UnableToScan and (ScannedDrugIEN = '') then begin
              CreateWardStock(ScannedDrugIEN, CurFlowUID,
                toBeWardStock);
              OrderList.Free;
              exit;

            end
            else if not isValidDrug(ScannedDrugIEN) then
              Exit

            else begin
              if piece(ResultString, '^', 1) = 'DD' then begin
                if not isVisibleOrder(IEN, Name, idxOrder, idxDrug) then
                  Exit;

                if ScreenReaderSystemActive then
                  sgUnitDose.Row := idxOrder + 1 // rpk 9/7/2010
                else begin
                  for y := 0 to lstUnitDose.Items.Count - 1 do
                    lstUnitDose.selected[y] := false;
                  aMedOrder :=
                    TBCMA_MedOrder(VisibleMedList.items[idxOrder]);
                  lstUnitDose.Selected[idxOrder] := True;

                end;
              end
              else begin
                CreateWardStock(ScannedDrugIEN, CurFlowUID, toBeWardStock);
                OrderList.Free;
                exit;

              end;

            end;
          end;

        OrderList.Free;
      end;
  end;

  if aMedOrder <> nil then
    with aMedOrder do begin
      if ScreenReaderSystemActive then begin
        if sgUnitDose.Row > 0 then
          idxOrder := sgUnitDose.Row - 1
      end
//      else
      else if (idxOrder <> lstUnitDose.ItemIndex) then // rpk 12/2/2010
        idxOrder := lstUnitDose.ItemIndex;

      UnknownMessageDisplayed := false;
      if OrderStatus = 'H' then
        DefMessageDlg('Order is on Provider Hold!' + #13#13 +
          'DO NOT GIVE!', mtError, [mbOK], 0)

      else if ValidOrder then begin
        if DisplayInstructions then begin
          //          if lstUnitDose.ItemIndex > -1 then
          if idxOrder > -1 then {// rpk 8/25/2010} begin
            // revert to simple message dialog ala PSB*3*28
            if DefMessageDlg(SpecialInstructions, mtInformation, [mbOK,
              mbCancel], 0) <> idOK then
              Exit;
            //            tbool := DisplaySOMemo(SpecialInstructions, PatientIEN, OrderNumber,
            //              OrderType, True); // rpk 10/8/2009
            //            if not tbool then // rpk 5/12/2009
            //              Exit;
          end;
        end;

        AdministrationUnitNeeded := (AdministrationUnit = '');
        case status of
          -1: begin {Do Not Give!}
              DefMessageDlg(StatusMessage, mtError, [mbOK], 0);
            end;

          0: begin {Do Give!}
              if lstCurrentTab = ctPB then begin
                if OrderTypeID = otUnitDose then
                  if not scanMultipleDoses(idxOrder, idxDrug) then begin
                    if AdministrationUnitNeeded then
                      ClearDispensedDrugsEnteredData;
                    //-if not UnableToScan then
                    if WorkFlowType <> WF_UAS_MEDICATION then
                      DefMessageDlg('Order Administration Cancelled!',
                        mtWarning,
                        [mbOK], 0); {JK 9/17/2008}
                    Exit;
                  end;

                Result := LogOrder(mtMedPass, 'G', aIVBag); //bjr 10/20/10 for BCMA00000571

                //--AddSecondComment;
                LogComments(MedLogIEN, AdditionalComments);
                //--Result := True;  {JK 5/21/2008}
                if InjectionSiteNeeded and (InjectionSite = '') then
                  Result := False;

              end // if lstCurrentTab = ctPB
              else if scanMultipleDoses(idxOrder, idxDrug) then begin
                Result := LogOrder(mtMedPass, 'G', nil);
                //--AddSecondComment;
                LogComments(MedLogIEN, AdditionalComments);
                //Result := True; {JK 5/21/2008}
                if InjectionSiteNeeded and (InjectionSite = '') then
                  Result := False;
              end
              else begin
                if AdministrationUnitNeeded then
                  ClearDispensedDrugsEnteredData;
                //-                  if WorkFlowType <> WF_UAS_Medication then
                //-                      DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK], 0); {JK 9/17/2008}
              end;
            end;

          1: begin {Do Give with Confirmation!}
              if lstCurrentTab = ctPB then begin

                //                if OrderTypeID = otUnitDose then
                                    {JK 8/14/2008 - removed this to because PRN did not pop up first}
                //                  if not scanMultipleDoses(idxOrder, idxDrug) then begin
                //                    if AdministrationUnitNeeded then
                //                      ClearDispensedDrugsEnteredData;
                //                    Exit;
                //                  end;
                //
                //                if ConfirmOrder(VisibleMedList.items[idxOrder]) then begin
                //                  Result := LogOrder(mtMedPass, 'G', nil);
                //                  AddSecondComment;
                //                  // Result := True; {JK 5/21/2008}
                //                  if InjectionSiteNeeded and (InjectionSite = '') then
                //                    Result := false
                //                end

                                {JK 8/14/2008 - added this to make PRN pop up first}
                                {JK 8/22/2008 - added DeferPRNProcessing to handle IVPB IV types}
                ConfirmOrder(VisibleMedList.items[idxOrder], DeferPRNProcessing,
                  isPRNCancelled, PRNVitalsInfo, PRNPainInfo);
                if not isPRNCancelled then begin

                  if OrderTypeID = otUnitDose then begin
                    if not scanMultipleDoses(idxOrder, idxDrug) then begin
                      if AdministrationUnitNeeded then
                        ClearDispensedDrugsEnteredData;
                      //-                       if WorkFlowType = WF_Reset then
                      //-                         DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK], 0); {JK 9/17/2008}
                      Exit;

                      {JK 8/15/2008 - Moved Vitals Call from ProcessPRNs to here}
                      if PRNPainInfo <> '' then
                        aMedOrder.AdditionalComments.Add(PRNPainInfo);
                    end;
                  end;

                  Result := LogOrder(mtMedPass, 'G', aIVBag);

                  {JK 8/15/2008 - Moved Vitals Server Call from ProcessPRNs to here}
                  if not isPRNCancelled then begin
                    if not SendVitals(PRNVitalsInfo) then
                      DefMessageDlg('Error writing Vitals information from PRN screen',
                        mtError, [mbOK], 0);
                    if PRNPainInfo <> '' then begin
                      aMedOrder.AdditionalComments.Add(PRNPainInfo);
                      LogComments(MedLogIEN, aMedOrder.AdditionalComments);
                    end;
                  end;

                  //Result := True;  {JK 5/21/2008}
                  if InjectionSiteNeeded and (InjectionSite = '') then
                    Result := False
                end
                else if AdministrationUnitNeeded then
                  ClearDispensedDrugsEnteredData;

                //-              end else if ConfirmOrder(VisibleMedList.items[idxOrder]) then

              end
              else begin {if lstCurrentTab <> ctPB}
                ConfirmOrder(VisibleMedList.items[idxOrder], False,
                  isPRNCancelled, PRNVitalsInfo, PRNPainInfo);
                if not isPRNCancelled then
                  if scanMultipleDoses(idxOrder, idxDrug) then begin

                    Result := LogOrder(mtMedPass, 'G', nil);
                    if PRNPainInfo <> '' then begin
                      aMedOrder.AdditionalComments.Add(PRNPainInfo);
                      LogComments(MedLogIEN, aMedOrder.AdditionalComments);
                    end;

                    {JK 8/15/2008 - Moved Vitals Call from ProcessPRNs to here}
                    if not SendVitals(PRNVitalsInfo) then
                      DefMessageDlg('Error writing Vitals information from PRN screen',
                        mtError, [mbOK], 0);

                    if InjectionSiteNeeded and (InjectionSite = '') then
                      Result := False
                  end
                  else if AdministrationUnitNeeded then
                    ClearDispensedDrugsEnteredData;
              end;
            end;

        else
          DefMessageDlg('Unknown Order Status Returned!' + #13#13 +
            'DO NOT GIVE!!', mtError, [mbOK], 0);
        end;

        if not result and (ForceRefresh = False) then begin
          ValidOrder := False;
          if WorkFlowType = WF_Reset then
            DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK],
              0);
          UAS_LogState := LA_CANCELLED;

          {Set AdministrationUnit back to previous value if user selects cancel.
          If we don't do this, and the user tries to re-select this order again,
          it would never reprompt the user for the number of units given.}
          ClearDispensedDrugsEnteredData;

        end
        else if result and (ForceRefresh = True) then begin
          ForceRefresh := False;
          ValidOrder := False;
          ClearDispensedDrugsEnteredData;
          RebuildVirtualDueList(True);
        end;
      end
      else if Status = -2 then
        ForceVDLRebuild
      else if Status = -10 then
        Exit
      else
        DefMessageDlg('Could NOT validate this order!' + #13#13 +
          '''' + StatusMessage + '''' + #13#13 +
          'DO NOT GIVE!', mtError, [mbOK], 0);
      ClearAdminInfo;
    end;

end; // isValidScannedDrug

procedure TfrmMain.edtScannerInputKeyPress(Sender: TObject; var Key: Char);
var
  CurFlowUID: string; // the unique ID of a currently infusing bag
  BagOld, BagNew: string; // track separate bag ids
  toBeWardStock, //pointer to an order that correlates to the wardstock bag the user is trying to give
    nilPointer: Pointer;
begin
  {disable Cut and Paste}

  if key = #22 then
    key := #0;

  if (edtScannerInput.Text <> '') or (UnableToScan and (edtScannerInput.Text =
    '')) then
    if key = chr(VK_RETURN) then begin
      StopKeyboardTimer;
      edtScannerInput.Enabled := False;
      ScannedInput := edtScannerInput.text;
      toBeWardStock := nil;

      {JK 7/12/2008 CQ Ticket # 116 and 118}
      if (WorkFlowType <> WF_TakeActionOnBag) and (WorkFlowType <>
        WF_UAS_CreateWardStock) then begin
        //Logic Added to Deal with Keyed Bar Codes
        if KeyedBarCode then begin
          LogUnableToScan('', '', '', '', 'M', 1);
          KeyedBarCode := False;
        end
        else if not UnableToScan then
          LogUnableToScan('', '', '', '', 'M', 2);
      end;

      case lstCurrentTab of
        ctUD:
          if not isUnableToScan then begin {JK 4/26/2008}
            if isValidScannedDrug(ScannedInput, True, False, '', '') then
              RebuildVirtualDueList(False);
          end; {if not isUnableToScan}

        ctPB:
          if not isUnableToScan or (WorkFlowType = WF_UAS_CreateWardStock) then begin {JK 4/26/2008}
            if WorkFlowType = WF_UAS_CreateWardStock then begin
              if isValidScannedDrug(ScannedInput, True, False, '', '') then
                RebuildVirtualDueList(False);
            end
            else begin
              if isValidScannedDrug(ScannedInput, False, False, '', '') then
                RebuildVirtualDueList(False);
            end;
          end; {if not UnableToScan}

        ctIV:
          {first scan}
          if not isUnableToScan or (WorkFlowType = WF_TakeActionOnBag)
            or (WorkFlowType = WF_UAS_CreateWardStock) then begin {JK 4/26/2008}
            BagNew := ScannedInput;
            if ScanIV(ScannedInput, atScan, CurFlowUID, toBeWardStock) then begin

              (* If a Unique ID (CurFlowUID) is returned, or a pointer to
                 an order (toBeWardStock) is returned, then the first scan encountered
                 a bag already infusing.

                 If CurFlowUID is not null, then this contains the unigue id of
                 the currently infusing bag.

                 If CurFlowUID is not null, and the first scan was a wardstock bag,
                 AND a bag is currently infusing, toBeWardStock will contain a pointer
                 to the order that matches the scanned additives and solutions.

                 ScannedInput will hold the Unique ID of the original scan, if it
                 wasn't a ward stock *)
              BagOld := CurFlowUID;
              if (CurFlowUID <> '') or (toBeWardStock <> nil) then begin

                {second scan, scan the currently infusing bag}
                if ScanIV(CurFlowUID, atScan, CurFlowUID, nilPointer) then

                  {afterwards, go back and scan the original bag via the Unique ID}
                  if toBeWardStock = nil then
                    ScanIV(ScannedInput, atScan, CurFlowUID, toBeWardStock)
                  else
                    ScanIV('', atScan, CurFlowUID, toBeWardStock);
              end; // else
              //frmMain.UAS_LogState := LA_Cancelled; {JK 5/16/2008}

              self.Repaint;
              RebuildVirtualDueList(False);
            end;
          end; {if not UnableToScan}
      end; // if case lstcurrenttab

      isUnableToScan := False; {JK 4/26/2008}
      //--xUnableToScan := False; {JK 5/21/2008}

      edtScannerInput.Enabled := True;
      edtScannerInput.text := '';
      ScannedInput := '';
      ScannerActivate;
      WorkFlowType := WF_Reset;
    end; // if return key
end; // edtScannerInputKeyPress

procedure TfrmMain.StatusBarTimerTimer(Sender: TObject);
var
  fVariance: real;
//  location: String;
begin
  if FirstPass = True then begin
    StatusBarTimer.Tag := StatusBarTimer.Tag + 1;
    if ((frmmain.visible = true) and (screen.activeform = frmmain)) then begin
      FirstPass := False;
      ActionJoinUseExistingContext.Execute;
      StatusBarTimer.Interval := 5000;
      if BCMA_Patient.IEN = '' then
        actionFileOpenPatient.Execute;
    end

      //Make sure we don't get stuck in an endless loop for some strange reason.
    else if ((FirstPass = True) and (StatusBarTimer.Tag = 3)) then begin
      FirstPass := False;
      StatusBarTimer.Interval := 5000;
    end
  end
  else begin
    if BCMA_SiteParameters = nil then
      fVariance := 0.0
    else
      fVariance := BCMA_SiteParameters.ServerClockVariance;

    StatusBar.Panels[3].text := 'Server Time: ' +
      FormatDateTime('ddddd hh:mm',
      now + fVariance);

  // DEBUG for identifying active form / control
  // display active control only in test mode.
{$IFDEF TEST_ON}
    ShowActiveCtrlName; // DEBUG: rpk 10/5/2010
{$ENDIF}

    CheckIdleTimeout;
  end;
end; // StatusBarTimerTimer

procedure TfrmMain.ToolsAppClick(Sender: TObject);
var
  AppString: string;
begin
  AppString := BCMA_SiteParameters.ToolsApps[TMenuItem(sender).tag];
  LaunchApplication(Copy(AppString, Pos('=', AppString) + 1,
    Length(AppString)));
end;

procedure TfrmMain.actionDueListAddCommentExecute(Sender: TObject);
begin

  //Add a comment to a bag
  if lstCurrentTab = ctIV then
    AddCommentIV
      //Add a comment to an administration
  else
    with lstUnitDose do
      AddComment(VisibleMedList.items[ItemIndex], nil);

  edtScannerInput.Enabled := True;
  ScannerActivate;
  //  edtScannerInput.SetFocus;
end;

procedure TfrmMain.actionDueListPRNEffectExecute(Sender: TObject);
//var
//  NewItems: TListItems;
//  NewItem: TListItem;
begin
//  if ScreenReaderSystemActive then begin
//    with sgUnitDose do begin
//      if Row > 0 then
//
//      frmPRNEffectiveness.Execute(VisibleMedList.items[lstUnitDose.ItemIndex]);
//    end;
//  end
//  else begin
  with lstUnitDose do
    frmPRNEffectiveness.Execute(VisibleMedList.items[lstUnitDose.ItemIndex]);
//  end;

  pnlMainForm.Repaint;
  BCMA_Patient.LoadPRNEffectiveness('');

  //  try
  //    if lvwReminders.Items[0] = nil then   //JK 2-7-2008
  //      lvwReminders.AddItem(IntToStr(BCMA_Patient.PRNEffectList.Count), nil);
  //    //-else begin
  //      lvwReminders.Items[0].Caption := IntToStr(BCMA_Patient.PRNEffectList.Count);
  //      lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
  //    //-end;
  //  except
  //    on E:Exception do
  //      Showmessage('Clinical Reminders error 2: msg = ' + E.Message);
  //  end;

  try
    if lvwReminders.Items[0] = nil then begin //JK 11/29/2007
      lvwReminders.AddItem(IntToStr(BCMA_Patient.PRNEffectList.Count), nil);
      //          NewItems := TListItems.Create(lvwReminders);
      //          NewItem := TListItem.Create(NewItems);
      //          lvwReminders.Items.Add;
      //          lvwReminders.Items[0].Caption := IntToStr(BCMA_Patient.PRNEffectList.Count);
      //          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
      lvwReminders.Items[0].SubItems.Add('PRN Effectiveness'); // rpk 7/22/2009
    end
    else begin
      lvwReminders.Items[0].Caption :=
        IntToStr(BCMA_Patient.PRNEffectList.Count); //JK 11/29/2007
      lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
    end;
  except
    on E: Exception do
      ;
  end;

end;

procedure TfrmMain.actionReportsMissedMedsExecute(Sender: TObject);
begin
  MissedMedicationsReport(BCMA_Patient.IEN);
end;

procedure TfrmMain.actionSortByActiveMedExecute(Sender: TObject);
//var
//  x: integer;
begin
  //  with ActionManager do
  //    for x := 0 to ActionCount - 1 do
  //      if Actions[x].Category = '&Sort By' then
  //      begin
  //        if (Sender as TAction).Tag = (Actions[x] as TAction).Tag then
  //          (Actions[x] as TAction).Checked := True
  //        else
  //          (Actions[x] as TAction).Checked := false;
  //      end;

  with sender as TAction do begin
    case lstCurrentTab of
      //    case pgctrlVirtualDueList.ActivePageIndex of
      ctUD:
        SortColUD := TVDLColumnTypes(tag);
      ctPB:
        case Tag of
          0:
            SortColPB := lstPBColumnTypes(0);
          1:
            SortColPB := lstPBColumnTypes(1);
          3:
            SortColPB := lstPBColumnTypes(2);
          6:
            SortColPB := lstPBColumnTypes(5);
          7:
            SortColPB := lstPBColumnTypes(6);
          8:
            SortColPB := lstPBColumnTypes(7);
          9:
            SortColPB := lstPBColumnTypes(3);
          10:
            SortColPB := lstPBColumnTypes(4);
        end;
      ctIV:
        case Tag of
          0:
            SortColIV := lstIVColumnTypes(0);
          1:
            SortColIV := lstIVColumnTypes(1);
          3:
            SortColIV := lstIVColumnTypes(2);
          6:
            SortColIV := lstIVColumnTypes(5);
          9:
            SortColIV := lstIVColumnTypes(3);
          10:
            SortColIV := lstIVColumnTypes(4);
          11:
            SortColIV := lstIVColumnTypes(6);
        end;

    end;
  end;
  RebuildVirtualDueList(False);
end;

procedure TfrmMain.actionDueListRefreshExecute(Sender: TObject);
begin
  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do begin
      RebuildMe;
    end
  else
    RebuildVirtualDueList(True);
end;

procedure TfrmMain.setDLMenus;
var
  //  x: integer;
  TempNode: TTreeNode;
begin
  if lstCurrentTab = ctIV then
    TempNode := fraIV1.tvwIVHistory.Selected;
end;

procedure TfrmMain.mnDueListClick(Sender: TObject);
begin
  setDLMenus;
end;

procedure TfrmMain.rePatientDemographicsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Shift = [ssLeft] then
    showPatientDemographics;
  ScannerActivate;
  //  if edtScannerInput.Enabled then
  //  begin
  //    edtScannerInput.SetFocus;
   // end;
end;

procedure TfrmMain.rePatientDemographicsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // For normal user, disable down arrow, right arrow, and page down.
  // Prevent user from moving to blank line below the five lines of text.
  // Previously, moving to the last blank line shifted the lines up and hid
  // the patient name.
  // For JAWS user, enable normal navigation to allow reading of text.
  if (not ScreenReaderSystemActive) and
    ((Key = VK_DOWN) or
    (Key = VK_RIGHT) or
    (Key = VK_NEXT) or
    (Key = VK_END)) then // rpk 10/2/2010
    Key := 0;
end;

procedure TfrmMain.rePatientDemographicsKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = chr(VK_RETURN) then
    showPatientDemographics;
end;

procedure TfrmMain.rePatientDemographicsEnter(Sender: TObject);
begin
  pnlPatientDemographics.BevelOuter := bvRaised;
  rePatientDemographics.Perform(WM_VSCROLL, SB_TOP, 0);
  rePatientDemographics.SelStart := 0;
end;

procedure TfrmMain.rePatientDemographicsExit(Sender: TObject);
begin
  rePatientDemographics.Perform(WM_VSCROLL, SB_TOP, 0);
  pnlPatientDemographics.BevelOuter := bvNone;
end;

procedure TFrmMain.AppDeactivate(Sender: TObject);
begin
  edtScannerInputExit(edtScannerInput);
  if (screen.ActiveForm <> nil) and (screen.ActiveForm <> frmMain) and
    (Assigned(screen.activeform.OnDeactivate)) then
    screen.ActiveForm.OnDeactivate(Application);
end;

procedure TfrmMain.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
//var
//  location: String;
begin
//  location := screen.ActiveForm.Name;
//  ShowCtrlName('OnIdle ' + location, screen.ActiveForm.ActiveControl);
//  ShowActiveCtrlName;  // DEBUG: rpk 10/5/2010
end;

procedure TFrmMain.AppActivate;
begin
  ScannerActivate;
  if (screen.ActiveForm <> nil) and (screen.ActiveForm <> frmMain) and
    //    (Assigned(screen.activeform.OnDeactivate)) then
  (Assigned(screen.activeform.OnActivate)) then {// rpk 6/4/2010} begin
    screen.ActiveForm.OnActivate(Application);
  end;
end;

function TfrmMain.GetRowHeight(CtrlCanvas: TCanvas; CtrlRect: TRect; rowidx:
  Integer): Integer; // rpk 8/5/2009
var
  OrderableItem,
    SpecialInstruct,
    DosageSchedule,
    MedRoute: string;
  CellHeight,
    DosageHeight,
    OrderedCount,
    SpecialInstructionsHeight,
    DrugsHeight,
    RouteHeight: Integer;
  ARect: TRect;
  iheight: Integer;

  procedure ResetRect;
  begin
    with ARect do begin
      Left := 0;
      Top := 0;
      Bottom := 0;
      case lstCurrentTab of
        ctUD:
          Right := hdrUnitDose.Sections[4].Width - 6;
        ctPB, ctIV:
          Right := hdrUnitDose.Sections[3].Width - 6;
      end;
    end;
  end; // ResetRect

begin
  Result := 0;

  if VisibleMedList.Count = 0 then
    Exit;
  with TBCMA_MedOrder(VisibleMedList[rowidx]) do begin
    OrderTooTall := False;
    OrderableItem := ActiveMedication;
    case OrderTypeID of
      otUnitDose:
        OrderedCount := OrderedDrugCount;
      otIV:
        OrderedCount := AdditiveCount + SolutionCount
    else
      OrderedCount := 1;
    end;
    SpecialInstruct := SpecialInstructions;
    DosageSchedule := Trim(Dosage) + ', ' + Schedule;
    MedRoute := Route;
  end;

  //    irow := rowidx + 1;
  //    ARect := lstUnitDose.ItemRect(rowidx);
  ARect := CtrlRect;
  ResetRect;
  CellHeight := DrawText(CtrlCanvas.Handle, PChar(OrderableItem),
    Length(OrderableItem),
    ARect, DT_END_ELLIPSIS or DT_NOPREFIX or DT_CALCRECT);

  ResetRect;
  ARect.Left := 7;
  DrugsHeight := DrawText(CtrlCanvas.Handle, PChar('Dispensed Drugs'),
    Length('Dispensed Drugs'), ARect, DT_END_ELLIPSIS or DT_NOPREFIX or
    DT_CALCRECT) *
    OrderedCount;
  CellHeight := CellHeight + DrugsHeight;

  ResetRect;
  with TBCMA_MedOrder(VisibleMedList[rowidx]) do
    if OrderTypeID = otUnitDose then
      ARect.Left := 7
    else
      ARect.Left := 14;
  CtrlCanvas.Font.Style := [fsBold];
  SpecialInstructionsHeight := DrawText(CtrlCanvas.Handle,
    PChar(SpecialInstruct),
    Length(SpecialInstruct), ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX
    or
    DT_EDITCONTROL or DT_WORD_ELLIPSIS);
  CellHeight := CellHeight + SpecialInstructionsHeight;
  CtrlCanvas.Font.Style := [];

  ResetRect;
  case lstCurrentTab of
    ctUD:
      ARect.Right := hdrUnitDose.Sections[5].Width - 6;
    ctPB, ctIV:
      ARect.Right := hdrUnitDose.Sections[4].Width - 6;
  end;
  DosageHeight := DrawText(CtrlCanvas.Handle, PChar(DosageSchedule),
    Length(DosageSchedule), ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX or
    DT_EDITCONTROL or DT_WORD_ELLIPSIS);

  ResetRect;
  case lstCurrentTab of
    ctUD:
      ARect.Right := hdrUnitDose.Sections[6].Width - 6;
    ctPB, ctIV:
      ARect.Right := hdrUnitDose.Sections[5].Width - 6;
  end;
  RouteHeight := DrawText(CtrlCanvas.Handle, PChar(MedRoute),
    Length(MedRoute), ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX or
    DT_EDITCONTROL or DT_WORD_ELLIPSIS);

  iHeight := Trunc(MaxValue([CellHeight, DosageHeight, RouteHeight])) + 2;

  outputDebugString(PChar(IntToStr(iHeight)));

  if iHeight > 255 then begin
    TBCMA_MedOrder(VisibleMedList[rowidx]).OrderTooTall := True;
    //      CellHeight := iHeight - DrugsHeight - SpecialInstructionsHeight;
          // change functionality to display ordered drugs above ErrRowTooTall
    CellHeight := Height - SpecialInstructionsHeight; // rpk 10/8/2009
    ResetRect;
    CtrlCanvas.Font.Style := [fsBold];
    ARect.Left := 7;
    CellHeight := CellHeight + DrawText(CtrlCanvas.Handle, PChar(ErrRowTooTall),
      Length(ErrRowTooTall), ARect, DT_WORDBREAK or DT_CALCRECT or
      DT_NOPREFIX or DT_EDITCONTROL or DT_WORD_ELLIPSIS);
    CtrlCanvas.Font.Style := [];
    iHeight := Max(CellHeight, DosageHeight);
  end;

  Result := iheight;

end; // GetRowHeight

procedure TfrmMain.lstUnitDoseMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
begin
  Height := GetRowHeight(lstUnitDose.Canvas, lstUnitDose.ItemRect(Index),
    Index);
end; // lstUnitDoseMeasureItem

procedure TfrmMain.lstUnitDoseDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
type
  TempArray = array of string;
var
  x,
    ii,
    CellRight,
    TempHeight,
    CurrentTop,
    TitleCount: integer;
  TextString: string;
  CurrentFontColor: TColor;
  ARect: TRect;
  outText: string;

begin
  if VisibleMedList.Count > 0 then
    with lstUnitDose do begin
      outputDebugString(PChar(IntToStr(Arect.Bottom - Arect.Top)));

      ARect := Rect;
      CurrentFontColor := Canvas.Font.Color;

      if (TBCMA_MedOrder(VisibleMedList[Index]).OrderStatus = 'H') and not
        (odSelected in State) then
        Canvas.Brush.Color := $00DDDDDD;

      with Canvas do begin
        FillRect(ARect);
        Pen.Color := clSilver;
        MoveTo(ARect.Left, ARect.Bottom - 1);
        LineTo(ARect.Right, ARect.Bottom - 1);
      end;

      CellRight := -3;
      case lstCurrentTab of
        //      case pgctrlVirtualDueList.ActivePageIndex of
        ctUD:
          TitleCount := Length(VDLColumnTitles);
        ctPB:
          TitleCount := Length(lstPBColumnTitles);
        ctIV:
          TitleCount := Length(lstIVColumnTitles);
      else
        TitleCount := Length(VDLColumnTitles);
      end;

      for x := 0 to TitleCount - 1 do begin
        CellRight := CellRight + hdrUnitDose.Sections[x].Width;
        Canvas.MoveTo(CellRight, ARect.Bottom - 1);
        Canvas.LineTo(CellRight, ARect.Top);
      end;

      for x := 0 to TitleCount - 1 do begin
        if x > 0 then
          ARect.Left := ARect.Right + 2
        else
          ARect.Left := 2;

        ARect.Right := ARect.Left + hdrUnitDose.Sections[x].Width - 6;

        TextString := '';
        OutText := '';
        with TBCMA_MedOrder(VisibleMedList[Index]) do
          case x of
            0: begin
                case lstCurrentTab of
                  //case pgctrlVirtualDueList.ActivePageIndex of
                  ctUD, ctPB:
                    OutText := ScanStatus;
                  ctIV:
                    OutText := GetOrderStatus(OrderStatus);
                end;
              end;
            1: begin
                OutText := VerifyNurse;
              end;
            2: begin
                case lstCurrentTab of
                  //case pgctrlVirtualDueList.ActivePageIndex of
                  ctUD:
                    OutText := SelfMed;
                  ctPB:
                    OutText := ScheduleType;
                  ctIV:
                    OutText := GetIVType(AdministrationUnit);
                end;
              end;
            3: begin
                case lstCurrentTab of
                  //case pgctrlVirtualDueList.ActivePageIndex of
                  ctUD:
                    OutText := ScheduleType;
                  ctPB, ctIV: begin
                      CurrentTop := ARect.Top;
                      TextString := ActiveMedication;
                      TempHeight := DrawText(Canvas.Handle,
                        PChar(TextString),
                        Length(TextString), ARect, DT_END_ELLIPSIS or
                        DT_NOPREFIX);
                      ARect.Left := ARect.Left + 7;

                      { if OrderTooTall then
                      begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsbold];
                        DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                          Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
                      end
                      else
                      begin
                        if OrderTypeID = otUnitDose then
                        begin
                          for ii := 0 to OrderedDrugCount - 1 do
                          begin
                            ARect.Top := ARect.Top + TempHeight;
                            TextString := OrderedDrugs[ii].Name;
                            DrawText(Canvas.Handle, PChar(TextString),
                              Length(TextString), ARect, DT_END_ELLIPSIS
                              or
                              DT_NOPREFIX);
                          end;
                        end
                        else
                        begin
                          for ii := 0 to AdditiveCount - 1 do
                          begin
                            ARect.Top := ARect.Top + TempHeight;
                            TextString := piece(Additives[ii], '^', 3) +
                              ' ' +
                              piece(Additives[ii], '^', 4) + ' ' +
                              piece(Additives[ii], '^', 5);
                            DrawText(Canvas.Handle, PChar(TextString),
                              Length(TextString), ARect, DT_END_ELLIPSIS
                              or
                              DT_NOPREFIX);
                          end;
                          ARect.Left := ARect.Left + 7;
                          for ii := 0 to SolutionCount - 1 do
                          begin
                            ARect.Top := ARect.Top + TempHeight;
                            TextString := piece(Solutions[ii], '^', 3) +
                              ' ' +
                              piece(Solutions[ii], '^', 4) + ' ' +
                              piece(Solutions[ii], '^', 5);
                            DrawText(Canvas.Handle, PChar(TextString),
                              Length(TextString), ARect, DT_END_ELLIPSIS
                              or
                              DT_NOPREFIX);
                          end
                        end;
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsbold];
//                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
//                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
//                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
//                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
//                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
//                          DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_EDITCONTROL);  // rpk 4/15/2009
                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS);  // rpk 4/29/2009
                      end; }// else order not too tall

                      // change functionality to always display ordered drugs,
                      // additives, solutions // rpk 10/4/2009

                      if OrderTypeID = otUnitDose then begin
                        for ii := 0 to OrderedDrugCount - 1 do begin
                          ARect.Top := ARect.Top + TempHeight;
                          TextString := OrderedDrugs[ii].Name;
                          TempHeight := DrawText(Canvas.Handle,
                            PChar(TextString),
                            Length(TextString), ARect, DT_END_ELLIPSIS
                            or DT_NOPREFIX);
                        end;
                      end
                      else begin
                        for ii := 0 to AdditiveCount - 1 do begin
                          ARect.Top := ARect.Top + TempHeight;
                          TextString := piece(Additives[ii], '^', 3) +
                            ' ' +
                            piece(Additives[ii], '^', 4) + ' ' +
                            piece(Additives[ii], '^', 5);
                          TempHeight := DrawText(Canvas.Handle,
                            PChar(TextString),
                            Length(TextString), ARect, DT_END_ELLIPSIS
                            or DT_NOPREFIX);
                        end;
                        ARect.Left := ARect.Left + 7;
                        for ii := 0 to SolutionCount - 1 do begin
                          ARect.Top := ARect.Top + TempHeight;
                          TextString := piece(Solutions[ii], '^', 3) +
                            ' ' +
                            piece(Solutions[ii], '^', 4) + ' ' +
                            piece(Solutions[ii], '^', 5);
                          TempHeight := DrawText(Canvas.Handle,
                            PChar(TextString),
                            Length(TextString), ARect, DT_END_ELLIPSIS
                            or DT_NOPREFIX);
                        end
                      end;

                      if OrderTooTall then begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsbold];
                        DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                          Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
                      end
                      else begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsbold];
                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS);
                        // rpk 4/29/2009
                      end; // else not OrderTooTall

                      ARect.Top := CurrentTop;
                    end;
                end;
              end;
            4: begin
                case lstCurrentTab of
                  //case pgctrlVirtualDueList.ActivePageIndex of
                  ctUD: begin
                      CurrentTop := ARect.Top;
                      TextString := ActiveMedication;
                      TempHeight := DrawText(Canvas.Handle,
                        PChar(TextString),
                        Length(TextString), ARect, DT_END_ELLIPSIS or
                        DT_NOPREFIX);
                      ARect.Left := ARect.Left + 7;

                      { if OrderTooTall then
                      begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsBold];
                        DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                          Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
                      end
                      else
                      begin
                        for ii := 0 to OrderedDrugCount - 1 do
                        begin
                          ARect.Top := ARect.Top + TempHeight;
                          TextString := OrderedDrugs[ii].Name;
                          DrawText(Canvas.Handle, PChar(TextString),
                            Length(TextString), ARect, DT_END_ELLIPSIS or
                            DT_NOPREFIX);
                        end;
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsBold];
//                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
//                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
//                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
//                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
//                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
//                          DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_EDITCONTROL);  // rpk 4/13/2009
                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
//                          DT_NOPREFIX or DT_END_ELLIPSIS or DT_EDITCONTROL);  // rpk 4/29/2009
                          DT_NOPREFIX or DT_END_ELLIPSIS or DT_WORD_ELLIPSIS);  // rpk 4/29/2009
                      end; }// else not OrderTooTall

                      for ii := 0 to OrderedDrugCount - 1 do begin
                        ARect.Top := ARect.Top + TempHeight;
                        TextString := OrderedDrugs[ii].Name;
                        TempHeight := DrawText(Canvas.Handle, PChar(TextString),
                          // rpk 10/2/2009
                          Length(TextString), ARect, DT_END_ELLIPSIS or
                          DT_NOPREFIX);
                      end;

                      if OrderTooTall then begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsBold];
                        DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                          Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_WORD_ELLIPSIS);
                      end
                      else begin
                        ARect.Top := ARect.Top + TempHeight;
                        Canvas.Font.Color := clRed;
                        Canvas.Font.Style := [fsBold];
                        DrawText(Canvas.Handle, PChar(SpecialInstructions),
                          Length(SpecialInstructions), ARect, DT_WORDBREAK or
                          DT_NOPREFIX or DT_END_ELLIPSIS or DT_WORD_ELLIPSIS);
                        // rpk 4/29/2009
                      end; // else not order too tall

                      ARect.Top := CurrentTop;
                    end;

                  ctPB, ctIV: begin
                      TextString := Trim(Dosage) + ', ' + Schedule;
                      DrawText(Canvas.Handle, PChar(TextString),
                        Length(TextString), ARect, DT_WORDBREAK or
                        DT_NOPREFIX or DT_WORD_ELLIPSIS);
                    end;
                end;
              end;

            5: begin
                case lstCurrentTab of
                  //case pgctrlVirtualDueList.ActivePageIndex of
                  ctUD: begin
                      TextString := Trim(Dosage) + ', ' + Schedule;
                      DrawText(Canvas.Handle, PChar(TextString),
                        Length(TextString), ARect, DT_WORDBREAK or
                        DT_NOPREFIX or DT_WORD_ELLIPSIS);
                    end;
                  ctPB, ctIV:
                    DrawText(Canvas.Handle, PChar(Route),
                      Length(Route), ARect, DT_WORDBREAK or DT_WORD_ELLIPSIS
                      or DT_NOPREFIX);
                end;
              end;
            6:
              case lstCurrentTab of
                //case pgctrlVirtualDueList.ActivePageIndex of
                ctUD:
                  DrawText(Canvas.Handle, PChar(Route),
                    Length(Route), ARect, DT_WORDBREAK or DT_WORD_ELLIPSIS
                    or DT_NOPREFIX);
                ctPB:
                  OutText := DisplayVADate(AdministrationTime);
                ctIV: begin
                    if LastActivityStatus = '' then
                      TempHeight := DrawText(Canvas.Handle, PChar(''),
                        Length(''), ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                    else
                      TempHeight := DrawText(Canvas.Handle,
                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                        DisplayVADateYearTime(TimeLastAction)),
                        Length(GetLastActivityStatus(LastActivityStatus) + ': '
                        + DisplayVADateYearTime(TimeLastAction)), ARect,
                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    ARect.Top := ARect.Top + TempHeight;
                    if OrderChangedData.Count > 0 then begin
                      OutText :=
                        GetLastActivityStatus(piece(OrderChangedData[0],
                        '^', 4));
                      OutText := OutText + ' bag on changed order';
                    end
                    else
                      OutText := '';

                    //OutText := GetLastActivityStatus(LastActivityStatus);
                  end;
              end;
            7:
              case lstCurrentTab of
                //case pgctrlVirtualDueList.ActivePageIndex of
                ctUD:
                  if ScheduleType = 'C' then
                    OutText := DisplayVADate(AdministrationTime);
                ctPB: begin
                    if LastActivityStatus = '' then
                      TempHeight := DrawText(Canvas.Handle, PChar(''),
                        Length(''), ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                    else
                      TempHeight := DrawText(Canvas.Handle,
                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                        DisplayVADateYearTime(TimeLastAction)),
                        Length(GetLastActivityStatus(LastActivityStatus) + ': '
                        + DisplayVADateYearTime(TimeLastAction)), ARect,
                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    ARect.Top := ARect.Top + TempHeight;
                    if OrderTransferred = '1' then
                      OutText := BCMA_Patient.TransferredTransactionType
                    else
                      OutText := '';
                  end;
              end;
            8:
              case lstCurrentTab of
                //case pgctrlVirtualDueList.ActivePageIndex of
                ctUD: begin
                    if LastActivityStatus = '' then
                      TempHeight := DrawText(Canvas.Handle, PChar(''),
                        Length(''), ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                    else
                      TempHeight := DrawText(Canvas.Handle,
                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                        DisplayVADateYearTime(TimeLastAction)),
                        Length(GetLastActivityStatus(LastActivityStatus) + ': '
                        + DisplayVADateYearTime(TimeLastAction)), ARect,
                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    ARect.Top := ARect.Top + TempHeight;
                    if OrderTransferred = '1' then
                      OutText := BCMA_Patient.TransferredTransactionType
                    else
                      OutText := '';
                  end;
              end;
          end;

        case lstCurrentTab of
          //case pgctrlVirtualDueList.ActivePageIndex of
          ctUD:
            case x of
              0, 1, 2, 3, 6, 7, 8:
                DrawText(Canvas.Handle, PChar(OutText), Length(OutText),
                  ARect,
                  DT_END_ELLIPSIS or DT_NOPREFIX);
            end;
          ctPB, ctIV:
            case x of
              0, 1, 2, 5, 6, 7:
                DrawText(Canvas.Handle, PChar(OutText), Length(OutText),
                  ARect,
                  DT_END_ELLIPSIS or DT_NOPREFIX);
            end;
        end;
        Canvas.Font.Color := CurrentFontColor;
        Canvas.Font.Style := [];
        ARect.Right := ARect.Right + 4;
      end;
    end;
end;

procedure TfrmMain.lstUnitDoseEnter(Sender: TObject);
begin
  with lstUnitDose do begin
    if (ItemIndex < 0) and (Items.Count > 0) then
      ItemIndex := 0;
  end;
end;

procedure TfrmMain.hdrUnitDoseSectionResize(HeaderControl: THeaderControl;
  Section: THeaderSection);
var
  ii,
    TotalWidth,
    TempWidth: Integer;

begin
  RedrawSuspend(hdrUnitDose.handle);

  //set the appropriate section in the array to the new size
  case lstCurrentTab of
    //  case pgctrlVirtualDueList.ActivePageIndex of
    ctUD:
      VDLColumnWidths[TVDLColumnTypes(section.index)] :=
        hdrUnitDose.Sections.Items[section.index].Width;
    ctPB:
      lstPBColumnWidths[lstPBColumnTypes(section.Index)] :=
        hdrUnitDose.Sections.Items[section.Index].Width;
    ctIV:
      lstIVColumnWidths[lstIVColumnTypes(section.Index)] :=
        hdrUnitDose.Sections.Items[section.Index].Width;
  end;

  //set changed to true so the new values are saved when the user exits
  BCMA_UserParameters.Changed := True;

  {Set the maxwidth so columns can't be scrolled off window; same as in RebuildVirtualDueList?}
  TotalWidth := 0;

  with hdrUnitDose.Sections do begin
    for ii := 0 to Count - 1 do begin
      TempWidth := ((Count - (ii + 1)) * 5);
      items[ii].maxwidth := hdrUnitDose.width - (TempWidth + TotalWidth);
      TotalWidth := TotalWidth + Items[ii].Width;
    end;
  end;

  ReDrawActivate(hdrUnitDose.handle);
end;

procedure TfrmMain.lstUnitDoseDblClick(Sender: TObject);
begin
  with lstUnitDose do begin
    Enabled := False;
    with TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
      DisplayOrder(PatientIEN, OrderNumber);
    Enabled := True;
  end;
  ScannerActivate
end;

procedure TfrmMain.hdrUnitDoseMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  RedrawSuspend(pgctrlVirtualDueList.handle);
  RebuildVirtualDueList(False);
  RedrawActivate(pgctrlvirtualduelist.handle);
  lstUnitDose.Repaint;
end;

procedure TfrmMain.pgctrlVirtualDueListChange(Sender: TObject);
var
  HeaderSection: THeaderSection;
  x: integer;
  tempWidth, totalWidth: Integer;

begin
  with lstUnitDose do begin
    ReDrawSuspend(Handle);
    Clear;
    Visible := False;
    MultiSelect := True;
  end;

  with sgUnitDose do begin
    ReDrawSuspend(Handle);
    sgFree(-1, -1, sgUnitDose);
    Visible := False;
  end;

  if lstCurrentTab = ctCS then begin
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      CloseMe;
  end;

  with pgctrlVirtualDueList do
    case ActivePageIndex of
      0: {// Cover Sheet} begin
          //msf disable
{$IFDEF MSF_ON}
          btnEnableScanner.Enabled := True;
{$ENDIF}

          lstCurrentTab := ctCS;
          pnlScannerInput.Enabled := False;
          //msf disable - what's with this anyway !?
{$IFDEF MSF_ON}
          btnEnableScanner.Enabled := false;
{$ENDIF}
          // Coversheet Features Help
          tbshtCoverSheet.helpcontext := 904; // rpk 9/15/2010
          edtScannerInput.HelpContext := 904; // rpk 9/15/2010
          frmMain.HelpContext := 904; // rpk 9/15/2010

          for x := 0 to GroupBox1.ControlCount - 1 do
            GroupBox1.Controls[x].Enabled := false;
          for x := 0 to GroupBox2.ControlCount - 1 do
            GroupBox2.Controls[x].Enabled := False;

          shpContinuous.Brush.Color := clWhite;
          shpPRN.Brush.color := clWhite;
          shpOneTime.Brush.Color := clWhite;
          shpOnCall.Brush.Color := clWhite;

          lblCoverSheetLoad.Repaint;
          with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do begin
            CreateMe(tbshtCoverSheet);
          end;
        end;
      1: {// Unit Dose} begin
          lstCurrentTab := ctUD;
          tbshtUnitDose.helpcontext := 7;
          edtScannerInput.HelpContext := 7; // rpk 8/24/2010
          frmMain.HelpContext := 3; // rpk 9/16/2010

          if ReadOnly = false then
            pnlScannerInput.Enabled := true;
          if ScreenReaderSystemActive then begin
            sgUnitDose.Parent := ActivePage;
            sgUnitDose.Align := alClient;
          end
          else begin
            hdrUnitDose.Parent := ActivePage;
            lstUnitDose.Parent := ActivePage;
            lstUnitDose.Align := alClient;
          end;
          //          lblVDLUnitDose.Parent := ActivePage;
          stVDLUnitDose.Parent := ActivePage;
          pgctrlVirtualDueList.Repaint;
          RebuildVirtualDueList(True);
          UDViewed := True;
        end;
      2: begin // IVP/IVPB
          lstCurrentTab := ctPB;
          tbshtIVPIVPB.helpcontext := 8;
          edtScannerInput.HelpContext := 8; // rpk 8/24/2010
          frmMain.HelpContext := 3; // rpk 9/16/2010

          if ReadOnly = false then
            pnlScannerInput.Enabled := true;
          if ScreenReaderSystemActive then begin
            sgUnitDose.Parent := ActivePage;
            sgUnitDose.Align := alClient;
          end
          else begin
            hdrUnitDose.Parent := ActivePage;
            lstUnitDose.Parent := ActivePage;
            lstUnitDose.Align := alClient;
          end;
          //          lblVDLUnitDose.Parent := ActivePage;
          stVDLUnitDose.Parent := ActivePage;
          pgctrlVirtualDueList.Repaint;
          RebuildVirtualDueList(True);
          PBViewed := True;
        end;
      3: begin // IV
          lstCurrentTab := ctIV;
          tbshtIV.HelpContext := 9;
          edtScannerInput.HelpContext := 9; // rpk 8/24/2010
          frmMain.HelpContext := 3; // rpk 9/16/2010
          if ReadOnly = false then
            pnlScannerInput.Enabled := true;
          if ScreenReaderSystemActive then begin
            sgUnitDose.Parent := pnlIV;
            sgUnitDose.Align := alClient;
            sgUnitDose.TabStop := True; // rpk 9/1/2010
            lstUnitDose.TabStop := False; // rpk 9/1/2010
          end
          else begin
            hdrUnitDose.Parent := pnlIV;
            lstUnitDose.Parent := pnlIV;
            lstUnitDose.Align := alClient;
            lstUnitDose.MultiSelect := False;
            sgUnitDose.TabStop := False; // rpk 9/1/2010
            lstUnitDose.TabStop := True; // rpk 9/1/2010
          end;
          pnlIV.Height := trunc(tbshtIV.Height / 2);

          TotalWidth := 0;
          with fraIV1 do begin
            sgIVBagDetail.ColCount := length(lstBagDetailColumnTitles) + 1;

            hdrIVBagDetail.sections.clear;
            for x := 0 to length(lstBagDetailColumnTitles) - 1 do begin
              TempWidth := ((length(lstBagDetailColumnTitles) - (x + 1)) *
                5);
              HeaderSection := hdrIVBagDetail.Sections.Add;
              HeaderSection.Text :=
                lstBagDetailColumnTitles[lstBagDetailColumnTypes(x)];
              hdrIVBagDetail.Sections.Items[x].Width :=
                lstBagDetailColumnWidths[lstBagDetailColumnTypes(x)];
              hdrIVBagDetail.Sections.Items[x].maxwidth :=
                fraIV1.hdrIVBagDetail.width - (TempWidth + TotalWidth);
              hdrIVBagDetail.Sections.Items[x].minwidth := 7;
              TotalWidth := TotalWidth +
                hdrIVBagDetail.Sections.Items[x].Width;
              sgIVBagDetail.Cells[x, 0] :=
                lstBagDetailColumnTitles[lstBagDetailColumnTypes(x)];
              // rpk 10/15/2009
            end;
          end;
          pgctrlVirtualDueList.Repaint;
          RebuildVirtualDueList(True);
          IVViewed := True;
        end; // 3
    end; // case ActivePageIndex

//  ShowCtrlName('pgctrlVDLchg', ActiveControl);
//  Tried RefreshComponents per Brian J. Does not help JAWS read IVP/IVPB tab
//  after switching tabs.
//  VA508AccessibilityManager1.RefreshComponents;  // rpk 9/28/2010

end; // pgctrlVirtualDueListChange

procedure TfrmMain.hdrUnitDoseSectionClick(HeaderControl: THeaderControl;
  Section: THeaderSection);
begin
  if Section <> nil then begin

    case lstCurrentTab of
      //case pgctrlVirtualDueList.ActivePageIndex of
      ctUD:
        SortColUD := TVDLColumnTypes(Section.Index);
      ctPB:
        SortColPB := lstPBColumnTypes(Section.Index);
      ctIV:
        SortColIV := lstIVColumnTypes(Section.Index);
    end;

    if SortType = stAscending then
      SortType := stDescending
    else if SortType = stDescending then
      SortType := stAscending;

  end;
end;

procedure TfrmMain.ShowHintProc(var HintStr: string;
  var CanShow: Boolean; var HintInfo: THintInfo);
begin
  exit;
  //  beep;
  //application.cancelhint;
  if HintInfo.HintControl = lstUnitDose then
    //If CurrentItem <> lstUnitDose.ItemAtPos(HintInfo.CursorPos,True) Then
    with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemAtPos(HintInfo.CursorPos,
        True)]) do begin

      Application.hidehint;
      CurrentItem := lstUnitDose.ItemAtPos(HintInfo.CursorPos, True);
      hintstr := activemedication;
      //application.ShowHint := true;
      //showhint := true;
      //application
    end;
end;

procedure TfrmMain.lstUnitDoseMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  APoint: TPoint;
begin
  if Button = mbRight then begin

    if lstUnitDose.SelCount < 2 then begin
      APoint.x := X;
      APoint.y := Y;
      if lstUnitDose.ItemAtPos(APoint, True) <> -1 then
        with lstUnitDose do begin
          if Multiselect = false then begin
            ItemIndex := ItemAtPos(APoint, True);
            lstUnitDoseClick(lstUnitDose);
          end
          else begin
            Selected[ItemIndex] := False;
            Selected[ItemAtPos(APoint, True)] := True;
          end;
        end;
    end;
  end;
end;

procedure TfrmMain.actionMarkUndoExecute(Sender: TObject);
begin
  MarkNotGiven('N');
end;

{Added by JK, 4/26/2008 to support a hanging bag if it doesn't scan}

procedure TfrmMain.mnuPopUpUnableToScanClick(Sender: TObject);
var
  x: char;
begin
  if CurrentBagID = nil then // rpk 10/25/2010
    Exit;
  ScannedInput := TBCMA_IVBags(CurrentBagID).UniqueID;
  edtScannerInput.Clear;
  edtScannerInput.Text := TBCMA_IVBags(CurrentBagID).UniqueID;
  x := chr(VK_RETURN);
  edtScannerInputKeyPress(edtScannerInput, x);
end;

procedure TfrmMain.mnuRemovedClick(Sender: TObject);
begin
  MarkNotGiven('RM');
end;

procedure TfrmMain.sbrVDLScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
  lstUnitDose.TopIndex := ScrollPos;
end;

procedure TfrmMain.lstUnitDoseCreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
    Style := Style and not (WS_HSCROLL or WS_VSCROLL);
end;

procedure TfrmMain.ScannerActivate();
begin
  if frmMain.Active and pnlScannerInput.Enabled and pnlMainForm.Visible and
    edtScannerInput.Enabled then begin
    stScannerStatusReady.Caption := 'Ready'; // rpk 8/18/2010
    if stScannerStatusReady.Canfocus then // rpk 9/8/2010
      stScannerStatusReady.SetFocus; // rpk 8/18/2010
    edtScannerInput.Clear;
    edtScannerInput.setfocus;
    edtScannerInputEnter(edtScannerInput);
  end;
end;

procedure TfrmMain.lstUnitDoseMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  setDLMenus;
  ScannerActivate;
end;

procedure TfrmMain.lstUnitDoseContextPopup(Sender: TObject;
  MousePos: TPoint; var Handled: Boolean);
var
  x: integer;
begin
  with lstUnitDose do
    if SelCount = 1 then begin
      ReDrawSuspend(lstUnitDose.Handle);
      // if not invoked by a mouse click position is -1, -1; keep selected row
      if (MousePos.X <> -1) and (MousePos.Y <> -1) then begin // rpk added 7/16/2009
        for x := 0 to lstUnitDose.Items.Count - 1 do
          selected[x] := false;
      end;
      if ItemAtPos(MousePos, true) <> -1 then
        Selected[ItemAtPos(MousePos, False)] := True;
      ReDrawActivate(lstUnitDose.Handle);
    end;
  case lstCurrentTab of
    ctUD, ctPB: begin
        AddComment1.Visible := True;
        //        PopUpAvailableBags.Visible := True;
                //PopUpDrugIENCode.Visible := True;

        mnuPopUpMark.Visible := True;
        MissingDose1.Visible := True;
        PRNEffectiveness1.Visible := True;
        mnuN1.Visible := True;
        mnuN2.Visible := True;
      end;
    ctIV: begin
        mnuN1.Visible := False;
        mnuN2.Visible := False;

        AddComment1.Visible := False;
        //        PopUpAvailableBags.Visible := False;
                //PopUpDrugIENCode.Visible := False;
        mnuPopUpMark.Visible := False;
        MissingDose1.Visible := False;
        PRNEffectiveness1.Visible := False;
      end;
  end;

end;

procedure TfrmMain.lstUnitDoseKeyPress(Sender: TObject; var Key: Char);
begin
  if key = chr(VK_RETURN) then
    lstUnitDoseDblClick(Self);
end;

{function TfrmMain.GetIVOrder(UniqueIDString: string): TBCMA_MedOrder;
var
  idxorder,
    ii,
    jj: integer;
begin
  idxorder := -1;
  Result := nil;
  with VisibleMedList do
    for ii := 0 to VisibleMedList.count - 1 do
      with TBCMA_MedOrder(items[ii]) do
        for jj := 0 to UniqueIDCount - 1 do
          if piece(UniqueIDs.strings[jj], '^', 1) = UniqueIDString then
          begin
            if idxorder <> -1 then
            begin
              DefMessageDlg('The scanned bag was found in more then one order!'
                + #13 +
                'This indicates a possible error in the order.  The bag can not' + #13
                +
                'be scanned at this time', mtError, [mbOk], 0);
              Result := nil;
              Exit;
            end;
            Result := TBCMA_MedOrder(Items[ii]);
            idxorder := jj;
          end;
end; }

procedure TfrmMain.RebuildIVOrderHistory(RepaintIVHistory: Boolean;
  IVOrderNumber: string);
var
  x, y: Integer;
  DateList: TStringList;
  TempDate,
    ResultStr: string;
begin
  y := 0; // rpk 4/15/2009

  with FraIV1 do begin
    tvwIVHistory.Visible := False;
    lblIVHistory.Caption := 'Retrieving History...';
    lblIVHistory.Repaint;

    lblBagDetail.Caption := 'No Bag Selected.';
    stBagDetail.Caption := 'No Bag Selected.';

    lstIVBagDetail.Visible := False;
    sgIVBagDetail.Hide;
    lblBagDetail.Repaint;
    stBagDetail.Repaint;
  end;

  {Load the Bags for a specific order number}
  with BCMA_Patient do begin
    ResultStr := LoadIVBags(IVOrderNumber);

    if piece(ResultStr, '^', 1) = '-1' then
      FraIV1.lblIVHistory.Caption := piece(ResultStr, '^', 2)
    else
      IVHistoryDates := IVBags;

    with fraIV1.tvwIVHistory do begin
      IVHistClearing := True;
      Items.Clear;
      IVHistClearing := False;
    end;

  end;

  if RepaintIVHistory = True then
    if IVHistoryDates.Count > 0 then begin
      IVHistoryDates.Sort(IVHistoryDatesSort);
      {Build the list of dates for the tree}
      DateList := TStringList.Create;
      IVHistClearing := True;
      fraiv1.tvwivhistory.Items.Clear;
      IVHistClearing := False;

      fraiv1.tvwIVHistory.Items.beginUpdate;
      for x := 0 to IVHistoryDates.Count - 1 do begin
        TempDate := piece(TBCMA_IVBags(IVHistoryDates[x]).TimeLastGiven, '.', 1)
          + '.';

        {Make sure we only have unique dates}
        if DateList.IndexOf(TempDate) = -1 then begin
          DateList.Add(TempDate);
          with fraIV1.tvwIVHistory do begin
            Items.Add(nil, DisplayVADateYear(TempDate));
            y := items.count - 1;
          end;

        end;

        if ((TBCMA_IVBags(IVHistoryDates[x]).ScanStatus = 'I') or
          (TBCMA_IVBags(IVHistoryDates[x]).ScanStatus = 'S') and
          (fraIV1.tvwIVHistory.items[y].Expanded = false)) then begin
          fraIV1.tvwIVHistory.items[y].selected := true;
          fraIV1.tvwIVHistory.items[y].Expand(False);
        end;
      end;

      fraIV1.tvwIVHistory.Items[0].selected := True;
      fraIV1.tvwIVHistory.items[0].Expand(False);
      tvwIVExpandCollapse := False;
      fraiv1.tvwIVHistory.Items.EndUpdate;
      fraIV1.tvwIVHistory.Visible := True;
    end;
end; // RebuildIVOrderHistory

procedure TfrmMain.lstUnitDoseClick(Sender: TObject);
begin
  //showmessage(TComponent (Sender).Name);
  with lstUnitDose do
    if VisibleMedList.Count > 0 then
      case lstCurrentTab of
        ctIV:
          if ItemIndex > -1 then begin
            with TBCMA_MedOrder(VisibleMedList.items[ItemIndex]) do
              if OrderNumber <> CurrentOrderNumber then begin
                currentOrderNumber := OrderNumber;
                RebuildIVOrderHistory(True, CurrentOrderNumber);
              end;
          end;
      end;
end;

procedure TfrmMain.fraIV1tvwIVHistoryChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
//var
//  TempNode: TTreeNode;
begin
  if IVHistClearing = True then
    Exit;
  allowchange := True;
end;

procedure TfrmMain.fraIV1tvwIVHistoryChange(Sender: TObject;
  Node: TTreeNode);
var
  i, j, zImageIndex: integer;
  tempNode: TTreeNode;
  zStatus: string;
  SortedStatusList: TList;
begin
  //if we are clearing the tree, then there is no need to go through here
  if IVHistClearing = True then
    Exit;

  try
    if (Node.Level = 0) and (Node.GetFirstChild = nil) then begin
      SortedStatusList := tvwIVHistorySortStatus(Node.Text);
      for i := 0 to SortedStatusList.Count - 1 do begin
        fraiv1.tvwIVHistory.Items.AddChildObject(Node,
          TBCMA_IVBags(SortedStatusList[i]).UniqueID +
          ' - ' +
          GetLastActivityStatus(TBCMA_IVBags(SortedStatusList[i]).ScanStatus),
          SortedStatusList[i]);
        zStatus := TBCMA_IVBags(SortedStatusList[i]).ScanStatus;
        with fraiv1.tvwIVHistory do begin
          tempnode := Node.GetLastChild;
          if zStatus = 'A' then
            zImageIndex := 6
          else if zStatus = 'I' then
            zImageIndex := 5
          else if zStatus = 'C' then
            zImageIndex := 3
          else if zStatus = 'S' then
            zImageIndex := 1
          else if zStatus = 'H' then
            zImageIndex := 7
          else if zStatus = 'R' then
            zImageIndex := 4
          else if zStatus = 'M' then
            zImageIndex := 6
          else
            zImageIndex := -1;

          tempnode.ImageIndex := zImageIndex;
          tempnode.SelectedIndex := zImageIndex;

          for j := 0 to TBCMA_IVBags(SortedStatusList[i]).Additives.Count
            - 1 do begin
            fraiv1.tvwIVHistory.Items.AddChildObject(tempnode,
              piece(TBCMA_IVBags(SortedStatusList[i]).Additives[j], '^',
              3),
              SortedStatusList[i]);
            TempNode.GetLastChild.ImageIndex := 2;
            TempNode.GetLastChild.SelectedIndex := 2;
          end;

          for j := 0 to TBCMA_IVBags(SortedStatusList[i]).Solutions.Count
            - 1 do begin
            fraiv1.tvwIVHistory.Items.AddChildObject(tempnode, '  ' +
              piece(TBCMA_IVBags(SortedStatusList[i]).Solutions[j], '^',
              3),
              SortedStatusList[i]);
            TempNode.GetLastChild.ImageIndex := 2;
            TempNode.GetLastChild.SelectedIndex := 2;
          end;
          fraiv1.lstIVBagDetail.Visible := False;
          fraiv1.sgIVBagDetail.Hide;
          fraiv1.lblBagDetail.Caption := 'No Bag Selected.';
          fraIV1.stBagDetail.Caption := 'No Bag Selected.';

          //if the bag is infusing or stopped, display the bag contents.
          if ((zStatus = 'I') or (zStatus = 'S')) then
            tempNode.Expand(True);
        end;
      end;
    end;

    with fraIV1 do
      if node.level > 0 then begin
        if Node <> nil then
          if Node.Data <> CurrentBagID then
            if Node.Level > 0 then
              with TBCMA_IVBags(Node.Data) do begin
                lstIVBagDetail.Visible := False;
                sgIVBagDetail.Hide;
                lstIVBagDetail.Clear;
                sgFree(-1, -1, sgIVBagDetail);

                if Node <> nil then
                  CurrentBagId := Node.Data;
                Repaint;
                RebuildBagDetail(True);
              end;

      end;
    if Node <> nil then
      CurrentBagId := Node.Data;

    tvwIVExpandCollapse := False;

  except

  end;

end;

function TfrmMain.tvwIVHistorySortStatus(StringIn: string): TList;
var
  i, jj: integer;
  TempDate,
    zSort: string;
begin
  Result := TList.Create;
  for jj := 0 to length(IVHistorySortOrder) - 1 do begin
    zSort := IVHistorySortOrder[TIVHistoryStatusTypes(jj)];
    for i := 0 to IVHistoryDates.Count - 1 do begin
      TempDate :=
        DisplayVADateYear(TBCMA_IVBags(IVHistoryDates[i]).TimeLastGiven);
      if TempDate = StringIn then
        if zSort = TBCMA_IVBags(IVHistoryDates[i]).ScanStatus then
          Result.Add(IVHistoryDates[i]);
    end;
  end;
end;

procedure TfrmMain.fraIV1tvwIVHistoryMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  TempNode: TTreeNode;
  p: TPoint;
begin
  tempNode := fraIV1.tvwIVHistory.getnodeat(x, y);
  exit;
  with tempNode do
    if tempNode <> nil then
      if (button = mbRight) then begin
        selected := True;
        if (level = 1) then begin
          selected := True;
          SetIVHistMenus;
          p := fraIV1.ClientToScreen(Point(x, y));
        end;
      end;
end;

procedure TfrmMain.fraIV1lstIVBagDetailMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
var
  TextString: string;
  CellHeight: Integer;
  ARect: TRect;

begin
  with fraIV1.lstIVBagDetail do begin
    ARect := ItemRect(Index);
    with ARect do begin
      Left := 0;
      Top := 0;
      Bottom := 0;
    end;

    ARect.Right := fraIV1.hdrIvBagDetail.Sections[3].Width - 6;
    TextString := piece(items[Index], '^', 4);
    if TextString = '' then
      TextString := ' ';
    CellHeight := DrawText(Canvas.Handle, PChar(TextString),
      Length(TextString),
      ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX);

    with ARect do begin
      Left := 0;
      Top := 0;
      Bottom := 0;
    end;
  end;
  Height := CellHeight + 2;
  BagDetailItemsHeight := BagDetailItemsHeight + Height;
end; // fraIV1lstIVBagDetailMeasureItem

procedure TfrmMain.fraIV1lstIVBagDetailDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
type
  TempArray = array of string;
var
  x,
    CellRight,
    TitleCount: integer;
  TextString: string;
  CurrentFontColor: TColor;
  ARect: TRect;
  outText: string;

begin
  if VisibleMedList.Count > 0 then
    with fraIV1.lstIVBagDetail do begin

      ARect := Rect;
      CurrentFontColor := Canvas.Font.Color;

      with Canvas do begin
        FillRect(ARect);
        Pen.Color := clSilver;
        MoveTo(ARect.Left, ARect.Bottom - 1);
        LineTo(ARect.Right, ARect.Bottom - 1);
      end;

      CellRight := -3;

      TitleCount := fraIV1.hdrIvBagDetail.Sections.Count;

      for x := 0 to TitleCount - 1 do begin
        CellRight := CellRight + fraIV1.hdrIVBagDetail.Sections[x].Width;
        Canvas.MoveTo(CellRight, ARect.Bottom - 1);
        Canvas.LineTo(CellRight, ARect.Top);
      end;

      for x := 0 to TitleCount - 1 do begin
        if x > 0 then
          ARect.Left := ARect.Right + 2
        else
          ARect.Left := 2;

        ARect.Right := ARect.Left + fraIV1.hdrIVBagDetail.Sections[x].Width
          - 6;

        TextString := '';
        OutText := '';

        case x of
          0: begin
              OutText := DisplayVADateYearTime(piece(items[Index], '^', 1));
            end;
          1: begin
              OutText := piece(items[Index], '^', 2);
            end;
          2: begin
              OutText := piece(items[Index], '^', 3);
            end;
          3: begin
              OutText := piece(items[Index], '^', 4);
            end;
        end;

        case x of
          0, 1, 2:
            DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect,
              DT_END_ELLIPSIS or DT_NOPREFIX);
          3:
            DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect,
              DT_WORDBREAK or DT_NOPREFIX);
        end;
        Canvas.Font.Color := CurrentFontColor;
        ARect.Right := ARect.Right + 4;
      end;
    end;
  with fraIV1 do begin
    if BagDetailItemsHeight > lstIVBagDetail.Height then
      //sbrBagDetail.Enabled := True
    else
      //sbrVDL.Enabled := False;
  end;
end; // fraIV1lstIVBagDetailDrawItem

// JAWS sees text modified in static text field when focus set back to that field.
// use stAlt as means of switching focus away from and back to stVDLUnitDose.

procedure TfrmMain.SetVDLMessage(StringIn: string);
begin
  case lstCurrentTab of
    ctUD, ctPB: begin
        //      lblVDLUnitDose.Caption := StringIn;
        stVDLUnitDose.Caption := StringIn;
        //      if ScreenReaderSystemActive then begin
        //        sgUnitDose.SendToBack;
        //      end
        //      else begin
        //        lstUnitDose.SendToBack;
        //        lblVDLUnitDose.BringToFront;
        //        lblVDLUnitDose.Show;
        //        lblVDLUnitDose.Repaint;
        //      end;
        stAlt.Enabled := True;
        stAlt.BringToFront;
        stAlt.Show;
        if stAlt.CanFocus then
          stAlt.SetFocus;
        stVDLUnitDose.BringToFront;
        stVDLUnitDose.Show;
        if stVDLUnitDose.CanFocus then
          stVDLUnitDose.SetFocus;
        stVDLUnitDose.Repaint;
      end;
    ctIV: begin
        lblVDLIV.Caption := StringIn;
        lblVDLIV.Repaint;
      end;
  end;

end;

procedure TfrmMain.SetSGRowOld(irow: Integer);
var
  outstr: string;
  I: Integer;
begin
  with sgUnitDose do begin
    with TBCMA_MedOrder(VisibleMedList[irow - 1]) do begin
      case lstCurrentTab of
        ctUD: begin
            Cells[0, irow] := ScanStatus;
            Cells[1, irow] := VerifyNurse;
            Cells[2, irow] := SelfMed;
            Cells[3, irow] := ScheduleType;

            outstr := ActiveMedication;
            for I := 0 to OrderedDrugCount - 1 do
              outstr := outstr + ' ' + OrderedDrugs[i].Name;
            outstr := outstr + ' ';
            Cells[4, irow] := outstr;
            Canvas.Font.Color := clRed;
            Canvas.Font.Style := [fsBold];
            Cells[4, irow] := Cells[4, irow] + SpecialInstructions;
            Canvas.Font.Color := clBlack;
            Canvas.Font.Style := [];

            Cells[5, irow] := Trim(Dosage) + ', ' + Schedule;
            Cells[6, irow] := Route;

            if ScheduleType = 'C' then
              Cells[7, irow] := DisplayVADate(AdministrationTime);

            if LastActivityStatus = '' then
              outstr := ''
            else
              outstr := GetLastActivityStatus(LastActivityStatus) + ': ' +
                DisplayVADateYearTime(TimeLastAction);
            if OrderTransferred = '1' then
              outstr := outstr + ' ' + BCMA_Patient.TransferredTransactionType;
            Cells[8, irow] := outstr;
          end;
      end;
    end;
  end;

end; // SetSGRowOld

procedure TfrmMain.SetSGRow(irow: Integer);
type
  TempArray = array of string;
var
  ii, icol: Integer;
  TextString, si_txt: string;
  OutText: string;
  MO_tmp: TBCMA_MedOrder;
begin
  if (iRow = 0) or (iRow > VisibleMedList.Count) then
    Exit;

  sgUnitDose.Rowheights[irow] := GetSGRowHeight(irow);

  if VisibleMedList.Count > 0 then
    with sgUnitDose do begin
      MO_tmp := TBCMA_MedOrder(VisibleMedList[irow - 1]);

      with MO_tmp do begin
        for icol := 1 to sgUnitDose.ColCount - 1 do begin
          TextString := '';
          OutText := '';
          case icol of
            1: begin
                case lstCurrentTab of
                  ctUD, ctPB:
                    OutText := ScanStatus;
                  ctIV:
                    OutText := GetOrderStatus(OrderStatus);
                end;
              end;
            2: begin
                OutText := VerifyNurse;
              end;
            3: begin
                case lstCurrentTab of
                  ctUD:
                    OutText := SelfMed;
                  ctPB:
                    OutText := ScheduleType;
                  ctIV:
                    OutText := GetIVType(AdministrationUnit);
                end;
              end;

            4: begin
                case lstCurrentTab of
                  ctUD:
                    OutText := ScheduleType;
                  ctPB, ctIV: begin
                      TextString := ActiveMedication;
                      sgUnitDose.Cells[iCol, iRow] := TextString;
                      // rpk 9/24/2009

                      if OrderTypeID = otUnitDose then begin
                        for ii := 0 to OrderedDrugCount - 1 do begin
                          //                            ARect.Top := ARect.Top + TempHeight;
                          TextString := OrderedDrugs[ii].Name;
                          if TextString > '' then
                            sgUnitDose.Cells[iCol, iRow] :=
                              sgUnitDose.Cells[iCol, iRow] + ', ' + TextString;
                          // rpk 9/24/2009
                        end;
                      end
                      else begin
                        for ii := 0 to AdditiveCount - 1 do begin
                          TextString := piece(Additives[ii], '^', 3) +
                            ' ' +
                            piece(Additives[ii], '^', 4) + ' ' +
                            piece(Additives[ii], '^', 5);
                          if Trim(TextString) > '' then
                            sgUnitDose.Cells[iCol, iRow] :=
                              sgUnitDose.Cells[iCol, iRow] + ', ' + TextString;
                          // rpk 9/24/2009
                        end;
                        //                          ARect.Left := ARect.Left + 7;
                        for ii := 0 to SolutionCount - 1 do begin
                          //                            ARect.Top := ARect.Top + TempHeight;
                          TextString := piece(Solutions[ii], '^', 3) +
                            ' ' +
                            piece(Solutions[ii], '^', 4) + ' ' +
                            piece(Solutions[ii], '^', 5);
                          if Trim(TextString) > '' then
                            sgUnitDose.Cells[iCol, iRow] :=
                              sgUnitDose.Cells[iCol, iRow] + ', ' + TextString;
                          // rpk 9/24/2009
                        end
                      end; // else not unit dose

                      if OrderTooTall then begin
                        sgUnitDose.Cells[iCol, iRow] := sgUnitDose.Cells[iCol,
                          iRow] +
                          ' ' + ErrRowTooTall;
                      end
                      else begin
                        si_txt := Trim(SpecialInstructions); // rpk 10/28/2009
                        si_txt := StripLB(si_txt); // rpk 10/28/2009
                        if si_txt > '' then // rpk 10/28/2009
                          sgUnitDose.Cells[iCol, iRow] :=
                            sgUnitDose.Cells[iCol, iRow] + ' ' + si_txt;
                        // rpk 10/28/2009
                      end; // else not ordertootall

                    end;
                end;
              end;

            5: begin
                case lstCurrentTab of
                  ctUD: begin
                      TextString := ActiveMedication;
                      sgUnitDose.Cells[iCol, iRow] := TextString; // rpk 8/5/2009

                      for ii := 0 to OrderedDrugCount - 1 do begin
                        //                          ARect.Top := ARect.Top + TempHeight;
                        TextString := OrderedDrugs[ii].Name;
                        if TextString > '' then
                          sgUnitDose.Cells[iCol, iRow] :=
                            sgUnitDose.Cells[iCol, iRow] + ', ' + TextString;
                        // rpk 8/5/2009
                      end;

                      if OrderTooTall then begin
                        sgUnitDose.Cells[iCol, iRow] := sgUnitDose.Cells[iCol,
                          iRow] +
                          ' ' + ErrRowTooTall;
                      end
                      else begin
                        si_txt := Trim(SpecialInstructions); // rpk 10/28/2009
                        si_txt := StripLB(si_txt); // rpk 10/28/2009
                        if si_txt > '' then // rpk 10/28/2009
                          sgUnitDose.Cells[iCol, iRow] :=
                            sgUnitDose.Cells[iCol, iRow] + ' ' + si_txt;
                        // rpk 10/28/2009
                      end;

                    end;
                  ctPB, ctIV: begin
                      TextString := Trim(Dosage) + ', ' + Schedule;
                      sgUnitDose.Cells[iCol, iRow] := TextString;
                      // rpk 9/24/2009
                    end;
                end;
              end;

            6:
              case lstCurrentTab of
                ctUD: begin
                    TextString := Trim(Dosage) + ', ' + Schedule;
                    sgUnitDose.Cells[iCol, iRow] := TextString;
                  end;
                ctPB, ctIV: begin
                    sgUnitDose.Cells[iCol, iRow] := Route; // rpk 9/24/2009
                  end;
              end;
            7:
              case lstCurrentTab of
                ctUD: begin
                    sgUnitDose.Cells[iCol, iRow] := Route; // rpk 8/5/2009
                  end;
                ctPB:
                  OutText := DisplayVADate(AdministrationTime);
                ctIV: begin
                    if LastActivityStatus > '' then begin
                      TextString := GetLastActivityStatus(LastActivityStatus) +
                        ': ' +
                        DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                      sgUnitDose.Cells[iCol, iRow] := TextString;
                      // rpk 9/24/2009
                    end;
                    if OrderChangedData.Count > 0 then begin
                      OutText :=
                        GetLastActivityStatus(piece(OrderChangedData[0],
                        '^', 4));
                      OutText := OutText + ' bag on changed order';
                    end
                    else
                      OutText := '';

                  end;
              end; // case
            8:
              case lstCurrentTab of
                ctUD:
                  if ScheduleType = 'C' then begin
                    OutText := DisplayVADate(AdministrationTime);
                  end;
                ctPB: begin
                    if LastActivityStatus > '' then begin
                      TextString := GetLastActivityStatus(LastActivityStatus) +
                        ': ' +
                        DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                      sgUnitDose.Cells[iCol, iRow] := TextString;
                      // rpk 9/24/2009
                    end;
                    if OrderTransferred = '1' then
                      OutText := BCMA_Patient.TransferredTransactionType
                    else
                      OutText := '';
                  end;
              end; // case
            9:
              case lstCurrentTab of
                ctUD: begin
                    if LastActivityStatus > '' then begin
                      TextString := GetLastActivityStatus(LastActivityStatus) +
                        ': ' +
                        DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                      sgUnitDose.Cells[iCol, iRow] := TextString;
                      // rpk 9/24/2009
                    end;
                    if OrderTransferred = '1' then
                      OutText := BCMA_Patient.TransferredTransactionType
                    else
                      OutText := '';
                  end;
              end;
          end; // case icol

          case lstCurrentTab of
            ctUD:
              case icol of
                1, 2, 3, 6, 7, 8, 9: begin
                    if OutText <> '' then
                      sgUnitDose.Cells[iCol, iRow] :=
                        sgUnitDose.Cells[iCol, iRow] + ' ' + OutText;
                    // rpk 8/5/2009
                  end;
              end;
            ctPB, ctIV:
              case icol of
                1, 2, 5, 6, 7, 8: begin
                    if OutText <> '' then
                      sgUnitDose.Cells[iCol, iRow] :=
                        sgUnitDose.Cells[iCol, iRow] + ' ' + OutText;
                    // rpk 9/24/2009
                  end;
              end;
          end; // case lstCurrentTab

        end; // for icol
      end; // with MO_tmp
      sgUnitDose.Invalidate; // rpk 8/13/2009
    end; // with sgUnitDose
end; // SetSGRow

procedure TfrmMain.sgUnitDoseClick(Sender: TObject);
begin
  //showmessage(TComponent (Sender).Name);
  with sgUnitDose do
    if VisibleMedList.Count > 0 then
      case lstCurrentTab of
        ctIV:
          if row > 0 then begin
            with TBCMA_MedOrder(VisibleMedList.items[row - 1]) do
              if OrderNumber <> CurrentOrderNumber then begin
                currentOrderNumber := OrderNumber;
                RebuildIVOrderHistory(True, CurrentOrderNumber);
              end;
          end;
      end;
end;

procedure TfrmMain.sgUnitDoseContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
var
  acol, arow: Integer;
  gr: TGridRect;
begin
  sgUnitDose.MouseToCell(MousePos.x, MousePos.y, acol, arow);
  if (acol > 0) and (arow > 0) then begin
    gr.Left := acol;
    gr.Right := acol;
    gr.Top := arow;
    gr.Bottom := arow;
    sgUnitDose.Selection := gr;
  end;

  case lstCurrentTab of
    ctUD, ctPB: begin
        AddComment1.Visible := True;
        mnuPopUpMark.Visible := True;
        MissingDose1.Visible := True;
        PRNEffectiveness1.Visible := True;
        mnuN1.Visible := True;
        mnuN2.Visible := True;
      end;
    ctIV: begin
        mnuN1.Visible := False;
        mnuN2.Visible := False;
        AddComment1.Visible := False;
        mnuPopUpMark.Visible := False;
        MissingDose1.Visible := False;
        PRNEffectiveness1.Visible := False;
      end;
  end;

  //  Handled := True;

end;

procedure TfrmMain.sgUnitDoseDblClick(Sender: TObject);
begin
  with sgUnitDose do begin
    Enabled := False;
    with TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.row - 1]) do
      DisplayOrder(PatientIEN, OrderNumber);
    Enabled := True;
  end;
  ScannerActivate;
end;

procedure TfrmMain.sgUnitDoseDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
type
  TempArray = array of string;
var
  x,
    ii,
    CellRight,
    TempHeight,
    CurrentTop,
    TitleCount,
    Index: integer;
  TextString, si_txt: string;
  CurrentFontColor: TColor;
  ARect: TRect;
  OutText: string;
  MO_tmp: TBCMA_MedOrder;
begin
  if (ARow = 0) or (ACol = 0) or (ARow > VisibleMedList.Count) then
    Exit;

  // avoid repetitive drawing once cell is filled.
  if sgUnitDose.Cells[ACol, ARow] <> '' then
    Exit;

  //
  // Todo: add cr-lfs to format long lines as multiple lines.
  //
  Index := ARow - 1;
  si_txt := '';
  ARect := Rect;
  TempHeight := 0;

  // Assuming that we are drawing on every paint, make sure that cell is empty
  // to avoid repeatedly adding to same cell.
//  sgUnitDose.Cells[ACol, ARow] := '';  // rpk 8/27/2010

  if VisibleMedList.Count > 0 then
    with sgUnitDose do begin
      outputDebugString(PChar(IntToStr(Arect.Bottom - Arect.Top)));

      CurrentFontColor := Canvas.Font.Color;

      //      if Index > 0 then
      //        ii := Index
      //      else
      //        ii := -1;
      MO_tmp := TBCMA_MedOrder(VisibleMedList[Index]);

      //
      // on Provider Hold
      //
      //      if (TBCMA_MedOrder(VisibleMedList[Index]).OrderStatus = 'H') and not
      if (MO_tmp.OrderStatus = 'H') and not
        (gdSelected in State) then
        Canvas.Brush.Color := $00DDDDDD;

      //      with Canvas do begin
      //        FillRect(ARect);
      //        Pen.Color := clSilver;
      //        MoveTo(ARect.Left, ARect.Bottom - 1);
      //        LineTo(ARect.Right, ARect.Bottom - 1);
      //      end;

      //      CellRight := -3;
      //      case lstCurrentTab of
      //        ctUD:
      //          TitleCount := Length(VDLColumnTitles);
      //        ctPB:
      //          TitleCount := Length(lstPBColumnTitles);
      //        ctIV:
      //          TitleCount := Length(lstIVColumnTitles);
      //      else
      //        TitleCount := Length(VDLColumnTitles);
      //      end;

      //      for x := 0 to TitleCount - 1 do begin
      //        CellRight := CellRight + hdrUnitDose.Sections[x].Width;
      //        Canvas.MoveTo(CellRight, ARect.Bottom - 1);
      //        Canvas.LineTo(CellRight, ARect.Top);
      //      end;

      //      for x := 0 to TitleCount - 1 do begin
      x := ACol - 1;
      if x > 0 then
        ARect.Left := ARect.Right + 2
      else
        ARect.Left := 2;

      TextString := '';
      OutText := '';

      //        with TBCMA_MedOrder(VisibleMedList[Index]) do
      with MO_tmp do
        case x of
          0:
            case lstCurrentTab of
              ctUD, ctPB:
              // DEBUG Provider Hold
                if (MO_tmp.OrderStatus = 'H') then
                  OutText := '(PH), ' + ScanStatus
                else
                  OutText := ScanStatus;
              ctIV:
                OutText := GetOrderStatus(OrderStatus);
            end;
          1:
            OutText := VerifyNurse;
          2:
            case lstCurrentTab of
              ctUD:
                OutText := SelfMed;
              ctPB:
                OutText := ScheduleType;
              ctIV:
                OutText := GetIVType(AdministrationUnit);
            end;

          3:
            case lstCurrentTab of
              ctUD:
                OutText := ScheduleType;
              ctPB, ctIV: begin
                  CurrentTop := ARect.Top;
                  TextString := ActiveMedication;
                  sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                  TempHeight := DrawText(Canvas.Handle,
                    PChar(TextString),
                    Length(TextString), ARect,
                    DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                    DT_EDITCONTROL or DT_NOPREFIX);
                  ARect.Left := ARect.Left + 7;

                  { if OrderTooTall then
                  begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsbold];
                    sgUnitDose.Cells[ACol, ARow] := ErrRowTooTall;
                    DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                      Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                      DT_NOPREFIX or DT_WORD_ELLIPSIS);
                  end
                  else
                  begin
                    if OrderTypeID = otUnitDose then
                    begin
                      for ii := 0 to OrderedDrugCount - 1 do
                      begin
                        ARect.Top := ARect.Top + TempHeight;
                        TextString := OrderedDrugs[ii].Name;
                        if TextString > '' then
                          sgUnitDose.Cells[ACol, ARow] :=
                            sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;  // rpk 9/24/2009
                        DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect, DT_END_ELLIPSIS
                          or
                          DT_NOPREFIX);
                      end;
                    end
                    else
                    begin
                      for ii := 0 to AdditiveCount - 1 do
                      begin
                        ARect.Top := ARect.Top + TempHeight;
                        TextString := piece(Additives[ii], '^', 3) +
                          ' ' +
                          piece(Additives[ii], '^', 4) + ' ' +
                          piece(Additives[ii], '^', 5);
                        if Trim(TextString) > '' then
                          sgUnitDose.Cells[ACol, ARow] :=
                            sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;  // rpk 9/24/2009
                        DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect, DT_END_ELLIPSIS
                          or
                          DT_NOPREFIX);
                      end;
                      ARect.Left := ARect.Left + 7;
                      for ii := 0 to SolutionCount - 1 do
                      begin
                        ARect.Top := ARect.Top + TempHeight;
                        TextString := piece(Solutions[ii], '^', 3) +
                          ' ' +
                          piece(Solutions[ii], '^', 4) + ' ' +
                          piece(Solutions[ii], '^', 5);
                        if Trim(TextString) > '' then
                          sgUnitDose.Cells[ACol, ARow] :=
                            sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;  // rpk 9/24/2009
                        DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect, DT_END_ELLIPSIS
                          or
                          DT_NOPREFIX);
                      end
                    end;
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsbold];
                    if Trim(SpecialInstructions) > '' then
                      sgUnitDose.Cells[ACol, ARow] :=
                        sgUnitDose.Cells[ACol, ARow] + ' ' + SpecialInstructions;  // rpk 9/24/2009
                    DrawText(Canvas.Handle, PChar(SpecialInstructions),
                      Length(SpecialInstructions), ARect, DT_WORDBREAK or
                      DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS);  // rpk 4/29/2009
                  end; }// else not ordertootall

                  if OrderTypeID = otUnitDose then begin
                    for ii := 0 to OrderedDrugCount - 1 do begin
                      ARect.Top := ARect.Top + TempHeight;
                      TextString := OrderedDrugs[ii].Name;
                      if TextString > '' then begin
                        sgUnitDose.Cells[ACol, ARow] :=
                          sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;
                        // rpk 9/24/2009
                        DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect,
                          DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                          DT_EDITCONTROL or DT_NOPREFIX);
                      end;
                    end;
                  end
                  else begin
                    for ii := 0 to AdditiveCount - 1 do begin
                      ARect.Top := ARect.Top + TempHeight;
                      TextString := piece(Additives[ii], '^', 3) +
                        ' ' +
                        piece(Additives[ii], '^', 4) + ' ' +
                        piece(Additives[ii], '^', 5);
                      if Trim(TextString) > '' then begin
                        sgUnitDose.Cells[ACol, ARow] :=
                          sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;
                        // rpk 9/24/2009
                        TempHeight := DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect,
                          DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                          DT_EDITCONTROL or DT_NOPREFIX);
                      end;
                    end;
                    ARect.Left := ARect.Left + 7;
                    for ii := 0 to SolutionCount - 1 do begin
                      ARect.Top := ARect.Top + TempHeight;
                      TextString := piece(Solutions[ii], '^', 3) +
                        ' ' +
                        piece(Solutions[ii], '^', 4) + ' ' +
                        piece(Solutions[ii], '^', 5);
                      if Trim(TextString) > '' then begin
                        sgUnitDose.Cells[ACol, ARow] :=
                          sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;
                        // rpk 9/24/2009
                        TempHeight := DrawText(Canvas.Handle, PChar(TextString),
                          Length(TextString), ARect,
                          DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                          DT_EDITCONTROL or DT_NOPREFIX);
                      end;
                    end
                  end; // else not unit dose

                  if OrderTooTall then begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsbold];
                    sgUnitDose.Cells[ACol, ARow] := ErrRowTooTall;
                    DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                      Length(ErrRowTooTall), ARect,
                      DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                      DT_EDITCONTROL or DT_NOPREFIX);
                  end
                  else begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsbold];
                    //                        if Trim(SpecialInstructions) > '' then
                    //                          sgUnitDose.Cells[ACol, ARow] :=
                    //                            sgUnitDose.Cells[ACol, ARow] + ' ' + SpecialInstructions;  // rpk 9/24/2009
                    si_txt := Trim(SpecialInstructions); // rpk 10/28/2009
                    //                        si_txt := StripLB(si_txt);  // rpk 10/28/2009
                    if si_txt > '' then begin // rpk 10/28/2009
                      sgUnitDose.Cells[ACol, ARow] :=
                        sgUnitDose.Cells[ACol, ARow] + ' ' + si_txt;
                      // rpk 10/28/2009
                      DrawText(Canvas.Handle, PChar(si_txt),
                        Length(si_txt), ARect,
                        DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                        DT_EDITCONTROL or DT_NOPREFIX);
                    end;
                  end; // else not ordertootall

                  ARect.Top := CurrentTop;
                end; // ctPB, ctIV
            end; // case lstCurrentTab

          4:
            case lstCurrentTab of
              ctUD: begin
                  CurrentTop := ARect.Top;
                  TextString := ActiveMedication;
                  sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 8/5/2009
                  TempHeight := DrawText(Canvas.Handle,
                    PChar(TextString),
                    Length(TextString), ARect,
                    DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                    DT_EDITCONTROL or DT_NOPREFIX);
                  ARect.Left := ARect.Left + 7;

                  { if OrderTooTall then
                  begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsBold];
                    sgUnitDose.Cells[ACol, ARow] := ErrRowTooTall;
                    DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                      Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                      DT_NOPREFIX or DT_WORD_ELLIPSIS);
                  end
                  else
                  begin
                    for ii := 0 to OrderedDrugCount - 1 do
                    begin
                      ARect.Top := ARect.Top + TempHeight;
                      TextString := OrderedDrugs[ii].Name;
                      if TextString > '' then
                        sgUnitDose.Cells[ACol, ARow] :=
                          sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;  // rpk 8/5/2009
                      DrawText(Canvas.Handle, PChar(TextString),
                        Length(TextString), ARect, DT_END_ELLIPSIS or
                        DT_NOPREFIX);
                    end;
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsBold];
                    if Trim(SpecialInstructions) > '' then
                      sgUnitDose.Cells[ACol, ARow] :=
                        sgUnitDose.Cells[ACol, ARow] + ' ' + SpecialInstructions; // rpk 8/5/2009
                    DrawText(Canvas.Handle, PChar(SpecialInstructions),
                      Length(SpecialInstructions), ARect, DT_WORDBREAK or
                      DT_NOPREFIX or DT_END_ELLIPSIS or DT_WORD_ELLIPSIS);  // rpk 4/29/2009
                  end; }// else not ordertootall

                  for ii := 0 to OrderedDrugCount - 1 do begin
                    ARect.Top := ARect.Top + TempHeight;
                    TextString := OrderedDrugs[ii].Name;
                    if TextString > '' then begin
                      sgUnitDose.Cells[ACol, ARow] :=
                        sgUnitDose.Cells[ACol, ARow] + ', ' + TextString;
                      // rpk 8/5/2009
                      TempHeight := DrawText(Canvas.Handle, PChar(TextString),
                        Length(TextString), ARect,
                        DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                        DT_EDITCONTROL or DT_NOPREFIX);
                    end;
                  end;

                  if OrderTooTall then begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsBold];
                    sgUnitDose.Cells[ACol, ARow] := ErrRowTooTall;
                    DrawText(Canvas.Handle, PChar(ErrRowTooTall),
                      Length(ErrRowTooTall), ARect, DT_WORDBREAK or
                      DT_NOPREFIX or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS);
                  end
                  else begin
                    ARect.Top := ARect.Top + TempHeight;
                    Canvas.Font.Color := clRed;
                    Canvas.Font.Style := [fsBold];
                    //                        if Trim(SpecialInstructions) > '' then
                    //                          sgUnitDose.Cells[ACol, ARow] :=
                    //                            sgUnitDose.Cells[ACol, ARow] + ' ' + SpecialInstructions; // rpk 8/5/2009
                    si_txt := Trim(SpecialInstructions); // rpk 10/28/2009
                    //                        si_txt := StripLB(si_txt);  // rpk 10/28/2009
                    if si_txt > '' then begin // rpk 10/28/2009
                      sgUnitDose.Cells[ACol, ARow] :=
                        sgUnitDose.Cells[ACol, ARow] + ' ' + si_txt;
                      // rpk 10/28/2009
                  //                      TempHeight := DrawText(sgUnitDose.Canvas.Handle, PChar(SpecialInstructions),
                  //                        Length(SpecialInstructions), ARect,
                  //                        DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                  //                        DT_EDITCONTROL or DT_NOPREFIX);
                      TempHeight := DrawText(Canvas.Handle, PChar(si_txt),
                        Length(si_txt), ARect,
                        DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                        DT_EDITCONTROL or DT_NOPREFIX);
                    end;
                  end;
                  ARect.Top := CurrentTop;
                end; // ctUD

              ctPB, ctIV: begin
                  TextString := Trim(Dosage) + ', ' + Schedule;
                  sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                  DrawText(Canvas.Handle, PChar(TextString),
                    Length(TextString), ARect,
                    DT_WORDBREAK or DT_WORD_ELLIPSIS or DT_END_ELLIPSIS or
                    DT_EDITCONTROL or DT_NOPREFIX);
                end;
            end; // case lstCurrentTab

          5:
            case lstCurrentTab of
              ctUD: begin
                  TextString := Trim(Dosage) + ', ' + Schedule;
                  sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                  DrawText(Canvas.Handle, PChar(TextString),
                    Length(TextString), ARect, DT_WORDBREAK or
                    DT_NOPREFIX or DT_WORD_ELLIPSIS);
                end;
              ctPB, ctIV: begin
                  sgUnitDose.Cells[ACol, ARow] := Route; // rpk 9/24/2009
                  DrawText(Canvas.Handle, PChar(Route),
                    Length(Route), ARect, DT_WORDBREAK or DT_WORD_ELLIPSIS
                    or DT_NOPREFIX);
                end;
            end; // case lstCurrentTab

          6:
            case lstCurrentTab of
              ctUD: begin
                  sgUnitDose.Cells[ACol, ARow] := Route; // rpk 8/5/2009
                  DrawText(Canvas.Handle, PChar(Route),
                    Length(Route), ARect, DT_WORDBREAK or DT_WORD_ELLIPSIS
                    or DT_NOPREFIX);
                end;

              ctPB:
                OutText := DisplayVADate(AdministrationTime);

              ctIV: begin
                  TempHeight := 0;
                  //                    if LastActivityStatus = '' then
                  //                      TempHeight := DrawText(Canvas.Handle, PChar(''), Length(''),
                  //                      ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                  //                    else begin
                  if LastActivityStatus > '' then begin
                    TextString := GetLastActivityStatus(LastActivityStatus) +
                      ': ' +
                      DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                    //                      TempHeight := DrawText(Canvas.Handle,
                    //                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)),
                    //                        Length(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)), ARect,
                    //                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                    TempHeight := DrawText(Canvas.Handle,
                      PChar(TextString), Length(TextString), ARect,
                      DT_END_ELLIPSIS or DT_NOPREFIX); // rpk 9/24/2009
                  end;
                  ARect.Top := ARect.Top + TempHeight;
                  if OrderChangedData.Count > 0 then begin
                    OutText := GetLastActivityStatus(piece(OrderChangedData[0],
                      '^', 4));
                    OutText := OutText + ' bag on changed order';
                  end
                  else
                    OutText := '';
                end; // ctIV
            end; // case lstCurrentTab

          7:
            case lstCurrentTab of
              ctUD:
                if ScheduleType = 'C' then begin
                  //                    sgUnitDose.Cells[ACol, ARow] := DisplayVADate(AdministrationTime);
                  OutText := DisplayVADate(AdministrationTime);
                end;
              ctPB: begin
                  //                    if LastActivityStatus = '' then
                  //                      TempHeight := DrawText(Canvas.Handle, PChar(''), Length(''),
                  //                      ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                  //                    else begin
                  if LastActivityStatus > '' then begin
                    TextString := GetLastActivityStatus(LastActivityStatus) +
                      ': ' +
                      DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                    //                      TempHeight := DrawText(Canvas.Handle,
                    //                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)),
                    //                        Length(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)), ARect,
                    //                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                    TempHeight := DrawText(Canvas.Handle,
                      PChar(TextString), Length(TextString), ARect,
                      DT_WORDBREAK or DT_WORD_ELLIPSIS
                      or DT_NOPREFIX or DT_END_ELLIPSIS); // rpk 9/24/2009
                  end;
                  ARect.Top := ARect.Top + TempHeight;
                  if OrderTransferred = '1' then
                    OutText := BCMA_Patient.TransferredTransactionType
                  else
                    OutText := '';
                end; // ctPB
            end; // case lstCurrentTab

          8:
            case lstCurrentTab of
              ctUD: begin
                  //                    if LastActivityStatus = '' then
                  //                      TempHeight := DrawText(Canvas.Handle, PChar(''), Length(''),
                  //                      ARect, DT_END_ELLIPSIS or DT_NOPREFIX)
                  //                    else begin
                  if LastActivityStatus > '' then begin
                    TextString := GetLastActivityStatus(LastActivityStatus) +
                      ': ' +
                      DisplayVADateYearTime(TimeLastAction); // rpk 9/24/2009
                    //                      sgUnitDose.Cells[ACol, ARow] := GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction);
                    //                      TempHeight := DrawText(Canvas.Handle,
                    //                        PChar(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)),
                    //                        Length(GetLastActivityStatus(LastActivityStatus) + ': ' +
                    //                        DisplayVADateYearTime(TimeLastAction)), ARect,
                    //                        DT_END_ELLIPSIS or DT_NOPREFIX);
                    sgUnitDose.Cells[ACol, ARow] := TextString; // rpk 9/24/2009
                    TempHeight := DrawText(Canvas.Handle,
                      PChar(TextString), Length(TextString), ARect,
                      DT_END_ELLIPSIS or DT_NOPREFIX); // rpk 9/24/2009
                  end;
                  ARect.Top := ARect.Top + TempHeight;
                  if OrderTransferred = '1' then
                    OutText := BCMA_Patient.TransferredTransactionType
                  else
                    OutText := '';
                end; // UD
            end; // case

        end; // case x

      case lstCurrentTab of
        ctUD:
          case x of
            0, 1, 2, 3, 6, 7, 8: begin
                if OutText <> '' then begin
                  sgUnitDose.Cells[ACol, ARow] :=
                    sgUnitDose.Cells[ACol, ARow] + ' ' + OutText; // rpk 8/5/2009
                  DrawText(Canvas.Handle, PChar(OutText), Length(OutText),
                    ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                end;
              end;
          end;
        ctPB, ctIV:
          case x of
            0, 1, 2, 5, 6, 7: begin
                if OutText <> '' then begin
                  sgUnitDose.Cells[ACol, ARow] :=
                    sgUnitDose.Cells[ACol, ARow] + ' ' + OutText;
                  // rpk 9/24/2009
                  DrawText(Canvas.Handle, PChar(OutText), Length(OutText),
                    ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                end;
              end;
          end;
      end; // case lstCurrentTab

      Canvas.Font.Color := CurrentFontColor;
      Canvas.Font.Style := [];
      ARect.Right := ARect.Right + 4;

      // mark null cells with blank to prevent re-processing same cell on repaints.
      if sgUnitDose.Cells[ACol, ARow] = '' then
        sgUnitDose.Cells[ACol, ARow] := ' ';
      //      end;  // for x 0 to TitleCount -1
    end; // with sgUnitDose

  // Don't use Invalidate here, will put characters in locations on left side
  // of string grid.
  //  sgUnitDose.Invalidate; // rpk 8/13/2009;

end; // sgUnitDoseDrawCell

procedure TfrmMain.sgUnitDoseEnter(Sender: TObject);
var
  CanSelect: Boolean;
begin
  with sgUnitDose do begin
    // if in header row, reset to row 1
    if (Row < 1) and (RowCount > 1) then
      Row := 1;
    sgUnitDoseSelectCell(Sender, Col, Row, CanSelect);
  end;
end;

procedure TfrmMain.sgUnitDoseKeyPress(Sender: TObject; var Key: Char);
begin
  if key = chr(VK_RETURN) then
    sgUnitDoseDblClick(Self);
end; // sgUnitDoseKeyPress

procedure TfrmMain.sgUnitDoseMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  APoint: TPoint;
  acol, arow: Integer;
begin
  if Button = mbRight then begin

    //    if sgUnitDose.SelCount < 2 then
    if sgUnitDose.Selection.Top = sgUnitDose.Selection.Bottom then begin
      APoint.x := X;
      APoint.y := Y;
      //      if sgUnitDose.ItemAtPos(APoint, True) <> -1 then
      sgUnitDose.MouseToCell(X, Y, acol, arow);
      with sgUnitDose do begin
        //          if Multiselect = false then
        //          if selection.Top = selection.Bottom then
        if not (goRangeSelect in Options) then begin
          //            ItemIndex := ItemAtPos(APoint, True);
          //            selrow := arow;
          sgUnitDoseClick(sgUnitDose);
        end
        else begin
          //            Selected[ItemIndex] := False;
          //            Selected[ItemAtPos(APoint, True)] := True;
          //            seltop := Selection.Top;
          //            selbot := Selection.Bottom;
        end;
      end;
    end;
  end;
end; // sgUnitDoseMouseDown

procedure TfrmMain.sgUnitDoseMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  setDLMenus;
  ScannerActivate;
end;

procedure TfrmMain.sgUnitDoseSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  SelectedX := ACol;
  SelectedY := ARow;
  // Kludge: several functions still reference lstunitdose.ItemIndex directly.
  // Until those are modified to use something like idxOrder, lstunitdose.itemindex
  // will need to be kept in sync with sgunitdose when a screenreader (JAWS) is in use.
  if lstunitdose.ItemIndex <> (ARow - 1) then // rpk 8/26/2010
    lstunitdose.ItemIndex := ARow - 1;
end;

procedure TfrmMain.MarkHeldIV;
var
  CurFlowUID: string;
  nilPointer: Pointer;
begin
  if CurrentBagID = nil then // rpk 10/25/2010
    Exit;
  nilPointer := nil;
  ScannedInput := TBCMA_IVBags(CurrentBagID).UniqueID;
  ScanIV(TBCMA_IVBags(CurrentBagID).UniqueID, atHeld, CurFlowUID, nilPointer);
  ScannerActivate;
end;

procedure TfrmMain.MarkRefusedIV;
var
  CurFlowUID: string;
  nilPointer: Pointer;
begin
  if CurrentBagID = nil then // rpk 10/25/2010
    Exit;
  nilPointer := nil;
  ScannedInput := TBCMA_IVBags(CurrentBagID).UniqueID;
  ScanIV(TBCMA_IVBags(CurrentBagID).UniqueID, atRefused, CurFlowUID,
    nilPointer);
  ScannerActivate;
end;

function TfrmMain.MarkRemoved(aMedOrder: TBCMA_MedOrder = nil): Boolean;
var
  tmpMedOrder: TBCMA_MedOrder;
begin
  result := False;
  if aMedOrder = nil then
    tmpMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex])
  else
    tmpMedOrder := aMedOrder;
  with tmpMedOrder do begin
    UnknownMessageDisplayed := False;
    Action := 'RM';
    if ValidOrder then begin
      if LogOrder(mtMedPass, 'RM', nil) then
        Result := True;
      RebuildVirtualDueList(False);
      Self.repaint;
    end
    else if Status = -2 then
      ForceVDLRebuild
    else if Status = -10 then
      exit
    else
      DefMessageDlg(StatusMessage, mtInformation, [mbOK], 0)
  end;
end;

function TfrmMain.GetIVOrders(UniqueIDString: string): TStringList;
var
  ii,
    jj: integer;
begin
  Result := TStringList.Create;
  with VisibleMedList do
    for ii := 0 to VisibleMedList.count - 1 do
      with TBCMA_MedOrder(VisibleMedList[ii]) do
        for jj := 0 to UniqueIDCount - 1 do
          if piece(UniqueIDs.strings[jj], '^', 1) = UniqueIDString then
            if TBCMA_MedOrder(VisibleMedList[ii]).ScanStatus <> 'G' then
              if TBCMA_MedOrder(VisibleMedList[ii]).OrderStatus = 'H' then begin
                HoldOrder := True;
                Result.Clear;
                exit;
              end
              else
                Result.AddObject(IntToStr(ii), VisibleMedList[ii]);

end;

procedure TfrmMain.fraIV1tvwIVHistoryExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
  tvwIVExpandCollapse := True;
end;

procedure TfrmMain.fraIV1tvwIVHistoryCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  tvwIVExpandCollapse := True;
end;

procedure TfrmMain.AddCommentIV;
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
var
  ii, idxOrder: integer;
begin
  idxOrder := -1;
  TempNode := fraIV1.tvwIVHistory.Selected;
  aMedOrder := nil;
  if TempNode.Level = 1 then begin
    if Pos('WS', TBCMA_IVBags(tempNode.Data).UniqueID) > 0 then begin
      with IVHistoryDates do
        for ii := 0 to Count - 1 do
          with TBCMA_IVBags(Items[ii]) do
            if UniqueID = TBCMA_IVBags(tempNode.Data).UniqueID then begin
              if idxorder <> -1 then begin
                DefMessageDlg('The scanned bag was found more then once in the history!'
                  + #13 +
                  'This indicates a possible error in the data.  The bag can not' + #13
                  +
                  'be scanned at this time', mtError, [mbOk], 0);
                Exit;
              end;
              aMedOrder :=
                TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
              idxOrder := ii;
            end;
      if idxOrder = -1 then
        DefMessageDlg('The Ward Stock Unique ID does not currently exist in the'
          + #13 +
          'highlighted order!', mtError, [mbOk], 0);
    end

    else
      //aMedOrder := GetIVOrder(TBCMA_IVBags(tempNode.Data).UniqueID);
      with VisibleMedList do
        for ii := 0 to count - 1 do
          with TBCMA_MedOrder(VisibleMedList[ii]) do
            if OrderNumber = TBCMA_IVBags(tempNode.Data).ordernumber then
              aMedOrder := TBCMA_MedOrder(VisibleMedList[ii]);

    if aMedOrder <> nil then begin
      AddComment(aMedOrder, TBCMA_IVBags(tempNode.Data));
      RebuildBagDetail(True);
    end;

  end;
end;

procedure TfrmMain.AddToolsApps;
var
  MenuItem: TMenuItem;
  x: integer;
begin
  for x := BCMA_SiteParameters.ToolsApps.Count - 1 downto 0 do begin
    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := Piece(BCMA_SiteParameters.ToolsApps[x], '=', 1);
    MenuItem.OnClick := ToolsAppClick;
    MenuItem.Tag := x;
    mnuMainMenu.Items[4].Insert(0, MenuItem);
  end;

end;

function TfrmMain.GetWardStockOrder(Add, Sol: TStringList): TStringList;
var
  ii, jj: integer;
  OrdMatch: Boolean;
  tempAdd, tempSol: TStringList;

begin
  tempAdd := TStringList.Create;
  tempSol := TStringList.Create;
  Result := TStringList.Create;

  for ii := 0 to Add.Count - 1 do
    tempAdd.Add(piece(Add[ii], '^', 2));

  for ii := 0 to Sol.Count - 1 do
    tempSol.Add(piece(Sol[ii], '^', 2));

  with VisibleMedList do
    for ii := 0 to VisibleMedList.count - 1 do begin
      OrdMatch := True;
      with TBCMA_MedOrder(VisibleMedList[ii]) do
        if Additives.Count = tempAdd.count then
          if Solutions.count = TempSol.count then begin
            for jj := 0 to Additives.count - 1 do
              if tempAdd.IndexOf(piece(Additives[jj], '^', 2)) = -1 then begin
                OrdMatch := False;
                Break;
              end;

            for jj := 0 to Solutions.count - 1 do begin
              if OrdMatch = False then
                Break;
              if tempSol.IndexOf(piece(Solutions[jj], '^', 2)) = -1 then begin
                OrdMatch := False;
                Break;
              end;
            end;
            if OrdMatch = True then begin
              if scanstatus <> 'G' then
                Result.Add(IntToStr(ii));
            end
          end;
    end;
  if Result.count = 0 then
    DefMessageDlg('The additives and/or solutions that were scanned do not match'
      + #13 +
      'any current order(s) on the Virtual Due List.' + #13#13 +
      'OR' + #13#13 +
      'The additives and/or solutions were scanned more than once.',
      mtInformation, [mbOK], 0);
end;

function TfrmMain.SelectOrderID(OrderList: TStringList; DefaultOnlyOne:
  Boolean): integer;
var
  ii: integer;
  HeaderSection: THeaderSection;
  frmMultOrder: TfrmMultOrder;
begin
  result := -1;
  frmMultOrder := nil;

  if OrderList.Count = 0 then begin
    DefMessageDlg('Scanned Item Not Found in the Virtual Due List or' + #13 +
      'the administration has already been given!', mtError, [mbOK], 0);
    exit;
  end;

  if (OrderList.Count = 1) and (DefaultOnlyOne = True) then begin
    Result := StrToInt(piece(OrderList[0], ';', 1));
    exit;
  end;

//  with TfrmMultOrder.create(application) do try
  try
    frmMultOrder := TfrmMultOrder.Create(application);
    with frmMultOrder do begin
      with TBcma_MedOrder(VisibleMedList[StrToInt(piece(OrderList[0], ';', 1))])
      do
      //      lblDispensedDrug.caption := lblDispensedDrug.caption + ' ' +
      //        ActiveMedication;
        lblDispensedDrug.caption := ActiveMedication;

      hdrMultiOrder.Sections.Clear;
//    with hdrMultiOrder do
      case lstCurrentTab of
        ctUD: begin
            HelpContext := 111; // Multiple Orders for Unit Dose; rpk 10/6/2010
            grdMultiOrder.ColCount := length(VDLColumnTitles);
            for ii := 0 to length(VDLColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := VDLColumnTitles[TVDLColumnTypes(ii)];
                Sections.Items[ii].Width :=
                  VDLColumnWidths[TVDLColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := VDLColumnTitles[TVDLColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := VDLColumnWidths[TVDLColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;
        ctPB: begin
            HelpContext := 148; // Multiple Orders for IVP/IVPB; rpk 10/6/2010
            grdMultiOrder.ColCount := length(lstPBColumnTitles);
            for ii := 0 to length(lstPBColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := lstPBColumnTitles[lstPBColumnTypes(ii)];
                Sections.Items[ii].Width := lstPBColumnWidths[lstPBColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := lstPBColumnTitles[lstPBColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := lstPBColumnWidths[lstPBColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;
        ctIV: begin
            grdMultiOrder.ColCount := length(lstIVColumnTitles);
            for ii := 0 to length(lstIVColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := lstIVColumnTitles[lstIVColumnTypes(ii)];
                Sections.Items[ii].Width :=
                  lstIVColumnWidths[lstIVColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := lstIVColumnTitles[lstIVColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := lstIVColumnWidths[lstIVColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;

      end;

      sgInit(grdMultiOrder, 1, OrderList.Count); // rpk 9/10/2010

      for ii := 0 to OrderList.Count - 1 do begin
      //          if TBCMA_MedOrder(OrderList.Objects[ii]).ScanStatus <> 'G' then
        lstMultiOrder.Items.Add(piece(OrderList[ii], ';', 1));
//        grdMultiOrder.Cells[0, ii] := piece(OrderList[ii], ';', 1);
      end;

      ii := ShowModal;
      if ii <> mrCancel then begin
        result := ii - 100;
        if TBcma_MedOrder(VisibleMedList[StrToInt(piece(OrderList[0], ';',
            1))]).OrderTypeID = otIV then
          result := StrToInt(piece(OrderList[result], ';', 1));
      end
      else
        result := -1;
    end;
  finally
    frmMultOrder.free;
  end;
end; // SelectOrderID

procedure TfrmMain.fraIV1mnuIVHistoryPopup(Sender: TObject);
var
  TempNode: TTreeNode;
begin
  TempNode := fraIV1.tvwIVHistory.Selected;

  if TempNode.Level = 1 then begin

    if LimitedAccess or ReadOnly then
      mnuTakeActionOnBag.Enabled := False {JK 5/29/2008}
    else begin
      with lstUnitDose do
        if ((TBCMA_IVBags(tempNode.data).ScanStatus = 'S') or
          (TBCMA_IVBags(tempNode.data).ScanStatus = 'I')) and
          (Pos(TBCMA_IVBags(tempNode.data).UniqueID, 'WS') = 0) then

          mnuTakeActionOnBag.Enabled := True
        else
          mnuTakeActionOnBag.Enabled := False; {JK 5/6/2008}
    end;

  end;

end;

procedure TfrmMain.fraIV1mnuMissingDoseClick(Sender: TObject);
var
  TempNode: TTreeNode;
begin
  TempNode := fraIV1.tvwIVHistory.Selected;
  if TempNode.Level = 1 then begin
    with lstUnitDose do
      ScannedInput := TBCMA_IVBags(tempNode.data).UniqueID;
    EnterMissingDose(VisibleMedList[lstUnitDose.ItemIndex],
      TBCMA_IVBags(tempNode.data));
    ScannedInput := ''
  end;
  RebuildIVOrderHistory(True,
    TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber);
end;

procedure TfrmMain.fraIV1sgIVBagDetailDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
type
  TempArray = array of string;
var
  x, y: Integer;
  //    CellRight,
  //    TitleCount: integer;
  TextString: string;
  //  CurrentFontColor: TColor;
  //  ARect: TRect;
  outText: string;
begin
  if (ARow = 0) or (ARow > VisibleMedList.Count) then
    Exit;

  // avoid repetitive drawing once cell is filled.
  if fraIV1.sgIVBagDetail.Cells[ACol, ARow] <> '' then
    Exit;

  //  Index := ACol - 1;
  if VisibleMedList.Count > 0 then
    with fraIV1.sgIVBagDetail do begin

      //      ARect := Rect;
      //      ARect.Top := Rect.Top + Top;
      //      ARect.Left := Rect.Left + Left;
      //      ARect.Right := ARect.Right + Rect.Right - Rect.Left;
      //      ARect.Bottom := ARect.Top + Rect.Bottom - Rect.Top;
      //      CurrentFontColor := Canvas.Font.Color;
      //
      //      with Canvas do
      //      begin
      //        FillRect(ARect);
      //        Pen.Color := clSilver;
      //        MoveTo(ARect.Left, ARect.Bottom - 1);
      //        LineTo(ARect.Right, ARect.Bottom - 1);
      //      end;

      //      CellRight := -3;

      //      TitleCount := fraIV1.hdrIvBagDetail.Sections.Count;

      //      for x := 0 to TitleCount - 1 do
      //      begin
      //        CellRight := CellRight + fraIV1.hdrIVBagDetail.Sections[x].Width;
      //        Canvas.MoveTo(CellRight, ARect.Bottom - 1);
      //        Canvas.LineTo(CellRight, ARect.Top);
      //      end;

      //      for x := 0 to TitleCount - 1 do
      //      begin
      //        if x > 0 then
      //          ARect.Left := ARect.Right + 2
      //        else
      //          ARect.Left := 2;

      //        ARect.Right := ARect.Left + fraIV1.hdrIVBagDetail.Sections[x].Width
      //          - 6;

      OutText := '';
      x := ACol;
      y := ARow;
      if CurrentBagID <> nil then begin
        if y < TBCMA_IVBags(CurrentBagID).BagDetails.Count then
          TextString := TBCMA_IVBags(CurrentBagID).BagDetails[y]
        else
          TextString := '';
      end
      else
        TextString := '';

      //        with TBCMA_IVBags(CurrentBagID).BagDetails[x] do
      if TextString > '' then begin
        case x of
          0: begin
              OutText := DisplayVADateYearTime(piece(TextString, '^', 1));
            end;
          1: begin
              OutText := piece(TextString, '^', 2);
            end;
          2: begin
              OutText := piece(TextString, '^', 3);
            end;
          3: begin
              OutText := piece(TextString, '^', 4);
            end;
        end;

        //          case x of
        //            0, 1, 2:
        //              DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect,
        //                DT_END_ELLIPSIS or DT_NOPREFIX);
        //            3:
        //              DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect,
        //                DT_WORDBREAK or DT_NOPREFIX);
        //          end;

        Cells[ACol, ARow] := OutText;
      end;

      //        Canvas.Font.Color := CurrentFontColor;
      //        ARect.Right := ARect.Right + 4;
      //      end;
    end;
  //  with fraIV1 do
  //  begin
  //    if BagDetailItemsHeight > lstIVBagDetail.Height then
        //sbrBagDetail.Enabled := True
  //    else
        //sbrVDL.Enabled := False;
  //  end;

end;

procedure TfrmMain.fraIV1sgIVBagDetailEnter(Sender: TObject);
var
  CanSelect: Boolean;
begin
  with fraIV1.sgIVBagDetail do begin
    if (Row > 0) and (RowCount > 1) then
      fraIV1sgIVBagDetailSelectCell(Sender, Col, Row, CanSelect);
  end;
end;

// fraIV1sgIVBagDetailDrawCell

procedure TfrmMain.fraIV1sgIVBagDetailSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  fraIV1.SelectedIVX := ACol;
  fraIV1.SelectedIVY := ARow;
end;

procedure TfrmMain.actionToolsOptionsExecute(Sender: TObject);
begin
  with TfrmOptions.Create(Application) do begin
    if BCMA_Broker <> nil then
      with BCMA_Broker do
        chkDebug.Checked := DebugMode;
    ShowModal;
  end;
end;

procedure TfrmMain.RebuildBagDetail(Reload: Boolean);
var
  zResult, textstring: string;
  icol, irow, icnt, ilen: integer;
begin
  if CurrentBagID = nil then
    exit;
  with fraIV1 do
    with TBCMA_IVBags(CurrentBagID) do begin
      if tvwIVHistory.Visible = False then
        exit;
      lstIVBagDetail.Visible := False;
      lstIVBagDetail.Clear;
      sgIVBagDetail.Hide;
      sgFree(-1, -1, sgIVBagDetail);

      with lblBagDetail do begin
        Caption := 'Retrieving Bag Details...';
        Repaint;
      end;
      with stBagDetail do begin
        Caption := 'Retrieving Bag Details...';
        Repaint;
      end;
      if Reload = True then
        zResult := GetBagDetails(UniqueId, OrderNumber)
      else
        zResult := '';
      if piece(zResult, '^', 1) = '-1' then begin
        lblBagDetail.Caption := piece(zResult, '^', 2);
        stBagDetail.Caption := piece(zResult, '^', 2);
      end
      else begin
        ilen := length(lstBagDetailColumnTitles);
        sgIVBagDetail.ColCount := ilen;
        SelectedIVX := 0;
        SelectedIVY := 1;

        for icol := 0 to ilen - 1 do
          sgIVBagDetail.Cells[icol, 0] :=
            lstBagDetailColumnTitles[lstBagDetailColumnTypes(icol)];
        // rpk 10/15/2009

        icnt := TBCMA_IVBags(CurrentBagID).BagDetails.Count;
        //        sgIVBagDetail.RowCount := TBCMA_IVBags(CurrentBagID).BagDetails.Count + 1;
        // sgIVBagDetail.RowCount := icnt + 1;
        sgInit(sgIVBagDetail, 1, icnt); // rpk 9/10/2010
        //        for i := 0 to TBCMA_IVBags(CurrentBagID).BagDetails.Count - 1 do begin
        for irow := 0 to icnt - 1 do begin
          TextString := TBCMA_IVBags(CurrentBagID).BagDetails[irow];
          //          lstIVBagDetail.Items.Add(TBCMA_IVBags(CurrentBagID).BagDetails[i]);
          lstIVBagDetail.Items.Add(TextString);

          if TextString > '' then begin
            sgIVBagDetail.Cells[0, irow + 1] :=
              DisplayVADateYearTime(piece(TextString, '^', 1));
            sgIVBagDetail.Cells[1, irow + 1] := piece(TextString, '^', 2);
            sgIVBagDetail.Cells[2, irow + 1] := piece(TextString, '^', 3);
            sgIVBagDetail.Cells[3, irow + 1] := piece(TextString, '^', 4);
          end;
        end; // for irow

        if ScreenReaderSystemActive then begin
          lstIVBagDetail.SendToBack;
          lstIVBagDetail.Hide;
          lstIVBagDetail.TabStop := False;
          hdrIVBagDetail.Hide;

          sgIVBagDetail.align := alClient;
          sgIVBagDetail.BringToFront;
          sgIVBagDetail.TabStop := True;
          sgIVBagDetail.Show;
          if sgIVBagDetail.CanFocus then
            sgIVBagDetail.SetFocus;
          sgIVBagDetail.Repaint;
        end
        else begin
          sgIVBagDetail.SendToBack;
          sgIVBagDetail.Hide;
          sgIVBagDetail.TabStop := False;

          hdrIVBagDetail.Show;
          hdrIVBagDetail.Repaint;
          lstIVBagDetail.align := alClient;
          lstIVBagDetail.BringToFront;
          lstIVBagDetail.TabStop := True;
          lstIVBagDetail.Show;
          if lstIVBagDetail.CanFocus then
            lstIVBagDetail.SetFocus;
          lstIVBagDetail.Repaint;
        end;
      end; // else valid result
    end; // with TBCMA_IVBags(CurrentBagID) do

end;

procedure TfrmMain.ForceVDLRebuild;
begin
  DefMessageDlg('Either the order status or the current scan status displayed on the VDL '
    +
    'does not match the status recorded in VistA!  This may be due to someone ' +
    'editing the order in VistA or another individual scanning the medication.' + #13
    + #13 +
    'Your current action will be cancelled and the VDL will be refreshed to ' +
    'reflect the most current order information.', mtInformation, [mbOK], 0);
  RebuildVirtualDueList(True);
end;

procedure TfrmMain.fraIV1hdrIVBagDetailSectionResize(
  HeaderControl: THeaderControl; Section: THeaderSection);
var
  ii,
    TotalWidth,
    TempWidth: Integer;

begin
  RedrawSuspend(fraIV1.hdrIVBagDetail.handle);

  //set the appropriate section in the array to the new size
  lstBagDetailColumnWidths[lstBagDetailColumnTypes(section.Index)] :=
    fraIV1.hdrIVBagDetail.Sections.Items[section.Index].Width;

  //  //set changed to true so the new values are saved when the user exits
  //  BCMA_UserParameters.Changed := True;

  TotalWidth := 0;

  with fraIV1.hdrIVBagDetail.Sections do begin
    for ii := 0 to Count - 1 do begin
      TempWidth := ((Count - (ii + 1)) * 5);
      items[ii].maxwidth := fraIV1.hdrIVBagDetail.width - (TempWidth +
        TotalWidth);
      TotalWidth := TotalWidth + Items[ii].Width
    end;
  end;

  ReDrawActivate(fraIV1.hdrIVBagDetail.handle);

end;

procedure TfrmMain.fraIV1hdrIVBagDetailMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ii,
    TotalWidth,
    TempWidth: Integer;

begin
  RedrawSuspend(fraIV1.hdrIVBagDetail.handle);
  with fraIV1 do
    for x := 0 to length(lstBagDetailColumnTitles) - 1 do
      hdrIVBagDetail.Sections.Items[x].Width :=
        lstBagDetailColumnWidths[lstBagDetailColumnTypes(x)];

  TotalWidth := 0;

  with fraIV1.hdrIVBagDetail.Sections do begin
    for ii := 0 to Count - 1 do begin
      TempWidth := ((Count - (ii + 1)) * 5);
      items[ii].maxwidth := fraIV1.hdrIVBagDetail.width - (TempWidth +
        TotalWidth);
      TotalWidth := TotalWidth + Items[ii].Width
    end;
  end;

  ReDrawActivate(fraIV1.hdrIVBagDetail.handle);

  RebuildBagDetail(False);
end;

procedure TfrmMain.actionMedTabUDExecute(Sender: TObject);
begin
  with pgctrlVirtualDueList do
    if ActivePage <> tbshtUnitDose then begin
      ActivePage := tbshtUnitDose;
      pgctrlVirtualDueListChange(Self);
    end;
end;

procedure TfrmMain.actionMedTabPBExecute(Sender: TObject);
begin
  with pgctrlVirtualDueList do
    if ActivePage <> tbshtIVPIVPB then begin
      ActivePage := tbshtIVPIVPB;
      pgctrlVirtualDueListChange(Self);
    end;

end;

procedure TfrmMain.actionMedTabIVExecute(Sender: TObject);
begin
  with pgctrlVirtualDueList do
    if ActivePage <> tbshtIV then begin
      ActivePage := tbshtIV;
      pgctrlVirtualDueListChange(Self);
    end;
end;

procedure TfrmMain.actionReportsPRNEffectivenessListExecute(Sender: TObject);
begin
  PRNEffectivenessListReport(BCMA_Patient.IEN);
end;

procedure TfrmMain.actionViewPatientDemographicsExecute(Sender: TObject);
begin
  showPatientDemographics;
end;

procedure TfrmMain.actionMOBExecute(Sender: TObject);
var
  BcmaOrder: IBcmaOrder;
  eMSG: string;
begin
  if BCMA_User.OrderRole <> 2 then begin
    DefMessageDlg('Only a user defined as a Nurse in CPRS may use the Med Order Button', mtError, [mbOK], 0);
    exit;
  end;
  with TfrmCPRSOrderManager.create(application) do try
    try
      actionMOB.Enabled := false;
      Screen.Cursor := crHourglass;
      BcmaOrder := CreateOleObject('BcmaOrderCom.BcmaOrder') as IBcmaOrder;
      BcmaOrder.ConnectToServer(ServerIP, StrToInt(PortString));
      aBCMAOrder := BCMAOrder;
      showmodal;
    except
      on e: EOleException do begin
        eMSG := 'An error has occured creating the CPRS object.' + #13 +
          'Error: ' + e.message + #13 +
          'Unable to launch the Order Manager';
        DefMessageDlg(eMSG, mtError, [mbOK], 0);

      end;
    end;
  finally
    if lstCurrentTab = ctIV then
      RebuildVirtualDueList(True)
    else if lstCurrentTab = ctCS then
      with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
        RebuildMe;

    Screen.Cursor := crDefault;
    actionMOB.Enabled := True;

    // NOTE: Check on need to free OleObject.
    if BCMAOrder <> nil then
      BCMAOrder := nil;
    free;
  end;
end;

procedure TfrmMain.MessageHandler(var Msg: TMsg; var Handled: Boolean);
begin
  case Msg.Message of
    WM_KEYDOWN, WM_LBUTTONDOWN,
      WM_RBUTTONDOWN, WM_MBUTTONDOWN:
      IdleTime := Now;

    //This section looks for Alt-x Keys, or accelerator(underscore) keys.  For some reason the action
    //toolbar displays the undscore character, implying an accelerator key, but the combination keystroke
    //does not work.  There doesn't appear to be away to hide the accelerator key without turning
    //it off on the actionmanager, wihch thn turns it off for the menu option, which we do want.
    //This just scans the entered keystrokes, checks for certain key combinations and executes the
    //proper code, giving the impression that the accelerator keys work on the button bar.
    WM_SYSKEYDOWN:
      if Screen.ActiveForm = frmMain then begin
        //Alt-I
        if (Msg.wParam = 73) and (actionDueListMissingDose.Enabled) then
          actionDueListMissingDose.Execute;
        //Alt-L
        if (Msg.wParam = 76) and (actionReportsMedLog.Enabled) then
          actionReportsMedLog.Execute;
        //Alt-E
        if (Msg.wParam = 69) and (actionReportsMAH.Enabled) then
          actionReportsMAH.Execute;
        //Alt-G
        if (Msg.wParam = 71) and (actionViewAllergies.Enabled) then
          actionViewAllergies.Execute;
        //Alt-M
        if (Msg.wParam = 77) and (actionMOB.Enabled) then
          actionMOB.Execute;
      end;
  end;
  Handled := false;
end;

procedure TFrmMain.CheckIdleTimeout;
var
  x: TDateTime;
  Diff: Extended;
  Result: Integer;
  //  h: THandle;
begin
  if frmmain.visible = False then
    exit;

  x := now;

  //  if (screen.activeform = frmmain) or (not CloseForms(False)) then
  //  begin
  Diff := trunc((x - IdleTime) * 24 * 60);
  //
  // DEBUG ONLY
  //
  //  Diff := BCMA_SiteParameters.IdleTimeout + 5;
  //
  if Diff >= BCMA_SiteParameters.IdleTimeout then begin
    if (screen.activeform = frmmain) or (not CloseForms(False)) then
      with TfrmTimeOut.create(application) do try
        StatusBarTimer.Enabled := false;
        SetForegroundWindow(frmMain.Handle);

        //        h := FindWindow('TfrmInputQry', nil);
        //        if h > 0 then SendMessage(h, WM_CLOSE, 0, 0);

        Result := ShowModal;

        case Result of
          mrAbort: begin
              IdleTime := Now;
              StatusBarTimer.Enabled := True;
              ForceForegroundWindow(screen.ActiveForm.Handle);
            end;
          mrCancel: begin
              StatusBarTimer.Enabled := False;
              Closing := True;
              //              h := FindWindow('TfrmInputQry', nil);
              //              if h > 0 then SendMessage(h, WM_CLOSE, 0, 0);
              CloseForms(True);
              //frmmain.Close;
              Application.Terminate;
            end;
        end;
      finally
        Free;
      end
  end;
  //  end;
end;

procedure TfrmMain.actionMarkRemovedExecute(Sender: TObject);
begin
  MarkNotGiven('RM');
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  actionFileClosePatient.Execute;

  if CloseFrm = True then
    CanClose := true
  else begin
    CloseFrm := True;
    CanClose := False;
  end;
end;

procedure TfrmMain.fraIV1tvwIVHistoryClick(Sender: TObject);
begin
  ScannerActivate;
end;

procedure TfrmMain.CreateWardStock(ScannedDrugIEN: string; var CurFlowUID:
  string; var toBeWardStock: Pointer);
var
  ii, idxOrder: integer;
  aMedOrder: TBCMA_MedOrder;
  ivOrders: TStringList;
  aIVBag: TBCMA_IVBags;
  InfusingBags: Boolean;
  OkToLog: TUASLogAction;
  CheckInfusingBagsBailOut: Boolean;

  isPRNCancelled: Boolean; {JK 8/15/2008}
  VitalsInfo, PainInfo: string; {JK 8/15/2008}

  PassFail: Boolean; {JK 9/17/2008}

begin
  aMedOrder := nil;
  OkToLog := LA_OkToLog; // rpk 9/24/2010;  LA_Cancelled was incorrect
  PassFail := False; // rpk 4/15/2009
  CheckInfusingBagsBailOut := False; // rpk 10/26/2010
  InfusingBags := False; // rpk 10/26/2010
  isPRNCancelled := False; // rpk 10/26/2010

  if toBeWardStock = nil then
    with TfrmWardStock.create(application) do try
      if UnableToScan or (ScanDrug(ScannedDrugIEN, (lstCurrentTab = ctPB))) then begin
        if showModal = mrOK then begin
          frmMain.UAS_LogState := LA_OkToLog;
          Self.Repaint;
          IVOrders := GetWardStockOrder(zAdditives, zSolutions);
          if IVOrders.Count = 0 then begin
            frmMain.UAS_LogState := LA_Cancelled; {JK 7/17/2008 CQ #116}
            Exit;
          end;

          {if the user selected unable to scan via the right click, they obviously
          selected an order. Delete any other orders that we may have matched
          that don't have the same order number
          }
          if UnableToScan and (lstUnitDose.ItemIndex <> -1) then begin
            ii := 0;
            while ii <= IVOrders.count - 1 do

              //          for ii := 0 to IVOrders.Count - 1 do
              if
                (TBCMA_MedOrder(VisibleMedList[StrToInt(IVOrders[ii])]).OrderNumber
                <>
                TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber) then
                IVOrders.Delete(ii)
              else
                inc(ii);
          end;

          {if our above loop removed all orders, display a message and exit}
          if IVOrders.Count = 0 then begin
            DefMessageDlg('The Ward Stock bag you created does not match ' +
              'the order you selected. The bag and administration will be canceled.',
              mtError, [mbOK], 0);
            Exit;
          end;

          idxOrder := SelectOrderID(IVOrders, True);
          if idxOrder = -1 then
            Exit;
          aMedOrder := TBCMA_MedOrder(VisibleMedList.Items[idxOrder]);

          {
                    if UnableToScan and ((aMedOrder.OrderNumber) <>
                      (TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber)) then
                    begin
                      DefMessageDlg('The Ward Stock bag you created does not match ' +
                        'the order you selected. The bag and administration will be canceled.',
                        mtError, [mbOK], 0);
                      exit;
                    end;
          }
          if (aMedOrder.OrderStatus = 'D') or (aMedOrder.OrderStatus = 'E')
            or
            (aMedOrder.OrderStatus = 'H') then begin
            DefMessageDlg('The order for this bag has either been DC''d, has expired, or is on Provider Hold. '
              +
              #13 + 'A ward stock bag can not be created.', mtError,
              [mbOK], 0);
            exit;
          end;

          with aMedOrder do begin
            WardStock := True;
          end;

          {if IV tab, Find the History for this bag}
          if lstCurrentTab = ctIV then begin

            RebuildIVOrderHistory(False, aMedOrder.OrderNumber);

            //            if CheckInfusingBags('', lstUnitDose, '', CurFlowUID, InfusingBags, CheckInfusingBagsBailOut) then
            if CheckInfusingBags('', '', CurFlowUID, InfusingBags,
              CheckInfusingBagsBailOut) then

              if CurFlowUID <> '' then begin
                ScannedInput := CurFlowUID;
                toBeWardStock := VisibleMedList.Items[idxOrder];
                lstUnitDose.ItemIndex := idxOrder;
                RebuildVirtualDueList(False);
                Exit
              end
              else begin
                lstUnitDose.ItemIndex := idxOrder;
                RebuildVirtualDueList(False);
                RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
                Exit;
              end
          end;
        end
        else
          frmMain.UAS_LogState := LA_CANCELLED;
      end;
    finally
      OkToLog := frmMain.UAS_LogState;
      free;
    end
  else begin
    aMedOrder := TBCMA_MedOrder(toBeWardStock);
  end;

  {JK - 11/1/2008}
  if CheckInfusingBagsBailOut then begin
    UAS_LogState := LA_CANCELLED;
    DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK], 0);
    RebuildVirtualDueList(False);
    if lstCurrentTab = ctIV then
      RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
    Exit;
  end;

  //if we have an order, log it
  ScannedInput := '';
  if aMedOrder <> nil then begin
    with aMedOrder do begin
      UserComments := '';
      if DisplayInstructions = True then
        if DefMessageDlg(SpecialInstructions, mtInformation, [mbOK,
          mbCancel], 0) <> idOK then begin
          RebuildVirtualDueList(False);
          if lstCurrentTab = ctIV then
            RebuildIVOrderHistory(True, OrderNumber);
          exit;
        end;

      //-- 8/27/2008 BUGGGGGGGGGGG in procedure CreateWardStock
//      if Status = 1 then
//        ConfirmOrder(aMedOrder, True, isPRNCancelled, VitalsInfo, PainInfo);
//      if isPRNCancelled then
//        Exit;

      // Moved statement up to set WardStock before first call to getValidOrder (validorder)
      WardStock := True; // rpk 10/25/2010;

      if aMedOrder.ValidOrder then begin
        OKToLog := LA_OkToLog; // Is this correct state here ???: rpk 9/24/2010
        if Status = 1 then begin
          isPRNCancelled := False;
          //if (VitalsInfo = '') and (aMedOrder.ScheduleTypeID = stPRN) and (aMedOrder.ReasonGivenPRN = '') then
          if (aMedOrder.ScheduleTypeID = stPRN) and (aMedOrder.ReasonGivenPRN =
            '') then
            ConfirmOrder(aMedOrder, False, isPRNCancelled, VitalsInfo,
              PainInfo);
          //-else
          //-  ConfirmOrder(aMedOrder, True, isPRNCancelled, VitalsInfo, PainInfo);
          if isPRNCancelled then begin
            DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK],
              0);
            UAS_LogState := LA_Cancelled;
            Exit;
          end;
        end; // if Status = 1
      end; // if ValidOrder

//      WardStock := True;  // moved up above ValidOrder call
    end; // with aMedOrder

    aIVBag := TBCMA_IVBags.Create(BCMA_Broker);

    with aIVBag do begin

      Additives.Assign(aMedOrder.Additives);
      Solutions.Assign(aMedOrder.Solutions);

      CurrentBagID := aIVBag;

      if aMedOrder.ValidOrder then begin

        if OkToLog = LA_OkToLog then {JK 5/16/2008}

          if aMedOrder.Status = 1 then begin
            ConfirmOrder(aMedOrder, True, isPRNCancelled, VitalsInfo, PainInfo);
            if not isPRNCancelled then
              if lstCurrentTab = ctIV then
                PassFail := aMedOrder.LogOrder(mtMedPass, 'I', aIVBag)
              else
                PassFail := aMedOrder.LogOrder(mtMedPass, 'G', aIVBag)
          end
          else begin
            if lstCurrentTab = ctIV then
              PassFail := aMedOrder.LogOrder(mtMedPass, 'I', aIVBag)
            else
              PassFail := aMedOrder.LogOrder(mtMedPass, 'G', aIVBag)
          end;

        {JK 9/17/2008}
        if aMedOrder.Status = 1 then begin
          if isPRNCancelled then
            DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK],
              0) {JK 5/12/2008}
          else if PassFail = False then
            DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK],
              0); {JK 5/12/2008}
        end;

        //
        // NOTE: Check this.  Will it break any code?
        //
        if CurrentBagID = aIVBag then // rpk 10/25/2010
          CurrentBagID := nil; // rpk 10/25/2010

        aIVBag.Clear;
//        aIVBag.Free;
        FreeAndNil(aIVBag); // rpk 10/25/2010
      end
      else begin
        if aMedOrder.Status = -2 then begin
          ForceVDLRebuild;
          Exit;
        end
        else begin
          DefMessageDlg(aMedOrder.StatusMessage, mtInformation, [mbOK], 0);
          Exit;
        end;
      end; {if ValidOrder}
    end;

    for ii := 0 to VisibleMedList.count - 1 do
      with TBCMA_MedOrder(VisibleMedList[ii]) do
        if OrderNumber = aMedOrder.OrderNumber then begin
          lstUnitDose.ItemIndex := ii;
          break;
        end;

    RebuildVirtualDueList(False);
    if lstCurrentTab = ctIV then
      RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
  end;
end; // CreateWardStock

procedure TfrmMain.fraIV1lstIVBagDetailClick(Sender: TObject);
begin
  ScannerActivate;
end;

procedure TfrmMain.actionDueListTakeActionOnWSExecute(Sender: TObject);
var
  x: char;
begin
  if CurrentBagID = nil then // rpk 10/25/2010
    Exit;
  ScannedInput := TBCMA_IVBags(CurrentBagID).UniqueID;
  edtScannerInput.Clear;
  edtScannerInput.Text := TBCMA_IVBags(CurrentBagID).UniqueID;
  x := chr(VK_RETURN);
  edtScannerInputKeyPress(edtScannerInput, x);
end;

procedure TfrmMain.SetIVHistMenus;
var
  TempNode: TTreeNode;
  //  p: TPoint;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;
begin
  TempNode := fraIV1.tvwIVHistory.Selected;
  with tempNode do
    if tempNode <> nil then begin
      if (level = 1) then begin
        aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
        with aMedOrder do begin
          if aMedOrder = nil then begin
            DefMessageDlg('This bag was not found in any of the orders currently'
              + #13 +
              'displayed on the VDL.  This could indicate a problem with the data.  An action ' + #13
              +
              'on this bag can not be taken at this time.', mtError,
              [mbOk], 0);
            exit;
          end;
          exit;
          ScanStat :=
            getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
          if (ScanStat <> ssAvailable) and (ScanStat <> ssMissed) then
            fraIV1.mnuAddComment.Enabled := True
          else
            fraIV1.mnuAddComment.Enabled := False;

          if (pos('WS', UpperCase(TBCMA_IVBags(tempNode.Data).UniqueID))
            <> 0)
            and
            ((ScanStat = ssInfusing) or (ScanStat = ssStopped) or
            (ScanStat = ssHeld) or (ScanStat = ssRefused)) then begin
            //mnuTakeActionOnWS.Visible := true;
            mnuTakeActionOnBag.Visible := True; {JK 5/6/2008}
          end
          else begin
            //mnuTakeActionOnWS.Visible := false;
            mnuTakeActionOnBag.Visible := False; {JK 5/6/2008}
          end;

          if (OrderStatus = 'A') or (OrderStatus = 'R') or
            (OrderStatus = 'O') then begin
            if ((ScanStat <> ssComplete) and (ScanStat <> ssInfusing)
              and
              (ScanStat <> ssMissed)) and
              (pos('WS', UpperCase(TBCMA_IVBags(tempNode.Data).UniqueID))
              = 0) then
              fraiv1.mnuMissingDose.Enabled := True
            else
              fraiv1.mnuMissingDose.Enabled := False;

            case ScanStat of
              ssStopped,
                ssAvailable, ssMissed, ssNotGiven: begin
                  with fraiv1 do begin
                    mnuMark.Enabled := True;
                    mnuHeld.Enabled := True;
                    mnuRefused.Enabled := True;
                  end;
                end;
              ssHeld:
                with fraiv1 do begin
                  mnuMark.Enabled := True;
                  mnuRefused.Enabled := True;
                  mnuHeld.Enabled := False;
                end;
              ssRefused, ssComplete:
                with fraiv1 do begin
                  mnuMark.Enabled := False;
                  ;
                end;
              ssInfusing:
                with fraiv1 do
                  mnuMark.Enabled := False;
            end;
          end
          else if (OrderStatus = 'H') or (OrderStatus = 'D') or
            (OrderStatus = 'E') then begin
            case ScanStat of
              ssComplete,
                ssInfusing,
                ssStopped,
                ssHeld:
                with fraIV1 do begin
                  mnuMark.Enabled := False;
                  mnuMissingDose.Enabled := False;
                end;
              ssAvailable,
                ssMissed,
                ssRefused:
                with fraIV1 do begin
                  mnuMark.Enabled := True;
                  mnuMissingDose.Enabled := False;
                  mnuRefused.Enabled := False;
                  mnuHeld.Enabled := True;
                end;
            end;
          end;
        end;
      end
      else begin
        //            tbtnMissingDose.Enabled := false;
      end

    end;
end;

procedure TFrmMain.DisplayIgnoredAdmins(TStringIn: TStringList);
var
  ii: integer;
  HeaderSection: THeaderSection;
  frmMultOrder: TfrmMultOrder;
begin
  frmMultOrder := nil;
  if TStringIn.Count = 0 then
    exit;

//  with TfrmMultOrder.create(application) do try
  try
    frmMultOrder := TfrmMultOrder.create(application);
    with frmMultOrder do begin
      hdrMultiOrder.Sections.Clear;
//    with hdrMultiOrder do
      case lstCurrentTab of
        ctUD: begin
            HelpContext := 111; // Multiple Orders for Unit Dose; rpk 10/6/2010
            grdMultiOrder.ColCount := length(VDLColumnTitles);
            for ii := 0 to length(VDLColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := VDLColumnTitles[TVDLColumnTypes(ii)];
                Sections.Items[ii].Width :=
                  VDLColumnWidths[TVDLColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := VDLColumnTitles[TVDLColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := VDLColumnWidths[TVDLColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;
        ctPB: begin
            HelpContext := 148; // Multiple Orders for IVP/IVPB; rpk 10/6/2010
            grdMultiOrder.ColCount := length(lstPBColumnTitles);
            for ii := 0 to length(lstPBColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := lstPBColumnTitles[lstPBColumnTypes(ii)];
                Sections.Items[ii].Width :=
                  lstPBColumnWidths[lstPBColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := lstPBColumnTitles[lstPBColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := lstPBColumnWidths[lstPBColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;
        ctIV: begin
            grdMultiOrder.ColCount := length(lstIVColumnTitles);
            for ii := 0 to length(lstIVColumnTitles) - 1 do begin
              with hdrMultiOrder do begin
                HeaderSection := Sections.Add;
                HeaderSection.Text := lstIVColumnTitles[lstIVColumnTypes(ii)];
                Sections.Items[ii].Width :=
                  lstIVColumnWidths[lstIVColumnTypes(ii)];
              end;
              with grdMultiOrder do begin
                Cells[ii, 0] := lstIVColumnTitles[lstIVColumnTypes(ii)]; // rpk 9/10/2010
                ColWidths[ii] := lstIVColumnWidths[lstIVColumnTypes(ii)]; // rpk 9/10/2010
              end;
            end;
          end;

      end;

      sgInit(grdMultiOrder, 1, TStringIn.Count); // rpk 9/10/2010
      for ii := 0 to TStringIn.Count - 1 do begin
        lstMultiOrder.Items.Add(TStringIn[ii]);
//      grdMultiOrder.Cells[0, ii] := piece(TStringIn[ii], ';', 1);
      end;

      caption := 'Information';
      lblDispensedDrug.Caption := '';
      lblSelectOrder.caption :=
        '&Held or Refused will NOT be applied to the following order(s):';
      tag := 1;
      ShowModal;
    end; // with frmMultOrder
  finally
    frmMultOrder.free;
  end;
end; // DisplayIgnoredAdmins

procedure TfrmMain.actionSortByMedSolUpdate(Sender: TObject);
begin
  with actionSortByMedSol do
    case lstCurrentTab of
      ctUD, ctCS:
        Visible := False;
    else
      Visible := True;
    end;
end;

procedure TfrmMain.actionSortByNurseUpdate(Sender: TObject);
begin
  with actionSortByNurse do
    case lstCurrentTab of
      ctCS:
        Visible := False;
    else
      Visible := True;
    end;

end;

procedure TfrmMain.actionSortByRouteUpdate(Sender: TObject);
begin
  with actionSortByRoute do
    case lstCurrentTab of
      ctCS:
        Visible := False;
    else
      Visible := True;
    end;
end;

procedure TfrmMain.actionSortByInfusionRateUpdate(Sender: TObject);
begin
  with actionSortByInfusionRate do
    case lstCurrentTab of
      ctUD, ctCS:
        Visible := False;
    else
      Visible := True;
    end;
end;

procedure TfrmMain.actionSortByActiveMedUpdate(Sender: TObject);
begin
  with actionSortByActiveMed do
    case lstCurrentTab of
      ctUD:
        Visible := True;
    else
      Visible := False;
    end;

end;

procedure TfrmMain.actionSortByDosageUpdate(Sender: TObject);
begin
  with actionSortByDosage do
    case lstCurrentTab of
      ctUD:
        Visible := True;
    else
      Visible := False;
    end;
end;

procedure TfrmMain.actionSortByHSMUpdate(Sender: TObject);
begin
  with actionSortByHSM do
    case lstCurrentTab of
      ctUD:
        Visible := True;
    else
      Visible := False;
    end;
end;

procedure TfrmMain.actionSortByAdminTimeUpdate(Sender: TObject);
begin
  with actionSortByAdminTime do
    case lstCurrentTab of
      ctUD, ctPB:
        Visible := True;
    else
      Visible := False;
    end;
end;

procedure TfrmMain.actionSortByLastActionUpdate(Sender: TObject);
begin
  with actionSortByLastAction do
    case lstCurrentTab of
      ctUD, ctPB:
        Visible := True;
    else
      Visible := False;
    end;
end;

procedure TfrmMain.actionSortByStatusUpdate(Sender: TObject);
var
  x: integer;
begin
  //this is called from here only. When the sort by menu is display, this code is called
  //and will check the appropriate menu option based off of the current sort column.
  with ActionManager do
    for x := 0 to ActionCount - 1 do
      if Actions[x].Category = '&Sort By' then
        case lstCurrentTab of
          ctUD:
            if (Actions[x] as TAction).Tag = lstValidUDSortColumns[SortColUD]
              then
              (Actions[x] as TAction).Checked := True
            else
              (Actions[x] as TAction).Checked := false;
          ctPB:
            if (Actions[x] as TAction).Tag = lstValidPBSortColumns[SortColPB]
              then
              (Actions[x] as TAction).Checked := True
            else
              (Actions[x] as TAction).Checked := false;
          ctIV:
            if (Actions[x] as TAction).Tag = lstValidIVSortColumns[SortColIV]
              then
              (Actions[x] as TAction).Checked := True
            else
              (Actions[x] as TAction).Checked := false;
        end;

  with actionSortByStatus do
    case lstCurrentTab of
      ctCS:
        Visible := False;
    else
      Visible := True;
    end;

end;

procedure TfrmMain.actionSortByTypeUpdate(Sender: TObject);
begin
  with actionSortByType do
    case lstCurrentTab of
      ctCS:
        Visible := False;
    else
      Visible := True;
    end;
end;

procedure TfrmMain.actionViewAllergiesUpdate(Sender: TObject);
begin
  actionViewAllergies.Enabled := (pnlMainForm.Visible and
    BCMA_Patient.Reactions);
end;

procedure TfrmMain.actionViewPatientDemographicsUpdate(Sender: TObject);
//var
//  Item: TActionClientItem;
begin
  actionViewPatientDemographics.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionMedTabUDUpdate(Sender: TObject);
begin
  actionMedTabUD.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionMedTabPBUpdate(Sender: TObject);
begin
  actionMedTabPB.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionMedTabIVUpdate(Sender: TObject);
begin
  actionMedTabIV.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionDueListAddCommentUpdate(Sender: TObject);
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;
begin
  if isRestricted or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count =
    0) or
    (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionDueListAddComment.enabled := False;
    exit;
  end;

  case lstCurrentTab of
    ctUD, ctPB: begin
        //If we are on the UD Tab, display rightclick, addcomment on VDL. Logical
        //place for code was here, but upon first pass thru this code,
        //PopUpAddComment.Visible := True didn't take, so moved it to
        //lstUnitDoseContextPopup which gets executed before we get here
        if ScreenReaderSystemActive then begin // rpk 9/11/2009
          with sgUnitDose do
            if (RowCount > 1) and (Row > 0) then begin
              with TBCMA_MedOrder(VisibleMedList[sgUnitDose.row - 1]) do
                if (OrderStatus = 'H') or (ScanStatus = 'U') then
                  actionDueListAddComment.Enabled := (ScanStatus = 'H')
                else
                  actionDueListAddComment.Enabled := not (ScanStatus = '');
            end
            else
              actionDueListAddComment.Enabled := False;
        end
        else begin
          with lstUnitDose do
            if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
              (VisibleMedList.count = 0) then
              actionDueListAddComment.Enabled := False
            else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex >
              -1)) then begin
              with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
                if (OrderStatus = 'H') or (ScanStatus = 'U') then
                  actionDueListAddComment.Enabled := (ScanStatus = 'H')
                else
                  actionDueListAddComment.Enabled := not (ScanStatus = '');
            end
            else
              actionDueListAddComment.Enabled := False;
        end;
      end;
    ctIV: begin
        //
        // add s508 code
        //
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if tempNode <> nil then
            if (level = 1) then begin
              aMedOrder :=
                TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
              with aMedOrder do
                if aMedOrder <> nil then begin
                  ScanStat :=
                    getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
                  if (ScanStat <> ssAvailable) and (ScanStat <> ssMissed) then
                    actionDueListAddComment.Enabled := True
                  else
                    actionDueListAddComment.Enabled := False;
                end;
            end
            else
              actionDueListAddComment.Enabled := False
          else
            actionDueListAddComment.Enabled := False;
      end;
  else
    actionDueListAddComment.Enabled := False;

  end; // case lstCurrentTab
end; // actionDueListAddCommentUpdate

procedure TfrmMain.fraIV1tvwIVHistoryContextPopup(Sender: TObject;
  MousePos: TPoint; var Handled: Boolean);
var
  TempNode: TTreeNode;
  p: TPoint;
begin
  tempNode := fraIV1.tvwIVHistory.getnodeat(MousePos.x, MousePos.Y);
  with tempNode do
    if tempNode <> nil then begin
      selected := True;
      if (level = 1) then begin
        //Since we couldn't use the PopUpActionBar because it had too many
        //bugs with the TreeVeiw, make sure the standard popup menus on the
        //popupmenu match the Actions state.

        actionDueListAddComment.Update;
        fraIV1.mnuAddComment.Enabled := actionDueListAddComment.Enabled;

        actionMarkHeld.Update;
        fraIV1.mnuHeld.Enabled := actionMarkHeld.Enabled;

        actionMarkRefused.Update;
        fraIV1.mnuRefused.Enabled := actionMarkRefused.Enabled;

        actionDueListMissingDose.Update;
        fraIV1.mnuMissingDose.Enabled := actionDueListMissingDose.Enabled;

        //actionDueListTakeActionOnWS.Update;
        //mnuTakeActionOnWS.Enabled := actionDueListTakeActionOnWS.Enabled;
        //mnuTakeActionOnWS.Visible := actionDueListTakeActionOnWS.Visible;
        actionDueListTakeActionOnBag.Update; {JK 5/6/2008}
        mnuTakeActionOnBag.Enabled := actionDueListTakeActionOnBag.Enabled;
        {JK 5/6/2008}
        mnuTakeActionOnBag.Visible := actionDueListTakeActionOnBag.Visible;
        {JK 5/6/2008}

      //msf disable commented out following code
      {There are two unable to scan menus, one on the app menu which is
      specific to an order on the VDL, and one that is on the right click
      on the IV Bag which may be enabled differently}
{$IFDEF MSF_ON}

        if (ReadOnly or (pos('WS',
          UpperCase(TBCMA_IVBags(tempNode.Data).UniqueID))
          <> 0)) or (TBCMA_IVBags(tempNode.Data).ScanStatus = 'C') then
          frmMain.mnuPopUpUnableToScan.Enabled := False
        else
          frmMain.mnuPopUpUnableToScan.Enabled := True;
{$ENDIF}

        p := fraIV1.ClientToScreen(Point(MousePos.x, MousePos.y));
        fraIV1.tvwIVHistory.PopupMenu.Popup(p.x + 2, p.y + 2);

      end;
    end;
end;

procedure TfrmMain.actionDueListAvailableBagsUpdate(Sender: TObject);
begin
  //  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) then
  //  begin
  //    actionDueListAvailableBags.enabled := False;
  //    exit;
  //  end;
  //  case lstCurrentTab of
  //    ctPB:
  //      begin
  //        actionDueListAvailableBags.Visible := True;
  //        with lstUnitDose do
  //          if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
  //            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
  //            begin
  //              if OrderTypeId = otIV then
  //                actionDueListAvailableBags.Enabled := True
  //              else
  //                actionDueListAvailableBags.Enabled := False;
  //            end
  //          else
  //            actionDueListAvailableBags.enabled := False;
  //      end;
  //  else
  //    actionDueListAvailableBags.Visible := False;
  //  end;

end;

procedure TfrmMain.actionDueListDisplayOrderUpdate(Sender: TObject);
begin
  //  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) or (lstCurrentTab = ctCS) then
  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) or
    (VisibleMedList.Count = 0) or (lstCurrentTab = ctCS) then {// rpk 9/10/2009} begin
    actionDueListDisplayOrder.enabled := False;
    exit;
  end;

  if ScreenReaderSystemActive then begin // rpk 9/11/2009
    actionDueListDisplayOrder.Enabled := sgUnitDose.RowCount > 1;
  end
  else begin
    with lstUnitDose do
      if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
        actionDueListDisplayOrder.Enabled := True
      else
        actionDueListDisplayOrder.enabled := False;
  end;
end;

procedure TfrmMain.actionDueListDrugIENExecute(Sender: TObject);
var
  ii, y: integer;
  msg,
    tempUID: string;
  Found: Boolean;

begin
  Found := false;
  msg := '';
  lstUnitDose.Enabled := False;

  with lstUnitDose do
    case lstCurrentTab of
      ctUD, ctPB:
        with TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
          case OrderTypeID of
            otUnitDose: begin
                if OrderedDrugNames.count > 0 then
                  for ii := 0 to OrderedDrugNames.count - 1 do
                    msg := msg + OrderedDrugNames[ii] + #9 + 'Drug IEN=' +
                      OrderedDrugIENs[ii] + #13
                else
                  msg := 'No Dispensed Drugs Found for this Order!';
                DefMessageDlg(msg, mtInformation, [mbOk], 0);
              end;
            otIV: begin
                if UniqueIDs.count > 0 then begin
                  msg := 'The following bags are available:' + #13 + #13;
                  for ii := 0 to UniqueIDs.count - 1 do begin
                    tempUID := piece(UniqueIDs[ii], '^', 1);
                    //with VisibleMedList do
                    with BCMA_Patient do
                      for y := 0 to MedOrders.count - 1 do
                        with TBCMA_MedOrder(MedOrders[y]) do
                          if UniqueID = tempUID then begin
                            Found := True;
                            break;
                          end;
                    if Found <> True then
                      msg := msg + tempUID + #13;
                    Found := False;
                  end;
                end
                else
                  msg := 'No bags are available for this order!';
                DefMessageDlg(msg, mtInformation, [mbOk], 0);
              end;

          end;

    end;

  lstUnitDose.Enabled := True;

  edtScannerInput.Enabled := True;
  //  edtScannerInput.SetFocus;
  ScannerActivate;
end; // actionDueListDrugIENExecute

procedure TfrmMain.actionDueListDrugIENUpdate(Sender: TObject);
begin
  // Commented out for MSF - JK 12/5/2007
  //  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) then
  //  begin
  //    actionDueListDrugIEN.enabled := False;
  //    exit;
  //  end;
  //
  //  case lstCurrentTab of
  //    ctUD, ctPB:
  //      begin
  //        actionDueListDrugIEN.Visible := True;
  //        with lstUnitDose do
  //          if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
  //            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
  //            begin
  //              if OrderTypeId = otUnitDose then
  //                actionDueListDrugIEN.Enabled := True
  //              else
  //                actionDueListDrugIEN.Enabled := False;
  //            end
  //          else
  //            actionDueListDrugIEN.enabled := False;
  //      end;
  //  else
  //    actionDueListDrugIEN.Visible := False;
  //  end;
end;

procedure TfrmMain.ActionDueListDspSpecInstrExecute(Sender: TObject);
var
  MO_tmp: TBCMA_MedOrder;
  tbool: Boolean;
begin
  if lstUnitDose.ItemIndex > -1 then begin
    MO_tmp := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
    with BCMA_Broker do begin
      with MO_tmp do begin
        tbool := DisplaySOMemo(SpecialInstructions, PatientIEN, OrderNumber,
          OrderType, False); // rpk 10/8/2009
      end;
    end;
  end;
end; // ActionDueListDspSpecInstrExecute

procedure TfrmMain.ActionDueListDspSpecInstrUpdate(Sender: TObject);
begin
  //
  // deactivate code until Special Instructions / Other Print Info project
  // is reactivated.
  //
  {if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) or
    (lstCurrentTab = ctCS) or
    (VisibleMedList.Count = 0) then // rpk 9/11/2009
  begin
    actionDueListDspSpecInstr.enabled := False;
    exit;
  end;

  if ScreenReaderSystemActive then
  begin // rpk 9/11/2009
    actionDueListDspSpecInstr.Enabled := sgUnitDose.RowCount > 1;
  end
  else
  begin
    with lstUnitDose do
      if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
        actionDueListDspSpecInstr.Enabled := True
      else
        actionDueListDspSpecInstr.enabled := False;
  end;}

  actionDueListDspSpecInstr.enabled := False;

end; // ActionDueListDspSpecInstrUpdate

procedure TfrmMain.actionMarkHeldUpdate(Sender: TObject);
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;

  procedure UpdMarkHeld(MedOrder: TBCMA_MedOrder);
  begin
    if MedOrder <> nil then begin
      with MedOrder do begin
        case OrderTypeID of
          otUnitDose, otIV:
            if OrderStatus = 'H' then
              if (ScanStatus = '') and (ScheduleTypeID <> stOnCall) and
                (ScheduleTypeID <> stPRN) then
                actionMarkHeld.Enabled := True
              else
                actionMarkHeld.Enabled := False
            else
              case ScheduleTypeID of
                stContinuous, stOneTime:
                  if (ScanStatus = '') or (ScanStatus = 'M') then
                    actionMarkHeld.Enabled := True
                  else if ScanStatus = 'G' then
                    actionMarkHeld.Enabled := False
                  else if ScanStatus = 'H' then
                    actionMarkHeld.Enabled := False
                  else if (ScanStatus = 'R') or (ScanStatus = 'RM') then
                    actionMarkHeld.Enabled := False
                  else
                    actionMarkHeld.Enabled := False;
                stPRN, stOnCall:
                  actionMarkHeld.Enabled := False;
              else
                actionMarkHeld.Enabled := False;
              end; //end of case ScheduleTypeID
        else
          actionMarkHeld.Enabled := False;
        end; //end of case OrderTypeID
      end;
    end; // if MedOrder <> nil
  end;

begin
  if isRestricted or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count =
    0) or
    (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionMarkHeld.enabled := False;
    exit;
  end;

  aMedOrder := nil;

  case lstCurrentTab of
    ctUD, ctPB:
      { begin
        with lstUnitDose do
          if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
            (VisibleMedList.count = 0) then
            actionMarkHeld.Enabled := False
          else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
              case OrderTypeID of
                otUnitDose, otIV:
                  if OrderStatus = 'H' then
                    if (ScanStatus = '') and (ScheduleTypeID <> stOnCall) and
                      (ScheduleTypeID <> stPRN) then
                      actionMarkHeld.Enabled := True
                    else
                      actionMarkHeld.Enabled := False
                  else
                    case ScheduleTypeID of
                      stContinuous, stOneTime:
                        if (ScanStatus = '') or (ScanStatus = 'M') then
                          actionMarkHeld.Enabled := True
                        else if ScanStatus = 'G' then
                          actionMarkHeld.Enabled := False
                        else if ScanStatus = 'H' then
                          actionMarkHeld.Enabled := False
                        else if (ScanStatus = 'R') or (ScanStatus = 'RM') then
                          actionMarkHeld.Enabled := False
                        else actionMarkHeld.Enabled := False;
                      stPRN, stOnCall:
                        actionMarkHeld.Enabled := False;
                    else actionMarkHeld.Enabled := False;
                    end; //end of case ScheduleTypeID
                else
                  actionMarkHeld.Enabled := False;
              end //end of case OrderTypeID

                // Multiple administrations selected
          else actionMarkHeld.Enabled := True;
      end; }begin
        if ScreenReaderSystemActive then begin
          if sgUnitDose.Row > 1 then
            UpdMarkHeld(TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]));
        end
        else begin
          with lstUnitDose do begin
            if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
              (VisibleMedList.count = 0) then
              actionMarkHeld.Enabled := False
            else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex >
              -1)) then begin
              UpdMarkHeld(TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]));
            end
            else
              actionMarkHeld.Enabled := True;
          end; // with lstUnitDose
        end; // else
      end; // case ctUD, ctPB

    ctIV: begin
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if (tempNode <> nil) and (level = 1) then begin
            aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
            ScanStat := getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
            with aMedOrder do
              if aMedOrder = nil then begin
                DefMessageDlg(ErrIVAction, mtError, [mbOk], 0);
                actionMarkHeld.Enabled := False;
                exit;
              end
              else if (OrderStatus = 'A') or (OrderStatus = 'R') or (OrderStatus
                = 'O') then
                case ScanStat of
                  ssStopped,
                    ssAvailable,
                    ssMissed,
                    ssNotGiven:
                    actionMarkHeld.Enabled := True;
                  ssHeld,
                    ssRefused,
                    ssComplete,
                    ssInfusing:
                    actionMarkHeld.Enabled := False;
                else
                  actionMarkHeld.Enabled := False;
                end
              else if (OrderStatus = 'H') or (OrderStatus = 'D') or (OrderStatus
                = 'E') then
                case ScanStat of
                  ssComplete,
                    ssInfusing,
                    ssStopped,
                    ssHeld:
                    actionMarkHeld.Enabled := False;
                  ssAvailable,
                    ssMissed,
                    ssRefused:
                    actionMarkHeld.Enabled := True
                else
                  actionMarkHeld.Enabled := False;
                end
              else
                actionMarkHeld.Enabled := False;
          end // end (tempNode <> nil) and (level = 1)
          else
            actionMarkHeld.Enabled := false;
      end; //end ctIV
  else
    actionMarkHeld.Enabled := False
  end; //end Case lstCurrentTab
end;

procedure TfrmMain.actionMarkUndoUpdate(Sender: TObject);
  procedure UpdMarkUndo(aMedOrder: TBCMA_MedOrder);
  begin
    with aMedOrder do
      case OrderTypeID of
        otUnitDose, otIV:
          if OrderStatus = 'H' then
            actionMarkUndo.Enabled := False
          else
            case ScheduleTypeID of
              stContinuous, stOneTime, stPRN, stOnCall:
                if (ScanStatus = 'G') or (ScanStatus = 'RM') then begin
                  if (((OrderStatus = 'A') or (OrderStatus = 'R') or
                    (OrderStatus = 'RE')) and CanMarkNG) then begin
                    actionMarkUndo.Enabled := True;
                    actionMarkUndo.Caption := '&Undo - ' +
                      GetLastActivityStatus(ScanStatus);
                  end
                  else
                    actionMarkUndo.Enabled := False;
                end
                else
                  actionMarkUndo.Enabled := False;
            else
              actionMarkUndo.Enabled := False;
            end; //end of case ScheduleTypeID
      else
        actionMarkUndo.Enabled := False;
      end //end of case OrderTypeID
  end; // UpdMarkUndo

begin // actionMarkUndoUpdate
  actionMarkUndo.Caption := '&Undo';
  if not LimitedAccess then {JK 6/16/2008}
    if ReadOnly or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0)
      or
      (VisibleMedList.Count = 0) then begin
      actionMarkUndo.enabled := False;
      exit;
    end;

  case lstCurrentTab of
    ctUD, ctPB: begin
        if ScreenReaderSystemActive then begin

          with sgUnitDose do
            if (RowCount = 1) then
              actionMarkUndo.Enabled := False
            else if (RowCount > 1) and (Row > 0) then
              UpdMarkUndo(TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]));
        end
        else begin
          with lstUnitDose do
            if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1))
              then
              actionMarkUndo.Enabled := False
            else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex >
              -1)) then
              UpdMarkUndo(TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]))
                { with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
case OrderTypeID of
otUnitDose, otIV:
if OrderStatus = 'H' then
actionMarkUndo.Enabled := False
else
case ScheduleTypeID of
stContinuous, stOneTime, stPRN, stOnCall:
  if (ScanStatus = 'G') or (ScanStatus = 'RM') then
  begin
    if (((OrderStatus = 'A') or (OrderStatus = 'R') or
      (OrderStatus = 'RE')) and CanMarkNG) then
    begin
      actionMarkUndo.Enabled := True;
      actionMarkUndo.Caption := '&Undo - ' + GetLastActivityStatus(ScanStatus);
    end
    else
      actionMarkUndo.Enabled := False;
  end
  else actionMarkUndo.Enabled := False;
else actionMarkUndo.Enabled := False;
end; //end of case ScheduleTypeID
else actionMarkUndo.Enabled := False;
                  end }//end of case OrderTypeID

            else // More then one order highlighted
              actionMarkUndo.Enabled := False;
        end // not screen reader
      end;
  else
    actionMarkUndo.Enabled := False;
  end; //end case lstCurrentTab
end; // actionMarkUndoUpdate

procedure TfrmMain.ActionReportsMedicationOverviewExecute(Sender: TObject);
begin
  MedOverviewReport;
end;

procedure TfrmMain.ActionReportsMedicationTherapyExecute(Sender: TObject);
begin
  MedTherapyReport;
end;

procedure TfrmMain.actionMarkRefusedUpdate(Sender: TObject);
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;

  procedure UpdMarkRefused(bMedOrder: TBCMA_MedOrder);
  begin
    with bMedOrder do
      case OrderTypeID of
        otUnitDose, otIV:
          if OrderStatus = 'H' then
            actionMarkRefused.Enabled := False
          else
            case ScheduleTypeID of
              stContinuous, stOneTime:
                if (ScanStatus = '') or (ScanStatus = 'M') or (ScanStatus = 'H')
                  then
                  actionMarkRefused.Enabled := True
                else
                  actionMarkRefused.Enabled := False;
              stPRN:
                actionMarkRefused.Enabled := False;
              stOnCall:
                if (ScanStatus = '') or (ScanStatus = 'M') then
                  actionMarkRefused.Enabled := True
                else
                  actionMarkRefused.Enabled := False;
            else
              actionMarkRefused.Enabled := False
            end; //end of case ScheduleTypeID
      else
        actionMarkRefused.Enabled := False;
      end; //end of Case OrderTypeID
  end;

begin
  if isRestricted or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count =
    0) or
    (VisibleMedList.Count = 0) then begin
    actionMarkRefused.enabled := False;
    exit;
  end;

  case lstCurrentTab of
    ctUD, ctPB: begin
        if ScreenReaderSystemActive then begin
          if (sgUnitDose.RowCount > 1) and (sgUnitDose.Row > 0) then
            UpdMarkRefused(TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]));
        end
        else begin
          with lstUnitDose do begin
            if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
              (VisibleMedList.count = 0) then
              actionMarkRefused.Enabled := False
            else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex >
              -1)) then
              {with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
                case OrderTypeID of
                  otUnitDose, otIV:
                    if OrderStatus = 'H' then
                      actionMarkRefused.Enabled := False
                    else
                      case ScheduleTypeID of
                        stContinuous, stOneTime:
                          if (ScanStatus = '') or (ScanStatus = 'M') or (ScanStatus = 'H') then
                            actionMarkRefused.Enabled := True
                          else
                            actionMarkRefused.Enabled := False;
                        stPRN:
                          actionMarkRefused.Enabled := False;
                        stOnCall:
                          if (ScanStatus = '') or (ScanStatus = 'M') then
                            actionMarkRefused.Enabled := True
                          else
                            actionMarkRefused.Enabled := False;
                      else actionMarkRefused.Enabled := False
                      end; //end of case ScheduleTypeID
                else actionMarkRefused.Enabled := False;
                end //end of Case OrderTypeID}
              UpdMarkRefused(TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]))
            else // more then one administration selected
              actionMarkRefused.Enabled := True;
          end; // with lstunitdose
        end; // else not screen reader
      end; // case

    ctIV: begin
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if (tempNode <> nil) and (level = 1) then begin
            aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
            ScanStat := getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
            with aMedOrder do
              if aMedOrder = nil then begin
                DefMessageDlg(ErrIVAction, mtError, [mbOk], 0);
                actionMarkRefused.Enabled := False;
                exit;
              end
              else if (OrderStatus = 'A') or (OrderStatus = 'R') or (OrderStatus
                = 'O') then
                case ScanStat of
                  ssStopped,
                    ssAvailable,
                    ssMissed,
                    ssNotGiven,
                    ssHeld:
                    actionMarkRefused.Enabled := True;
                else
                  actionMarkRefused.Enabled := False;
                end
              else
                actionMarkRefused.Enabled := False;
          end
          else
            actionMarkRefused.Enabled := False;
      end;
  else
    actionMarkRefused.Enabled := False;
  end;
end; // actionMarkRefusedUpdate

procedure TfrmMain.actionMarkRemovedUpdate(Sender: TObject);
begin
  if ReadOnly or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0)
    or
    (VisibleMedList.Count = 0) then begin
    actionMarkRemoved.enabled := False;
    exit;
  end;

  case lstCurrentTab of
    ctUD, ctPB:
      if ScreenReaderSystemActive then begin
        with sgUnitDose do
          if RowCount = 1 then
            actionMarkRemoved.Enabled := False
          else if (RowCount > 1) and (Row > 0) then
            with TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]) do
              actionMarkRemoved.Enabled := (AdministrationUnit = 'PATCH') and
                (ScanStatus = 'G');
      end
      else begin
        with lstUnitDose do
          if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
            (VisibleMedList.count = 0) then
            actionMarkRemoved.Enabled := False
          else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1))
            then
            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
              actionMarkRemoved.Enabled := (AdministrationUnit = 'PATCH') and
                (ScanStatus = 'G')
          else
            actionMarkRemoved.Enabled := false;
      end // not screen reader
    else // case
      actionMarkRemoved.Enabled := False;
  end;
end;

procedure TfrmMain.actionDueListMedHistoryUpdate(Sender: TObject);
begin
  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) or
    (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionDueListMedHistory.enabled := False;
    exit;
  end;

  if ScreenReaderSystemActive then begin // rpk 9/11/2009
    actionDueListMedHistory.Enabled := sgUnitDose.RowCount > 1;
  end
  else begin
    with lstUnitDose do
      if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
        actionDueListMedHistory.enabled := True
      else
        actionDueListMedHistory.Enabled := False;
  end;
end;

procedure TfrmMain.actionDueListMissingDoseUpdate(Sender: TObject);
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;
begin
  if isRestricted or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count =
    0) or (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionDueListMissingDose.enabled := False;
    exit;
  end;

  case lstCurrentTab of
    ctUD, ctPB:
      if ScreenReaderSystemActive then begin // rpk 9/11/2009
        if (sgUnitDose.RowCount > 1) and (sgUnitDose.Row > 0) then
          with TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]) do begin
            if (OrderStatus = 'H') or (ScanStatus = 'U') then
              actionDueListMissingDose.Enabled := False
            else
              actionDueListMissingDose.Enabled := ((ScanStatus <> 'G') and
                (ScanStatus <> 'M'));
          end
      end
      else begin
        with lstUnitDose do
          if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
            (VisibleMedList.count = 0) then
            actionDueListMissingDose.Enabled := False
          else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1))
            then
            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
              if (OrderStatus = 'H') or (ScanStatus = 'U') then
                actionDueListMissingDose.Enabled := False
              else
                actionDueListMissingDose.Enabled := ((ScanStatus <> 'G') and
                  (ScanStatus <> 'M'));
            end
          else
            actionDueListMissingDose.Enabled := False;
      end;
    ctIV: begin
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if (tempNode <> nil) and (level = 1) then begin
            aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
            ScanStat := getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
            with aMedOrder do
              if aMedOrder = nil then begin
                DefMessageDlg(ErrIVAction, mtError, [mbOk], 0);
                actionDueListMissingDose.Enabled := False;
                exit;
              end
              else if (OrderStatus = 'A') or (OrderStatus = 'R') or (OrderStatus
                = 'O') then
                if ((ScanStat <> ssComplete) and (ScanStat <> ssInfusing)
                  and (ScanStat <> ssMissed)) and
                  (pos('WS', UpperCase(TBCMA_IVBags(tempNode.Data).UniqueID)) =
                  0) then
                  actionDueListMissingDose.Enabled := True
                else
                  actionDueListMissingDose.Enabled := False
              else
                actionDueListMissingDose.Enabled := False;
          end
          else
            actionDueListMissingDose.Enabled := False;
      end;
  else
    actionDueListMissingDose.Enabled := False;
  end;
end;

procedure TfrmMain.actionDueListPRNEffectUpdate(Sender: TObject);
begin
  if not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) or
    (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionDueListPRNEffect.enabled := False;
    exit;
  end;

  case lstCurrentTab of
    ctUD, ctPB:
      if ScreenReaderSystemActive then begin // rpk 9/11/2009
        if (sgUnitDose.RowCount > 1) and (sgUnitDose.Row > 0) then
          with TBCMA_MedOrder(VisibleMedList[sgUnitDose.Row - 1]) do begin
            actionDueListPRNEffect.Enabled := uppercase(ScheduleType) = 'P';
          end
      end
      else begin
        with lstUnitDose do
          if (SelCount = 0) or ((MultiSelect = False) and (ItemIndex = -1)) or
            (VisibleMedList.count = 0) then
            actionDueListPRNEffect.Enabled := False
          else if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1))
            then
            with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
              if uppercase(ScheduleType) = 'P' then
                actionDueListPRNEffect.Enabled := True
              else
                actionDueListPRNEffect.Enabled := False;
            end
          else
            actionDueListPRNEffect.Enabled := false;
      end
    else
      actionDueListPRNEffect.Enabled := False;
  end;
end;

procedure TfrmMain.actionDueListUnableToScanUpdate(Sender: TObject);
begin
  //msf disable
{$IFNDEF MSF_ON}
  exit;
{$ENDIF}

  actionDueListUnableToScan.enabled := False;
  if ReadOnly or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0)
    or
    (VisibleMedList.Count = 0) then // rpk 9/11/2009
    exit;

  case lstCurrentTab of
    ctUD, ctPB: begin
        if ScreenReaderSystemActive then begin // rpk 9/11/2009
          if (sgUnitDose.RowCount > 1) and (sgUnitDose.Row > 0) then begin
            with TBCMA_MedOrder(VisibleMedList[sgUnitDose.row - 1]) do begin
              if ScanStatus <> 'G' then
                actionDueListUnableToScan.Enabled := True;
              if ScanStatus = 'RM' then {JK 11/15/2008 CodeCR 261}
                actionDueListUnableToScan.Enabled := False;
            end
          end;
        end
        else begin
          with lstUnitDose do begin
            if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1))
              then begin
              with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
                if ScanStatus <> 'G' then
                  actionDueListUnableToScan.Enabled := True;
                if ScanStatus = 'RM' then {JK 11/15/2008 CodeCR 261}
                  actionDueListUnableToScan.Enabled := False;
              end
            end;
          end;
        end; // else not screenreader
      end; // ctUD, ctPB
    ctCS:
      actionDueListUnableToScan.enabled := False
  else
    with lstUnitDose do
      if (SelCount = 1) or ((MultiSelect = False) and (ItemIndex > -1)) then
        actionDueListUnableToScan.Enabled := True;
  end; // case
end; // actionDueListUnableToScanUpdate

procedure TfrmMain.actionReportsExpiredOrdersExecute(Sender: TObject);
begin
  ExpiredOrdersReport;
end;

{JK 5/7/2008 - removed the following method}
//procedure TfrmMain.actionDueListTakeActionOnWSUpdate(Sender: TObject);
//var
//  TempNode: TTreeNode;
//  aMedOrder: TBCMA_MedOrder;
//  ScanStat: TScanStatus;
//begin
(*  if ReadOnly or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0) then
  begin
    actionDueListTakeActionOnWS.enabled := False;
    exit;
  end;
  case lstCurrentTab of
    ctUD, ctPB:
      actionDueListTakeActionOnWS.Visible := False;
    ctIV:
      begin
        actionDueListTakeActionOnWS.Visible := True;
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if (tempNode <> nil) and (level = 1) then
          begin
            aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
            ScanStat := getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
            with aMedOrder do
              if aMedOrder = nil then
              begin
                DefMessageDlg(ErrIVAction, mtError, [mbOk], 0);
                actionDueListTakeActionOnWS.Enabled := False;
                exit;
              end
              else if (pos('WS', UpperCase(TBCMA_IVBags(tempNode.Data).UniqueID)) <> 0)
                and ((ScanStat = ssInfusing) or (ScanStat = ssStopped) or
                (ScanStat = ssHeld) or (ScanStat = ssRefused)) then
                actionDueListTakeActionOnWS.Enabled := true
              else
                actionDueListTakeActionOnWS.Enabled := false;
          end
          else actionDueListTakeActionOnWS.Enabled := false;
      end;
  else actionDueListTakeActionOnWS.enabled := false;
  end;
  *)
//end;

procedure TfrmMain.actionDueListRefreshUpdate(Sender: TObject);
begin
  actionDueListRefresh.enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionMOBUpdate(Sender: TObject);
begin
  if ReadOnly then begin
    actionMOB.Enabled := False;
    exit;
  end;

  if BCMA_SiteParameters.MedOrderButton = true then
    if (BCMA_User.IsMOButtonUser) and (pnlMainForm.Visible) then
      actionMOB.enabled := true
    else
      actionMOB.enabled := false
  else
    actionMOB.enabled := false;
end;

procedure TfrmMain.ActionReportsPRNOverviewExecute(Sender: TObject);
begin
  PRNOverviewReport;
end;

procedure TfrmMain.DisplayTransferMessage;
  procedure DisplayTransMessage;
  begin
    with BCMA_Patient do
      DefMessageDlg('Review the Last Action column on the VDL' + #13 +
        'for each medication due to' + #13 + #13 + 'Patient Movement type:' + #13
        +
        '[ ' + TransferredTransactionType + ' ]   [ ' + TransferredMovementType
        +
        ' ]',
        mtWarning, [mbOk], 0, 'NOTICE');
  end;
begin
  with BCMA_Patient do
    case lstCurrentTab of
      ctUD:
        if (not TransferUDDisplayed) and (Transferred) then begin
          DisplayTransMessage;
          TransferUDDisplayed := True;
        end;
      ctPB:
        if (not TransferPBDisplayed) and (Transferred) then begin
          DisplayTransMessage;
          TransferPBDisplayed := True
        end;
    end;
end;

procedure TfrmMain.lvwRemindersInfoTip(Sender: TObject; Item: TListItem;
  var InfoTip: string);
var
  x: integer;
begin
  Screen.HintFont.Name := 'Courier';
  Screen.HintFont.Size := 8;

  application.HintHidePause := 5000;
  InfoTip := '';
  if item.Index = 0 then
    with BCMA_Patient do
      if PRNEffectList.Count > 0 then begin
        InfoTip := padr('ADMIN DATE', 16) + ' - ' + padr('ORDERABLE ITEM', 20) +
          ' - ' +
          padr('PRN REASON', 15) + ' - ' + padr('ADMINISTERED BY', 15);
        InfoTip := InfoTip + #13 + StringOfChar('-', 77);
        for x := 0 to PRNEffectList.Count - 1 do begin
          if x > 3 then
            break;
          InfoTip := InfoTip + #13 +
            padr(DisplayVADateYearTime(TBCMA_PRNEffectList(PRNEffectList[x]).AdminDateTime), 16) + ' - ' +
            padr(TBCMA_PRNEffectList(PRNEffectList[x]).AdministeredDrug, 20) +
            ' - ' +
            padr(TBCMA_PRNEffectList(PRNEffectList[x]).PRNReason, 15) + ' - ' +
            padr(TBCMA_PRNEffectList(PRNEffectList[x]).AdministeredBy, 15);
        end;
      end;
end;

procedure TfrmMain.lvwRemindersDblClick(Sender: TObject);
//var
//  NewItems: TListItems;
//  NewItem: TListItem;
begin
  if lvwReminders.Selected <> nil then
    if lvwReminders.Selected.index = 0 then begin
      frmPRNEffectiveness.Execute(nil);
      pnlMainForm.Repaint;
      BCMA_Patient.LoadPRNEffectiveness('');

      //      try
      //        if lvwReminders.Items[0] = nil then  //JK 2-7-2008
      //          lvwReminders.AddItem(IntToStr(BCMA_Patient.PRNEffectList.Count), nil);
      //       //- else begin
      //          lvwReminders.Items[0].Caption := IntToStr(BCMA_Patient.PRNEffectList.Count);
      //          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
      //       //- end;
      //      except
      //        on E:Exception do
      //          Showmessage('Clinical Reminders error 3: msg = ' + E.Message);
      //      end;

      try
        if lvwReminders.Items[0] = nil then begin //JK 11/29/2007
          lvwReminders.AddItem(IntToStr(BCMA_Patient.PRNEffectList.Count), nil);
          //          NewItems := TListItems.Create(lvwReminders);
          //          NewItem := TListItem.Create(NewItems);
          //          lvwReminders.Items.Add;
          //          lvwReminders.Items[0].Caption := IntToStr(BCMA_Patient.PRNEffectList.Count);
          //          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
          lvwReminders.Items[0].SubItems.Add('PRN Effectiveness');
          // rpk 7/22/2009
        end
        else begin
          lvwReminders.Items[0].Caption :=
            IntToStr(BCMA_Patient.PRNEffectList.Count); //JK 11/29/2007
          lvwReminders.Items[0].SubItems[0] := 'PRN Effectiveness';
        end;
      except
        on E: Exception do
          ;
      end;
    end;

  ScannerActivate;
end; // lvwRemindersDblClick

procedure TfrmMain.lvwRemindersKeyPress(Sender: TObject; var Key: Char);
begin
  if key = chr(VK_RETURN) then
    lvwRemindersDblClick(lvwReminders);
end;

procedure TfrmMain.actionReportsVarianceExecute(Sender: TObject);
begin
  MedicationVarianceLog(BCMA_Patient.IEN);
end;

procedure TfrmMain.FormDeactivate(Sender: TObject);
begin
  AppDeactivate(Sender);
end;

procedure TfrmMain.actionReportVitalsCumulativeExecute(Sender: TObject);
begin
  VitalsCumulativeReport(BCMA_Patient.IEN);
end;

procedure TfrmMain.actionReportVitalsCumulativeUpdate(Sender: TObject);
begin
  actionReportVitalsCumulative.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.ActionSortByBagInformationUpdate(Sender: TObject);
begin
  with actionSortByBagInformation do
    case lstCurrentTab of
      ctIV:
        Visible := True;
    else
      Visible := False;
    end;

end;

procedure TfrmMain.AbleToChangeContext(var OkayToChange:
  TVACCOW_AbleToChangeStatus; var MsgText: string;
  const NewID, NewICN, NewName, Site: string; const NewProd: Boolean);
//CCOW is about to change patient and is asking if it is alright to change.
var
  //  i: integer;
  RequirePBView: Boolean;
begin
  //If the request comes in for the patient we currently have open, do nothing and just agree
  //to switch patients
  if (
    (BCMA_Patient.IEN = NewID) and (BCMA_Patient.ICN = NewICN) and
    (BCMA_User.SiteIEN = Site) and
    (piece(BCMA_Patient.Name, ',', 1) = piece(NewName, '^', 1)) and
    (piece(BCMA_Patient.Name, ',', 2) = piece(NewName, '^', 2)) and
    (BCMA_User.ProductionAccount = NewProd)
    ) then begin
    OkayToChange := acYes;
    exit;
  end;

  //Close what forms we can, if any remain Modal, hang, which causes a not responding
  //message in CCOW
  if CloseForms(False) then begin
    Sleep(12000);
    exit;
  end;
  Application.ProcessMessages;
  //Check to see if the user viewed the PB tab, if not, display a message in CCOW
  with BCMA_Patient do
    if (ActivePBOrders = True) and (PBViewed = False) then
      RequirePBView := True
    else
      RequirePBView := false;
  if RequirePBView then begin
    MsgText :=
      'Are you sure you want to close BCMA without viewing these IVP/IVPB tab?';
    OkayToChange := acNo;
    Closing := True;
  end;
end;

procedure TfrmMain.CCOWError(const CCOWErr: Exception);
var
  ErrorText: string;
begin
  ErrorText := CCOWErrorMessage;
  ErrorText := ErrorText + #13#13 + 'Error Information:' + #13 +
    CCOWErr.Message;
  DefMessageDlg(ErrorText, mtError, [mbOK], 0);
end;

procedure TfrmMain.UpdateCCOWLinkStatus(const Status: TVACCOW_LinkStatus);
var
  bmp: tbitmap;
begin
  bmp := tbitmap.create;
  case Status of
    lsJoined: begin
        imgCCOWStatus.Hint := 'Patient Context is Joined';
        ImageList2.getbitmap(0, bmp);
      end;
    lsBroken: begin
        imgCCOWStatus.Hint := 'Patient Context is Broken';
        ImageList2.getbitmap(1, bmp);
      end;
    lsChanging: begin
        imgCCOWStatus.Hint := 'Patient Context is Changing';
        ImageList2.getbitmap(2, bmp);
      end;
  end;
  imgCCOWStatus.picture.bitmap.assign(bmp);
  SetFormCaption;
  bmp.Free;
end;

procedure TfrmMain.VA508CompAccessSgIVBagDetailItemQuery(Sender: TObject;
  var Item: TObject);
//var
//   i, j: Integer;
begin
  //  j := Integer(Item);
  //  i := 100 * fraIV1.SelectedIVY + fraIV1.SelectedIVX;
  //  Item := TObject(i);
end;

procedure TfrmMain.VA508CompAccessSgIVBagDetailValueQuery(Sender: TObject;
  var Text: string);
begin
  Text := 'Column ' + fraIV1.sgIVBagDetail.Cells[fraIV1.SelectedIVX, 0] + ', ' +
    fraIV1.sgIVBagDetail.Cells[fraIV1.SelectedIVX, fraIV1.SelectedIVY];
end;

procedure TfrmMain.VA508CompAccessSgUnitDoseItemQuery(Sender: TObject;
  var Item: TObject);
var
  i, j: Integer;
begin
  j := Integer(Item);
  i := 100 * SelectedY + SelectedX;
  Item := TObject(i);
end;

procedure TfrmMain.VA508CompAccessSgUnitDoseValueQuery(Sender: TObject;
  var Text: string);
var
  Index: Integer;
begin
  //  Text := ' Med ' + sgUnitDose.Cells[0, SelectedY]+ ' ' +
  //    'Column ' + hdrUnitDose.Sections[SelectedX+1].Text + ', ' +
  //    sgUnitDose.Cells[SelectedX, SelectedY];

  // Announce Provider Hold in first column.
  Index := SelectedY - 1;
  if (Index >= 0) and (SelectedX = 1) and
    (TBCMA_MedOrder(VisibleMedList[Index]).OrderStatus = 'H') then
    Text := ' Order is on Provider Hold, '
  else
    Text := '';

  Text := Text + ' Med ' + sgUnitDose.Cells[0, SelectedY] + ' ' +
//  Text := ' Med ' + sgUnitDose.Cells[0, SelectedY] + ' ' +
  'Column ' + sgUnitDose.Cells[SelectedX, 0] + ', ' +
    sgUnitDose.Cells[SelectedX, SelectedY];
end; // VA508ComponentAccessibility1ValueQuery

procedure TfrmMain.SetCurrentPtIEN(const aPtId, aPtICN, aPtName, aPtSite:
  string; const aProd: Boolean);
//var
//  i: integer;
begin
  //if we are switching to the patient we currently have open, do nothing, exit
  if (
    (BCMA_Patient.IEN = aPtId) and (BCMA_Patient.ICN = aPtICN) and
    (BCMA_User.SiteIEN = aPtSite) and (piece(BCMA_Patient.Name, ',', 1) =
    piece(aPtName, '^', 1)) and
    (piece(BCMA_Patient.Name, ',', 2) = piece(aPtName, '^', 2)) and
    (BCMA_User.ProductionAccount = aProd)
    ) then
    exit;

  if CloseForms(True) then begin
    DefMessageDlg('A CCOW request has been received to change patients, however, '
      +
      'there are open dialogues in BCMA that can not be closed.  The current patient link ' +
      'will be broken automatically', mtError, [mbOk], 0);

    if not VACCOW.SuspendContext then
      DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);
    exit;
  end;

  Application.ProcessMessages;
  actionFileClosePatient.execute;

  if BCMA_User.ProductionAccount <> aProd then
    exit;

  if aPtICN <> '' then
    OpenPatient(aPtICN, 'IC')
  else if aPtId <> '' then
    OpenPatient(aPtId, 'DF')
  else
    exit;
end;

procedure TfrmMain.ActionFileBreakPatientContextExecute(Sender: TObject);
begin
  if not VACCOW.SuspendContext then
    DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);

end;

procedure TfrmMain.ActionFileBreakPatientContextUpdate(Sender: TObject);
begin
  if VACCOW.LinkStatus <> lsNoCCOW then begin
    actionFileBreakPatientContext.Enabled := (VACCOW.LinkStatus <> lsBroken);
    actionFileBreakPatientContext.Visible := True;
  end
  else
    actionFileBreakPatientContext.Visible := False;
end;

procedure TfrmMain.ActionJoinSetNewContextExecute(Sender: TObject);
var
  CCOWPatientName: string;
begin
  if VACCOW.ResumeContext(rmSet) then begin
    CCOWPatientName := piece(BCMA_Patient.Name, ',', 1) + '^' +
      piece(BCMA_Patient.Name, ',', 2) + '^^^^';

    if not VACCOW.SetNewPatientContext(BCMA_Patient.IEN, BCMA_Patient.ICN,
      CCOWPatientName, BCMA_User.SiteIEN, BCMA_User.ProductionAccount) then begin
      DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);
      if not VACCOW.SuspendContext then
        DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);
    end;
  end
  else
    DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);

end;

procedure TFrmMain.GetSite(var Site: string);
begin
  Site := BCMA_User.SiteIEN;
end;

procedure TfrmMain.cmbxStartTimeEnter(Sender: TObject);
begin
  StartTime := cmbxStartTime.Text;
end;

procedure TfrmMain.cmbxStartTimeExit(Sender: TObject);
begin
  if StartTime <> cmbxStartTime.Text then
    cmbxStopTimeChange(Sender);
  StartTime := '';
end;

procedure TfrmMain.cmbxStartTimeKeyPress(Sender: TObject; var Key: Char);
begin
  //Prevent a backspace.  Apparently under W2K, a backspace will set
  //itemindex to -1.
  if Key = #8 then
    Key := #0;
end;

procedure TfrmMain.cmbxStopTimeKeyPress(Sender: TObject; var Key: Char);
begin
  //Prevent a backspace.  Apparently under W2K, a backspace will set
  //itemindex to -1.
  if Key = #8 then
    Key := #0;
end;

procedure TfrmMain.ActionFileEditMedLogExecute(Sender: TObject);
var
  ModResult: integer;
begin
  if BCMA_Patient.IEN = '' then begin
    //select a patient

    with TfrmPtSelect.create(application) do try
      EditMedLog := True; {JK 7/13/2008 CQ #125}
      ModResult := ShowModal;
    finally
      Free;
      EditMedLog := False; {JK 7/13/2008 CQ #125}
    end;

    if ((ModResult = mrOk) and (BCMA_Patient.IEN <> '')) then begin
      EditMedLogSelectAdministration;
      if not pnlMainForm.Visible then
        BCMA_Patient.Clear;
    end;
  end
  else if BCMA_Patient.IEN <> '' then begin
    EditMedLogSelectAdministration;
  end;

  if (EditedAdministration = True) and pnlMainForm.Visible and (lstCurrentTab <>
    ctCS) then
    RebuildVirtualDueList(True);
  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      RebuildMe;

  EditedAdministration := False;
end;

procedure TfrmMain.WhatsThis1Popup(Sender, HelpItem: TObject;
  HContext: THelpContext; X, Y: Integer; var DoPopup: Boolean);
begin
  //since the control the user clicked on isn't easily available,
  //we'll use the HContext to determine if this is a control where we want
  //to disable the right click popup 'what's this?'
  case HContext of
    30, // the vdl, already has right click options
      300, //Edit Med Log, grdMedications
      902, //coversheet, already has right click options.
      150: //Multiple Drugs CheckListBox
      DoPopup := false;
  end;
end;

procedure TfrmMain.SetFormCaption;
var
  ReadOnlyText, CCOWText, s508Text: string;
begin
  if ReadOnly and LimitedAccess then begin
    ReadOnlyText := ' - LIMITED ACCESS';
    lblReadOnly.Caption := 'Limited-Access BCMA'; // rpk 5/20/2009
    //    lblReadOnly.Visible := True;
    lblReadOnly.Show;
    lblReadOnly.Invalidate; // rpk 9/18/2009
  end
  else if ReadOnly then begin
    ReadOnlyText := ' - READ-ONLY';
    lblReadOnly.Caption := 'Read-Only BCMA'; // rpk 5/20/2009
    //    lblReadOnly.Visible := True;
    lblReadOnly.Show;
    lblReadOnly.Invalidate; // rpk 9/18/2009
  end
  else begin
    //    lblReadOnly.Visible := False;
    lblReadOnly.Hide;
  end;

  if (VACCOW <> nil) and (VACCOW.LinkStatus <> lsNoCCOW) then
    CCOWText := ' -  ' + imgCCOWStatus.Hint;

  if ScreenReaderSystemActive then // rpk 8/28/2009
    s508Text := ' - s508'
  else
    s508Text := '';

  //  Caption := Application.Title + ' - v' + AppFileVersion + ReadOnlyText + CCOWText; // + '  [/K=' + IntToStr(KeyBoardTimerInterval) + ']';
  Caption := Application.Title + ' - v' + AppFileVersion + ReadOnlyText +
    CCOWText +
    s508Text; // rpk 8/28/2009
  Update; // rpk 9/18/2009
end;

procedure TfrmMain.ActionFileEditMedLogUpdate(Sender: TObject);
begin
  {JK 5/27/2008 - removed the ReadOnly check per SRS 2.0 3.2.7.1, item 2}
  //  if ReadOnly then
  //    ActionFileEditMedLog.Enabled := False
  //  else
  ActionFileEditMedLog.Enabled := True;
end;

procedure TfrmMain.ActionFileOpenReadOnlyExecute(Sender: TObject);
begin
  OpenReadOnly := True;
  actionFileOpenPatient.Execute;
  SetFormCaption;
end;

procedure TfrmMain.actionFlagExecute(Sender: TObject);
begin
  DisplayPatientFlags;
end;

procedure TfrmMain.actionFileClosePatientUpdate(Sender: TObject);
begin
  if BCMA_Patient.IEN = '' then
    actionFileClosePatient.Enabled := False
  else
    actionFileClosePatient.Enabled := True;
end;

procedure TfrmMain.actionFlagUpdate(Sender: TObject);
begin
  if BCMA_Patient.PatientRecordFlags.Count > 0 then begin
    actionFlag.Enabled := True;
    ActionToolBar1.Font.Style := [fsBold];
  end
  else begin
    actionFlag.Enabled := False;
    ActionToolBar1.Font.Style := [];
  end;
end;

procedure TfrmMain.OpenPatient(CCOWPatientID: string = '';
  CCOWPatientIDType: string = 'SS';
  OpenLimitedAccess: Boolean = False);
//var
//  scanIEN,
//    CheckCaption: string;
//  CheckState: Boolean;
begin
  actionFileClosePatient.execute;
  if CloseFrm = False then
    Exit;
  lstUnitDose.Enabled := False;

  if (OpenReadOnly = True) or (CCOWPatientID <> '') then begin
    OpenReadOnly := False;
    ReadOnly := True;
    pnlScannerInput.Enabled := False;
    //msf disable
{$IFDEF MSF_ON}
    btnEnableScanner.Enabled := False;
{$ENDIF}
    SetFormCaption;
  end
    //msf disable
{$IFDEF MSF_ON}
  else
    btnEnableScanner.Enabled := True;
{$ENDIF}

  if (OpenLimitedAccess = True) then begin
    LimitedAccess := True;
    ReadOnly := True;
    SetFormCaption;
    mnuTakeActionOnBag.Enabled := False; {JK 5/29/2008}
    mnuDueListTakeActionOnBag.Enabled := False; {JK 5/29/2008}
  end;

  if CCOWPatientID <> '' then begin
    ReadOnly := true;
    BCMA_Patient.ScanPatient(CCOWPatientID, 0, CCOWPatientIDType);
    if BCMA_Patient.IEN = '' then
      if BCMA_User.IsReadOnly = False then
        ReadOnly := false;
  end
  else if ReadOnly = true then
    with TfrmPtSelect.create(application) do try
      ShowModal;
      if (BCMA_Patient.IEN = '') then begin
        if (BCMA_User.IsReadOnly = False) then
          ReadOnly := false;
        LimitedAccess := False;
      end
    finally
      free;
    end
  else begin
    //msf disable
{$IFDEF MSF_ON}
    IsUnableToScan := False;
    ScanPatientWristBand;
{$ENDIF}

    //uncommented following original code
{$IFNDEF MSF_ON}
    scanIEN := inputPrompt('Patient Lookup', 'Scan Patient Wristband', '', 50,
      true, false, CheckState, CheckCaption, false);
    if scanIEN <> '' then
      BCMA_Patient.ScanPatient(scanIEN, 0);
{$ENDIF}
    //end of original code

  end;

  if BCMA_Patient.IEN <> '' then
    with BCMA_Patient do begin
      SetVDLMessage('Loading Active Orders');
      lstUnitDose.visible := False;
      sgUnitDose.Hide;
      pnlMainForm.Visible := True;
      cbxContinuous.checked := True;
      cbxPRN.checked := BCMA_SiteParameters.PRNChecked;
      cbxOneTime.checked := True;
      cbxOnCall.checked := True;

      with rePatientDemographics do begin
        clear;
        lines.add(BCMA_Patient.Name + '  (' + Sex + ')');
        //        lines.add('SSN = ' + SSN);
        lines.add(BCMA_SiteParameters.PatientIDLabel + ' = ' +
          BCMA_Patient.SSN);
        // rpk 8/7/2009
      //        lines.add('DOB = ' + DOB + ' (' + intToStr(Age) + ')');
        lines.add('DOB = ' + DOB + ' (' + Age + ')'); // rpk 8/6/2009
        lines.add('Height = ' + BCMA_Patient.Height + ', Weight = ' +
          BCMA_Patient.Weight);
        lines.add('Location = ' + Ward + ' ' + RmBed);
      end;

      stAllergies.caption := 'ALLERGIES: ' + Allergies + '    ' + 'ADRs: ' +
        ADRs;
      //      pnlMainForm.Repaint;
            // include CoolBar, etc. repaint after patient selection
      frmMain.Repaint; // rpk 9/18/2009

      with pgctrlVirtualDueList do begin
        ActivePageIndex := integer(lstCurrentTab);
        pgctrlVirtualDueListChange(Self);
      end;

      if (Means = True) and (CCOWPatientIDType = 'SS') then begin
        DefMessageDlg(Means1 + #13 + #13 + Means2, mtInformation,
          [mbOk], 0);
      end;

      if not ReadOnly then
        ProcessGivenExpiredPatches;
      ScannerActivate;
    end;

  lstUnitDose.Enabled := True;
  isUnableToScan := False; {JK 7/27/2008, CQ #148}
end; // OpenPatient

procedure TfrmMain.ActionJoinSetNewContextUpdate(Sender: TObject);
begin
  if (VACCOW.LinkStatus <> lsNoCCOW) and (BCMA_Patient.IEN <> '') then
    ActionJoinSetNewContext.Enabled := (VACCOW.LinkStatus = lsBroken)
  else
    ActionJoinSetNewContext.Enabled := False;
end;

procedure TfrmMain.actionFileExitUpdate(Sender: TObject);
begin
  //Category menus don't have any events, so for the Rejoin Patient Link menu
  //which is actually a category.  The line of code below will hide the category
  //if there is no CCOW.  Since we always have an 'Exit', and this code will
  //always get called when they click file, we'll put it here.
  if (VACCOW.LinkStatus <> lsNoCCOW) then begin
    mnuFileN3.Visible := True;
    mnuFileRejoinPatientLink.Visible := True;
    //    ActionManager.ActionBars.ActionBars[7].Items[0].Items[6].Visible := True;
    //    ActionManager.ActionBars.ActionBars[7].Items[0].Items[7].Visible := True;
  end
  else begin
    mnuFileN3.Visible := False;
    mnuFileRejoinPatientLink.Visible := False;

    //    ActionManager.ActionBars.ActionBars[7].Items[0].Items[6].Visible := False;
    //    ActionManager.ActionBars.ActionBars[7].Items[0].Items[7].Visible := False;
  end;
end;

procedure TfrmMain.ActionJoinUseExistingContextExecute(Sender: TObject);
begin
  if not VACCOW.ResumeContext(rmGet) then
    DefMessageDlg(CCOWErrorMessage, mtError, [mbOk], 0);
end;

procedure TfrmMain.ActionJoinUseExistingContextUpdate(Sender: TObject);
begin
  if (VACCOW.LinkStatus <> lsNoCCOW) and ((VACCOW.LinkStatus = lsBroken) or
    (BCMA_Patient.IEN = '')) then
    ActionJoinUseExistingContext.Enabled := True
  else
    ActionJoinUseExistingContext.Enabled := False;
end;

procedure TfrmMain.edtScannerInputKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {Added a hidden IEN Drug display feature for the VDL. A user will get to it by
   pressing Ctrl-Shift and the = (equals) key together while focus is on the
   edtScanner field. JK 6-5-2008}
{$IFDEF TEST_ON}
  // only in test mode
  if (ssCtrl in Shift) and (ssShift in Shift) and (Key = 187) then try
    DisplayIEN;
  except
    on E: Exception do
      {do nothing};
  end;
{$ENDIF}

  {disable cut and paste}
  if key = VK_Insert then
    key := 0;
end;

procedure TfrmMain.edtScannerInputContextPopup(Sender: TObject;
  MousePos: TPoint; var Handled: Boolean);
begin
  {disable right click menu, specifcally don't allow paste}
//  Handled := True;
end;

procedure TfrmMain.actionMedTabCoverSheetExecute(Sender: TObject);
begin
  with pgctrlVirtualDueList do
    if ActivePage <> tbshtCoverSheet then begin
      ActivePage := tbshtCoverSheet;
      pgctrlVirtualDueListChange(Self);
    end;
end;

procedure TfrmMain.actionMedTabCoverSheetUpdate(Sender: TObject);
begin
  actionMedTabCoverSheet.Enabled := pnlMainForm.Visible;
end;

procedure TfrmMain.actionDueListTakeActionOnBagExecute(Sender: TObject);
var
  x: char;
begin
  if CurrentBagID = nil then // rpk 10/25/2010
    Exit;
  WorkflowType := WF_TakeActionOnBag;
  ScannedInput := TBCMA_IVBags(CurrentBagID).UniqueID;
  edtScannerInput.Clear;
  edtScannerInput.Text := TBCMA_IVBags(CurrentBagID).UniqueID;
  x := chr(VK_RETURN);
  edtScannerInputKeyPress(edtScannerInput, x);
end;

procedure TfrmMain.actionDueListTakeActionOnBagUpdate(Sender: TObject);
var
  TempNode: TTreeNode;
  aMedOrder: TBCMA_MedOrder;
  ScanStat: TScanStatus;
begin
  if ReadOnly or not pnlMainForm.Visible or (BCMA_Patient.MedOrders.count = 0)
    or
    (VisibleMedList.Count = 0) then {// rpk 9/11/2009} begin
    actionDueListTakeActionOnBag.Enabled := False;
    exit;
  end;
  case lstCurrentTab of
    ctUD, ctPB:
      actionDueListTakeActionOnBag.Visible := False;
    ctIV: begin
        actionDueListTakeActionOnBag.Visible := True;
        TempNode := fraIV1.tvwIVHistory.Selected;
        with tempNode do
          if (tempNode <> nil) and (level = 1) then begin
            aMedOrder := TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]);
            ScanStat := getScanStatusID(TBCMA_IVBags(tempNode.Data).ScanStatus);
            with aMedOrder do
              if aMedOrder = nil then begin
                DefMessageDlg(ErrIVAction, mtError, [mbOk], 0);
                actionDueListTakeActionOnBag.Enabled := False;
                exit;
              end
              else if (ScanStat = ssInfusing) or (ScanStat = ssStopped) then
                //end else if ((ScanStat = ssInfusing) or (ScanStat = ssStopped) or
                //  (ScanStat = ssHeld) or (ScanStat = ssRefused)) then
                actionDueListTakeActionOnBag.Enabled := True
              else
                actionDueListTakeActionOnBag.Enabled := False;
          end
          else
            actionDueListTakeActionOnBag.Enabled := False;
      end;
  else
    actionDueListTakeActionOnBag.Enabled := false;
  end;
end;

procedure TfrmMain.ActionCSExpandGrp0Execute(Sender: TObject);
begin
  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      GroupExpandCollapse(0);
end;

procedure TfrmMain.ActionCSExpandGrp1Execute(Sender: TObject);
begin
  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      GroupExpandCollapse(1);
end;

procedure TfrmMain.ActionCSExpandGrp2Execute(Sender: TObject);
begin
  if lstCurrentTab = ctCS then
    with TBaseInterfacedObject(BCMA_CoverSheet) as I_Anything do
      GroupExpandCollapse(2);
end;

procedure TfrmMain.actionReportUnknownActionsExecute(Sender: TObject);
begin
  UnknownActionsReport(BCMA_Patient.IEN);
end;

procedure TfrmMain.actionReportUnknownActionsUpdate(Sender: TObject);
begin
  actionReportUnknownActions.Visible := BCMA_User.IsManager;
end;

procedure TfrmMain.btnEnableScannerClick(Sender: TObject);
begin
  ScannerActivate;
end;

procedure TfrmMain.PopupActionBarPopup(Sender: TObject);
begin
  if ScreenReaderSystemActive then begin
    if sgUnitDose.Enabled and sgUnitDose.Visible and sgUnitDose.CanFocus then begin
      sgUnitDose.SetFocus;
    end
  end
  else begin
    if lstUnitDose.Enabled and lstUnitDose.Visible and lstUnitDose.CanFocus then begin
      lstUnitDose.SetFocus;
    end;
  end;
end;

//-function TfrmMain.ProcessPRNS(aMedOrder: TBCMA_MedOrder): Boolean;

procedure TfrmMain.ProcessPRNS(aMedOrder: TBCMA_MedOrder; var isPRNCancelled:
  Boolean; var PRNVitalsInfo, PRNPainInfo: string);
var
  //--  PainInfo,
  zNow: string;
begin
  zNow := 'N';

  with TfrmPRNMedLog.Create(Application) do try
    MedOrder := aMedOrder;

    PRNVitalsInfo := '';
    PRNPainInfo := '';

    //-    Result := (ShowModal = mrOK);
    isPRNCancelled := not (ShowModal = mrOK);

    aMedOrder.UserComments := '';

    //-    if Result = True then begin
    if not isPRNCancelled then begin
      MedOrder.ReasonGivenPRN := cmtReasonGivenPRN + '^' +
        IntToStr(Integer(cbxReason.Items.Objects[cbxReason.ItemIndex]));

      if cbxPainScore.ItemIndex <> -1 then begin
        //-        Result := SendVitals(IntToStr(Integer(cbxPainScore.Items.Objects[cbxPainScore.ItemIndex])));

        //-        if Result then begin
        PRNPainInfo := 'Pain Score of ' +
          IntToStr(Integer(cbxPainScore.Items.Objects[cbxPainScore.ItemIndex]))
          +
          ' entered into Vitals via BCMA at ' +
          FormatFMDateTime('MM/DD/YYYY@HH:NN',
          StrToFloat(ValidMDateTime(zNow)));
        //-          aMedOrder.AdditionalComments.Add(PainInfo);
        //-        end;

        PRNVitalsInfo :=
          IntToStr(Integer(cbxPainScore.Items.Objects[cbxPainScore.ItemIndex]));
      end;
    end;
  finally
    free;
  end
end;

procedure TfrmMain.fraIV1tvwIVHistoryDblClick(Sender: TObject);
begin
  ScannerActivate;
end;

procedure TfrmMain.edtScannerInputKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if edtScannerInput.Text <> '' then begin
    if (KeyBoardTimer = nil) or (KeyedBarCode = False) then
      StartKeyboardTimer;
  end
  else begin
    StopKeyboardTimer;
    KeyedBarCode := False;
  end;
end;

procedure TfrmMain.ActionReportUnableToScanDetailedExecute(
  Sender: TObject);
begin
  UnableToScanDetailedReport;
end;

procedure TfrmMain.ActionReportUnableToScanSummaryExecute(Sender: TObject);
begin
  UnableToScanSummaryReport;
end;

procedure TfrmMain.actionFileOpenLimitedAccessUpdate(Sender: TObject);
begin
  //msf disable
{$IFNDEF MSF_ON}
  exit;
{$ENDIF}
  actionFileOpenLimitedAccess.Enabled := not BCMA_User.IsReadOnly;
end;

procedure TfrmMain.actionFileOpenLimitedAccessExecute(Sender: TObject);
begin
  OpenReadOnly := True;
  OpenPatient('', '', True);
  SetFormCaption;
end;

procedure TfrmMain.actionFileCreateWardStockUpdate(Sender: TObject);
begin
  //msf disable
{$IFNDEF MSF_ON}
  exit;
{$ENDIF}

  if lstCurrentTab <> ctUD then
    actionFileCreateWardStock.Visible := True
  else
    actionFileCreateWardStock.Visible := false;
  actionFileCreateWardStock.enabled := false;
  with lstUnitDose do
    if not ReadOnly and (visibleMedList.Count <> 0) and ((SelCount = 1) or
      ((MultiSelect = False) and (ItemIndex > -1))) then
      with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
        if (OrderTypeID = otIV) and (OrderStatus <> 'D') and (OrderStatus <> 'E')
          and
          (OrderStatus <> 'H') then begin
          if (lstCurrentTab = ctPB) then begin
            if ScanStatus <> 'G' then
              actionFileCreateWardStock.Enabled := True
          end
          else if lstCurrentTab = ctIV then
            actionFileCreateWardStock.Enabled := True;
        end;
      end;

end;

procedure TfrmMain.actionFileCreateWardStockExecute(Sender: TObject);
var
  toBeWardStock: Pointer;
  CurFlowUID,
    ScannedDrugIEN: string;
  KeyStroke: Char;
  UnableToScanString: string;
  DispensedDrug: TBCMA_DispensedDrug;
  ReasonTxt, CommentTxt: string;
  ReturnVal: Boolean;

  InfusingBags: Boolean;
  CheckInfusingBagsBailOut: Boolean;
  OkToCompleteHangingBag: Boolean;
  OkToContinue: Boolean;
  ScanIVOK: Boolean;

  AbbreviatedRoute: string; {JK 8/6/2008}

  PRNVitalsInfo: string; {JK 8/25/2008}
  PRNPainInfo: string; {JK 8/25/2008}
begin
  OkToContinue := True;
  DispensedDrug := nil; // rpk 4/15/2009

  PRNVitalsInfo := '';
  PRNPainInfo := '';

  {JK 7/29/2008 CQ #150}
  if lstCurrentTab = ctIV then begin
    //    OkToCompleteHangingBag := CheckInfusingBags(
    //                         TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderIEN,
    //                         lstUnitDose,
    //                         TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).ScanStatus,
    //                         CurFlowUID,
    //                         InfusingBags,
    //                         CheckInfusingBagsBailOut);
    OkToCompleteHangingBag := CheckInfusingBags(
      TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderIEN,
      //                         lstUnitDose,
      TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).ScanStatus,
      CurFlowUID,
      InfusingBags,
      CheckInfusingBagsBailOut);
    if CheckInfusingBagsBailOut then
      Exit;

    if OkToCompleteHangingBag then begin
      ScanIVOK := ScanIV(
        CurFlowUID,
        atScan,
        CurFlowUID,
        toBeWardStock);
      if not ScanIVOK then begin
        RebuildVirtualDueList(False);
        RebuildIVOrderHistory(True,
          TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber);
        OkToContinue := False;
      end;
    end;
  end;

  if OkToContinue then begin

    UnableToScan := True;
    ToBeWardStock := nil;
    CurFlowUID := '';
    ScannedDrugIEN := '';

    WorkflowType := WF_UAS_CreateWardStock; {JK 5/21/2008}

    UnableToScanExecute(1,
      WorkflowType,
      nil,
      TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]),
      ReasonTxt,
      CommentTxt,
      ReturnVal,
      PRNVitalsInfo,
      PRNPainInfo,
      nil);

    if ReturnVal then begin

      with edtScannerInput do begin
        KeyStroke := #13;
        Text := '';
        OnKeyPress(nil, Keystroke);
      end;

      if frmMain.UAS_LogState = LA_OkToLog then begin

        with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do
          if OrderTypeID = otIV then
            if lstCurrentTab = ctPB then begin
              AbbreviatedRoute := '';
              UnableToScanString := 'ID^' + 'WS';
            end
            else
              UnableToScanString := 'ID^' +
                TBCMA_IVBAGS(BCMA_Patient.IVBags[0]).UniqueID
          else begin
            if DispensedDrug <> nil then
              UnableToScanString := 'DD^' + DispensedDrug.IEN
            else
              UnableToScanString := 'DD^' + OrderedDrugs[0].IEN;
          end;

        {JK 8/25/2008}
        if not SendVitals(PRNVitalsInfo) then
          DefMessageDlg('Error writing Vitals information from PRN screen',
            mtError, [mbOK], 0)
        else if PRNPainInfo <> '' then begin

          TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).AdditionalComments.Add(PRNPainInfo);

          with TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]) do begin
            //if UserComments2 <> '' then begin
            UserComments := UserComments2;
            //LogOrder(mtAddComment, '', nil);
            LogComments(MedLogIEN, AdditionalComments);
            UserComments2 := '';
            UserComments := '';
          end;
        end;

        LogUnableToScan(TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber,
          ReasonTxt,
          CommentTxt,
          UnableToScanString,
          'M');

        frmMain.UAS_LogState := LA_AlreadyLogged;
      end;
    end;
  end; {if OkToContinue}

  RebuildIVOrderHistory(True,
    TBCMA_MedOrder(VisibleMedList[lstUnitDose.ItemIndex]).OrderNumber);

  UnableToScan := False;
end;

procedure TfrmMain.CreateWardStockForFiveRightsOverride(aMedOrder:
  TBCMA_MedOrder;
  var BagName: string; var LogState: TUASLogAction; VitalsInfo, PainInfo:
  string);
var
  toBeWardStock: Pointer;
  CurFlowUID,
    ScannedDrugIEN: string;
  PassFail: Boolean;
begin
  UnableToScan := True;
  ToBeWardStock := nil;
  CurFlowUID := '';
  ScannedDrugIEN := '';

  CreateWardStock2(aMedOrder, ScannedDrugIEN, CurFlowUID, toBeWardStock,
    PassFail, VitalsInfo, PainInfo, LogState);

  if PassFail = False then begin
    BagName := '';
    LogState := LA_CANCELLED;
  end
  else begin
    BagName := CurFlowUID;
    LogState := LA_OkToLog;
  end;

  UnableToScan := False;

end;

procedure TfrmMain.ActionReportUnableToScanDetailedUpdate(Sender: TObject);
begin
  ActionReportUnableToScanDetailed.Visible := BCMA_User.UnableToScanKey;
end;

procedure TfrmMain.ActionReportUnableToScanSummaryUpdate(Sender: TObject);
begin
  ActionReportUnableToScanSummary.Visible := BCMA_User.UnableToScanKey;
end;

procedure TfrmMain.CreateWardStock2(MedOrder: TBCMA_MedOrder; ScannedDrugIEN:
  string; var CurFlowUID:
  string; var toBeWardStock: Pointer; var PassFail: Boolean; VitalsInfo,
  PainInfo: string; LogState: TUASLogAction);
var
  ii, idxOrder: integer;
  aMedOrder: TBCMA_MedOrder;
  ivOrders: TStringList;
  aIVBag: TBCMA_IVBags;
  InfusingBags, ScanIV_Ok_To_Continue: Boolean;
  OkToLog: TUASLogAction;
  CheckInfusingBagsBailOut: Boolean;

  isPRNCancelled: Boolean; {JK 8/15/2008}

  procedure AddSecondComment;
  begin
    with aMedOrder do
      if UserComments2 <> '' then begin
        UserComments := UserComments2;
        LogOrder(mtAddComment, '', nil);
        UserComments2 := '';
        UserComments := '';
      end
  end;

begin
  OkToLog := LogState;
  aIVBag := nil; // rpk 4/15/2009
  CheckInfusingBagsBailOut := False; // rpk 10/26/2010
  InfusingBags := False; // rpk 10/26/2010
  isPRNCancelled := False; // rpk 10/26/2010

  PassFail := False;
  ScanIV_Ok_To_Continue := True;
  //-  aMedOrder := nil;
  aMedOrder := MedOrder;
  if toBeWardStock = nil then
    with TfrmWardStock.create(application) do try
      if UnableToScan or (ScanDrug(ScannedDrugIEN, (lstCurrentTab = ctPB))) then begin
        if (aMedOrder.OrderStatus = 'D') or (aMedOrder.OrderStatus = 'E')
          or
          (aMedOrder.OrderStatus = 'H') then begin
          DefMessageDlg('The order for this bag has either been DC''d, has expired, or is on Provider Hold. '
            +
            #13 + 'A ward stock bag can not be created.', mtError,
            [mbOK], 0);
          exit;
        end;
        with aMedOrder do begin
          WardStock := True;

        end;

        {if IV tab, Find the History for this bag}
        if lstCurrentTab = ctIV then begin

          RebuildIVOrderHistory(False, aMedOrder.OrderNumber);

          //          if CheckInfusingBags('', lstUnitDose, '', CurFlowUID, InfusingBags, CheckInfusingBagsBailOut) then begin
          if CheckInfusingBags('', '', CurFlowUID, InfusingBags,
            CheckInfusingBagsBailOut) then begin

            {JK 5/15/2007 This part occurs when the user cancelled from the dialog in CheckInfusingBags}
            if CurFlowUID = '' then begin
              PassFail := False;
              Exit;
            end;

            IVOrders := GetWardStockOrder(aMedOrder.Additives,
              aMedOrder.Solutions);

            if IVOrders.Count = 0 then
              Exit;

            idxOrder := SelectOrderID(IVOrders, True);

            if idxOrder = -1 then
              Exit;

            if CurFlowUID <> '' then begin
              ScannedInput := CurFlowUID;
              toBeWardStock := VisibleMedList.Items[idxOrder];
              ScanIV_Ok_To_Continue := ScanIV(ScannedInput, atScan, CurFlowUID,
                toBeWardStock); {JK - 5/14/2008}
            end;
          end;
        end;
      end;
    finally
      free;
    end
  else begin
    aMedOrder := TBCMA_MedOrder(toBeWardStock);
  end;

  //if we have an order, log it
  ScannedInput := '';

  if aMedOrder <> nil then begin
    with aMedOrder do begin
      UserComments := '';
      if DisplayInstructions = True then
        if DefMessageDlg(SpecialInstructions, mtInformation, [mbOK, mbCancel], 0)
          <> idOK then begin
          RebuildVirtualDueList(False);
          if lstCurrentTab = ctIV then
            RebuildIVOrderHistory(True, OrderNumber);
          Exit;
        end;

      {JK 11/4/2008 - CodeCR 257 - removed redundant block below because
       it is already set up below in this routine.}
    //-if Status = 1 then begin
    //-  ConfirmOrder(aMedOrder, True, isPRNCancelled, VitalsInfo, PainInfo);
    //-  if isPRNCancelled then
    //-    Exit;
    //-end;

      Wardstock := True;

    end;

    try // rpk 9/13/2010
      aIVBag := TBCMA_IVBags.Create(BCMA_Broker);
      if aIVBag <> nil then begin // rpk 9/13/2010
        with aIVBag do begin
          Additives.Assign(aMedOrder.Additives);
          Solutions.Assign(aMedOrder.Solutions);
          CurrentBagID := aIVBag;

          if OkToLog = LA_OkToLog then begin {JK 5/16/2008}

            if aMedOrder.ValidOrder then begin

              if aMedOrder.Status = 1 then begin
                ConfirmOrder(aMedOrder, True, isPRNCancelled, VitalsInfo, PainInfo);
                if isPRNCancelled then
                  Exit;

            {JK 8/25/2008 - Moved Vitals Call from ProcessPRNs to here}
                if PainInfo <> '' then
                  aMedOrder.AdditionalComments.Add(PainInfo);
                if not SendVitals(VitalsInfo) then
                  DefMessageDlg('Error writing Vitals information from PRN screen',
                    mtError, [mbOK], 0);

                if not isPRNCancelled then begin
                  if lstCurrentTab = ctIV then
                    PassFail := aMedOrder.LogOrder(mtMedPass, 'I', aIVBag)
                  else
                    PassFail := aMedOrder.LogOrder(mtMedPass, 'G', aIVBag);
                  AddSecondComment;
                end;

              end
              else begin

                if ScanIV_Ok_To_Continue = True then begin

                  if lstCurrentTab = ctIV then
                    PassFail := aMedOrder.LogOrder(mtMedPass, 'I', aIVBag)
                  else
                    PassFail := aMedOrder.LogOrder(mtMedPass, 'G', aIVBag)
                end;
              end;

          {JK 9/17/2008}
              if aMedOrder.Status = 1 then begin
                if isPRNCancelled or (PassFail = False) then
                  DefMessageDlg('Order Administration Cancelled!', mtWarning,
                    [mbOK], 0); {JK 5/12/2008}
              end
              else begin
                if PassFail = False then
                  DefMessageDlg('Order Administration Cancelled!', mtWarning,
                    [mbOK], 0); {JK 5/12/2008}
              end;

              aIVBag.clear;
              aIVBag.Free;
              aIVBag := nil; // rpk 9/23/2010
            end
            else begin
              if aMedOrder.Status = -2 then begin
                ForceVDLRebuild;
                exit;
              end
              else begin
                DefMessageDlg(aMedOrder.StatusMessage, mtInformation, [mbOK], 0);
                exit;
              end;
            end;
          end; {if OkToLog = LA_OkToLog }
        end; // with aIVBag
      end; // if aIVBag <> nil

      // DEBUG
//      if aMedOrder.OrderNumber = '' then
//        DefMessageDlg('CreateWardStock2: aMedOrder.OrderNumber is empty', mtError, [mbOk], 0);

      for ii := 0 to VisibleMedList.count - 1 do
        with TBCMA_MedOrder(VisibleMedList[ii]) do
          if OrderNumber = aMedOrder.OrderNumber then begin
            lstUnitDose.ItemIndex := ii;
            break;
          end;

      RebuildVirtualDueList(False);
      if (lstCurrentTab = ctIV) and
        (aMedOrder.OrderNumber <> '') then // rpk 9/23/2010
        RebuildIVOrderHistory(True, aMedOrder.OrderNumber);

      try
        if aIVBag = nil then // rpk 9/23/2010
          CurFlowUID := '' // rpk 9/23/2010
        else // rpk 9/23/2010
          CurFlowUID := aIVBag.UniqueID;
      except
        CurFlowUID := '' {JK 5/13/2008}
      end;
    finally
      if aIVBag <> nil then // rpk 9/23/2010
        aIVBag.Free;
    end; // aIVBag create
  end; // if MedOrder <> nil

end; // CreateWardStock2

{JK Spring 2008 - utility method to support IEN lookup for testing purposes.
 BE SURE THIS METHOD IS NOT AVAILABLE IN PRODUCTION RELEASES!}

procedure TfrmMain.DisplayIEN;
var
  ii, y: integer;
  msg, tempUID: string;
  Found: Boolean;
begin
  try
    Found := false;
    msg := '';

    if ScreenReaderSystemActive then begin
      with sgUnitDose do
        case lstCurrentTab of
          ctUD, ctPB:
            with TBCMA_MedOrder(VisibleMedList.items[sgUnitDose.Row - 1]) do
              case OrderTypeID of
                otUnitDose: begin
                    if OrderedDrugNames.count > 0 then
                      for ii := 0 to OrderedDrugNames.count - 1 do
                        msg := msg + OrderedDrugNames[ii] + #9 + 'Drug IEN=' +
                          OrderedDrugIENs[ii] + #13
                    else
                      msg := 'No Dispensed Drugs Found for this Order!';
                    DefMessageDlg(msg, mtInformation, [mbOk], 0);
                  end;
                otIV: begin
                    if UniqueIDs.count > 0 then begin
                      //                  msg := 'The following bags are available (note: you are in TEST mode. The nurse will never be able to get access to this feature):' + #13 + #13;
                      msg := 'The following bags are available for ' +
                        ActiveMedication +
                        ' (note: you are in TEST mode. The nurse will never be able to get access to this feature):' + #13
                        + #13;
                      for ii := 0 to UniqueIDs.count - 1 do begin
                        tempUID := piece(UniqueIDs[ii], '^', 1);
                        with BCMA_Patient do
                          for y := 0 to MedOrders.count - 1 do
                            with TBCMA_MedOrder(MedOrders[y]) do
                              if UniqueID = tempUID then begin
                                Found := True;
                                break;
                              end;
                        if Found <> True then
                          msg := msg + tempUID + #13;
                        Found := False;
                      end;
                    end
                    else
                      msg := 'No bags are available for this order!';
                    DefMessageDlg(msg, mtInformation, [mbOk], 0);
                  end; // otIV
              end; // case OrderTypeID
        end; // case lstCurrentTab
    end
    else begin
      with lstUnitDose do
        case lstCurrentTab of
          ctUD, ctPB:
            with TBCMA_MedOrder(VisibleMedList.items[lstUnitDose.ItemIndex]) do
              case OrderTypeID of
                otUnitDose: begin
                    if OrderedDrugNames.count > 0 then
                      for ii := 0 to OrderedDrugNames.count - 1 do
                        msg := msg + OrderedDrugNames[ii] + #9 + 'Drug IEN=' +
                          OrderedDrugIENs[ii] + #13
                    else
                      msg := 'No Dispensed Drugs Found for this Order!';
                    DefMessageDlg(msg, mtInformation, [mbOk], 0);
                  end;
                otIV: begin
                    if UniqueIDs.count > 0 then begin
                      //                  msg := 'The following bags are available (note: you are in TEST mode. The nurse will never be able to get access to this feature):' + #13 + #13;
                      msg := 'The following bags are available for ' +
                        ActiveMedication +
                        ' (note: you are in TEST mode. The nurse will never be able to get access to this feature):' + #13
                        + #13;
                      for ii := 0 to UniqueIDs.count - 1 do begin
                        tempUID := piece(UniqueIDs[ii], '^', 1);
                        with BCMA_Patient do
                          for y := 0 to MedOrders.count - 1 do
                            with TBCMA_MedOrder(MedOrders[y]) do
                              if UniqueID = tempUID then begin
                                Found := True;
                                break;
                              end;
                        if Found <> True then
                          msg := msg + tempUID + #13;
                        Found := False;
                      end;
                    end
                    else
                      msg := 'No bags are available for this order!';
                    DefMessageDlg(msg, mtInformation, [mbOk], 0);
                  end; // otIV
              end; // case OrderTypeID
        end; // case lstCurrentTab
    end;

  except
    on E: Exception do
      ;
  end;

end;

{This is the major logic unit for IV Bag processing}

function TfrmMain.ScanIV(ScannedDrugIEN: string; aIVActionType: TIVActionTypes;
  var CurFlowUID: string; var toBeWardStock: Pointer): Boolean;
var
  //  ii,
  //    jj,
  idxorder,
    idxbag: Integer;
  aMedOrder: TBCMA_MedOrder;
  aIVBag: TBCMA_IVBags;
  //  OrderNumber: string;
  InfusingBags: Boolean;
  CheckInfusingBagsBailOut: Boolean;
  CheckInfusingBagsOkToContinue: Boolean;
  frmScanIV: TfrmScanIV;

  procedure BailOut;
  begin
    toBeWardStock := nil;
    CurFlowUID := '';
    Result := False;
    with lstUnitDose do
      if ItemIndex > -1 then
        RebuildIVOrderHistory(True,
          TBCMA_MedOrder(VisibleMedList.items[ItemIndex]).OrderNumber);
  end;

begin
  idxorder := -1;
  idxbag := -1;
  aIVBag := nil;
  aMedOrder := nil;
  {WardStock Unique ID}
  Result := True; // Was true in MSF version

  //is this a ward stock Unique ID?
  if Pos('WS', ScannedDrugIEN) > 0 then begin
    if lstUnitDose.ItemIndex = -1 then begin
      DefMessageDlg('You must select the order containing the wards stock bag first!', mtError, [mbOK], 0);
      BailOut;
      Exit;
    end;

    //get the order number from the history
    aIVBag := GetIVBagFromHistory(ScannedDrugIEN);
    if aIVBag = nil then begin
      DefMessageDlg('The Ward Stock Bag does not exist in the highlighted order!', mtError, [mbOK], 0);
      BailOut;
      Exit;

    end
    else begin

      //find the order on the VDL that contains this order number
      aMedOrder := GetOrderViaOrderNumber(aIVBag.OrderNumber, lstUnitDose);
      if aMedOrder = nil then begin
        BailOut;
        Exit;
      end;

      {JK 8/2/2008}
      if aIVBag.ScanStatus = 'C' then begin
        DefMessageDlg('Invalid Medication Lookup.' + #13#10#13#10 +
          'DO NOT GIVE!', mtError, [mbOK], 0);
        Result := False;
        RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
        Exit;
      end;

    end;

  end
  else
    {Find the order on the VDL that contains this Unique ID}
    aMedOrder := GetOrderFromUniqueID(ScannedDrugIEN, lstUnitDose);

  {if aMedOrder is still nil, then the user may be scanning additives/solutions}
  if aMedOrder = nil then
    CreateWardStock(ScannedDrugIEN, CurFlowUID, toBeWardStock)

    {we found the order, we scanned a Unique ID, continue on...}
  else begin
    with aMedorder do begin
      WardStock := false;
      RebuildIVOrderHistory(False, OrderNumber);

      {Find the History for this bag, iv we haven't already in the Ward Stock Code above}
      aIVBag := GetIVBagFromHistory(ScannedDrugIEN);
      if aIVBag = nil then begin
        DefMessageDlg('An order was found for this bag, but the bag history'
          + #13 + 'is missing, please report this error!', mtError, [mbOk], 0);
        BailOut;
        Exit;
      end;

      if (aIVActionType <> atHeld) and (aIVActionType <> atRefused) then begin

        if aIVBag.ScanStatus = 'C' then
          CheckInfusingBagsOkToContinue := False
        else
          CheckInfusingBagsOkToContinue := True;

        if CheckInfusingBagsOkToContinue then

          CheckInfusingBagsOkToContinue :=
            //            CheckInfusingBags(ScannedDrugIEN, lstUnitDose, aIVBag.ScanStatus,
//                              CurFlowUID, InfusingBags, CheckInfusingBagsBailOut);
          CheckInfusingBags(ScannedDrugIEN, aIVBag.ScanStatus,
            CurFlowUID, InfusingBags, CheckInfusingBagsBailOut);

        if CheckInfusingBagsOkToContinue then begin

          if CurFlowUID <> '' then
            Exit
          else begin
            BailOut;
            Exit;
          end;
        end;

        {JK - 7/26/2008 CQ #150}
        if CheckInfusingBagsBailOut then begin
          if aMedOrder.ScanStatus = 'C' then
            DefMessageDlg('Invalid Medication Lookup.' + #13#10#13#10 +
              'DO NOT GIVE!', mtError, [mbOK], 0);
          UAS_LogState := LA_CANCELLED;
          RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
          Exit;
        end;
      end;

      CurrentBagId := aIVBag;
      if aIVBag.ScanStatus = 'C' then
        DefMessageDlg('IV bag already completed!', mtError, [mbOK], 0)
      else if (OrderStatus = 'H') and
        ((aIVBag.ScanStatus = 'R') or
        (aIVBag.ScanStatus = 'A') and
        (aIVActionType <> atHeld)) then begin
        DefMessageDlg('Order is on Provider Hold!' + #13#13 +
          'DO NOT GIVE!', mtError, [mbOK], 0);
          // ???:  Result defaults to True here.
          // If order is on provider hold, take no further action on this order
          // including starting a next bag?  rpk 9/29/2010
          // proposed change: return false on provider hold
          //        Result := False;  // ??? rpk 9/29/2010
      end
      else begin
        if ValidOrder then begin
          with aMedorder do
            if DisplayInstructions = True then
              if DefMessageDlg(SpecialInstructions, mtInformation, [mbOK,
                mbCancel], 0) <> idOK then begin
                Result := False;
                RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
                Exit;
              end;
          //          with TfrmScanIV.create(application) do
          frmScanIV := TfrmScanIV.create(application); // rpk 8/9/2010
          with frmScanIV do {// rpk 8/9/2010} try
            begin
              ActionOverride := -1;
              if CurFlowUID <> '' then
                ActionOverride := 2;
              CurFlowUID := '';
              IVBag := aIVBag;
              MedOrder := aMedOrder;
              IVActionType := aIVActionType;
              MultInfusingBags := InfusingBags;
              if ShowModal = mrOK then begin
                Self.Repaint;
                case aIVActionType of
                  atScan:
                    with MedOrder do begin
                      UserComments := memComment.Text;
                      InjectionSite := cbxInjectionSite.Text;
                    end;
                  atHeld, atRefused: begin
                      UserComments :=
                        cbxInjectionSite.Items[cbxInjectionSite.ItemIndex];
                    end;
                end;
                //it was decided to validate the IV order a second time, in case the user sat on
                //this screen for an extended period of time.
//                if ValidOrder then
//                  MedOrder.LogOrder(mtMedPass,
//                    ScanStatusCodes[TScanStatus(cbxAction.Items.Objects[cbxAction.ItemIndex])], IVBag)
                if ValidOrder then begin
                  if frmScanIV.cbxAction.ItemIndex > -1 then // rpk 8/9/2010
//                    MedOrder.LogOrder(mtMedPass,
//                      ScanStatusCodes[TScanStatus(cbxAction.Items.Objects[cbxAction.ItemIndex])],
//                      IVBag);
                    Result := MedOrder.LogOrder(mtMedPass, // rpk 9/24/2010
                      ScanStatusCodes[TScanStatus(cbxAction.Items.Objects[cbxAction.ItemIndex])],
                      IVBag);
                end
                else if status = -2 then begin
                  Result := False;
                  ForceVDLRebuild;
                  Exit
                end
                else if status = -1 then begin
                  DefMessageDlg(StatusMessage, mtError, [mbOK], 0);
                  Result := False;
                  RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
                  Exit;
                end
                else if status = -10 then begin
                  Result := False;
                  Exit;
                end;

              end
              else begin
                Result := False;
                if aMedOrder <> nil then
                  RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
              end;
            end;

          finally
            Free;
          end;

        end
        else if Status = -2 then begin
          Result := False;
          ForceVDLRebuild
        end
        else if Status = -1 then begin
          DefMessageDlg(StatusMessage, mtError, [mbOK], 0);
          Result := False;
          RebuildIVOrderHistory(True, aMedOrder.OrderNumber);
          Exit;
        end
        else if Status = -10 then begin
          Result := False;
          Exit;
        end;
      end;
      //end;
    end;

//    if (CurrentOrderNumber <> '') and (ToBeWardStock = nil) and (CurFlowUID = '')
//      and Result then
    if (aMedOrder <> nil) and (CurrentOrderNumber <> '') and // rpk 9/23/2010
      (ToBeWardStock = nil) and (CurFlowUID = '') and Result then
      RebuildIVOrderHistory(True, aMedOrder.OrderNumber);

  end;

end;

initialization
  SpecifyFormIsNotADialog(TfrmMain);

end.

